/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.ByteArrayInputStream;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMList;
import net.rim.blackberry.api.pdap.BlackBerryPIM;
//import net.rim.blackberry.api.pdap.BlackBerryPIMList;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.helper.EventHelper;
import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.blackberry.helper.HelperFactory;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.source.BBPIMSyncListener;
import com.funambol.blackberry.source.BBPIMSyncSource;
import com.funambol.blackberry.source.BBPIMSyncSourceWithUpdateFix;
import com.funambol.blackberry.source.ContactAdditionalAttributes;
import com.funambol.blackberry.source.PhotoTracker;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;
import com.funambol.storage.StringKeyValueFileStore;
import com.funambol.syncml.client.CacheTracker;
import com.funambol.syncml.client.FileSyncSource;

/**
 * This class is responsible for handling all the sources at the application
 * level. A source at the application level is a type of data for which a
 * SyncSource exits, but also a sync button on the screen and configuration
 * parameters.
 * This class creates and registers the sources. The list of sources is handled
 * and allows other parts of the application to be completely parametric wrt to
 * this value.
 * For new sources to be added to the application it is necessary to register
 * them in this manager. The only other part of the application that must be
 * changed when new sources are added is the Customization which holds default
 * parameters for each source.
 */
public class AppSyncSourceManager {

    public static final int     SYNC_FLAG_CONTACTS        = 1;
    public static final int     SYNC_FLAG_CALENDAR        = 2;
    public static final int     SYNC_FLAG_TASKS           = 4;
    public static final int     SYNC_FLAG_NOTES           = 8;
    public static final int     SYNC_FLAG_MAIL            = 16;
    public static final int     SYNC_FLAG_PHOTOS          = 32;
    public static final int     SYNC_FLAG_ALL             = 255;

    public static final long    CONTACT_STORAGE_KEY       = 0xbc565a75eda5330L;
    public static final long    CALENDAR_STORAGE_KEY      = 0xbc565a75eda5340L;
    public static final long    NOTE_STOREAGE_KEY         = 0xbc565a75eda5350L;
    public static final long    TASK_STORAGE_KEY          = 0xbc565a75eda5360L;
    //public static final long    LUID_UID_STORAGE_KEY      = 0xbc565a75eda5370L;
    public static final long    PHOTO_STORAGE_KEY         = 0xbc565a75eda5380L;

    public static final long    CONTACTS_ADDITS_STORAGE_KEY = 0xbc565a75eda5390L;
    public static final long    CALENDAR_ADDITS_STORAGE_KEY = -1;
    public static final long    NOTE_ADDITS_STORAGE_KEY     = -1;
    public static final long    TASKS_ADDITS_STORAGE_KEY    = -1;

    // Configuration keys for the source configs
    public static final String  CONFIG_KEY_SYNC_CONTACT   = "SYNC_CONTACT_CONFIG";
    public static final String  CONFIG_KEY_SYNC_CALENDAR  = "SYNC_CAL_CONFIG";
    public static final String  CONFIG_KEY_SYNC_TASK      = "SYNC_TASK_CONFIG";
    public static final String  CONFIG_KEY_SYNC_NOTE      = "SYNC_NOTE_CONFIG";
    public static final String  CONFIG_KEY_SYNC_PHOTO     = "SYNC_PHOTO_CONFIG";

    // Types
    public static final String  CONTACT_TYPE_VCARD        = "text/x-vcard";
    public static final String  CONTACT_TYPE_SIF          = "text/x-s4j-sifc";
    public static final String  CALENDAR_TYPE_SIF         = "text/x-s4j-sife";
    public static final String  NOTE_TYPE_SIF             = "text/x-s4j-sifn";
    public static final String  TASK_TYPE_SIF             = "text/x-s4j-sift";
    public static final String  BRIEFCASE_TYPE            = "application/*";

    // Source config settings
    //public static final String  CONTACT_TYPE              = CONTACT_TYPE_VCARD;
    private static final String  CALENDAR_TYPE             = CALENDAR_TYPE_SIF;
    private static final String  NOTE_TYPE                 = NOTE_TYPE_SIF;
    private static final String  TASK_TYPE                 = TASK_TYPE_SIF;
    private static final String  PHOTO_TYPE                = BRIEFCASE_TYPE;
    //
    private static final String  CONTACT                   = "contact";
    private static final String  CALENDAR                  = "calendar";
    private static final String  NOTE                      = "note";
    private static final String  TASK                      = "task";
    private static final String  PHOTO                     = "photo";


    private static AppSyncSourceManager instance = null;
    private Hashtable idMap       = null;
    private Vector    sourcesList = null;
    private static Controller controller;

    private AppSyncSourceManager() {
        idMap       = new Hashtable();
        sourcesList = new Vector();
    }

    public static AppSyncSourceManager getInstance() {
        if (instance == null) {
            instance = new AppSyncSourceManager();
        }
        return instance;
    }

    public void registerSource(AppSyncSource source) {
        sourcesList.addElement(source);
        idMap.put(new Integer(source.getId()), source);
    }

    public int numberOfRegisteredSources() {
        return idMap.size();
    }

    public int numberOfEnabledSources() {
        int count = 0;
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled()) {
                count++;
            }
        }
        return count;
    }

    public int numberOfEnabledAndWorkingSources() {
        int count = 0;
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled() && appSource.isWorking()) {
                count++;
            }
        }
        return count;
    }


    public void setUpContactsSource() throws Exception {
        Log.info("[AppSyncSourceManager.setUpContactSource]");

        // Build the appropriate AppSyncSource and register it, even if the
        // corresponding SyncSource has not been built yet
        String name = StaticDataHelper.getLanguage("type_contacts");
        AppSyncSource contactSource = new AppSyncSource(name);
        contactSource.setId(SYNC_FLAG_CONTACTS);
        contactSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-contact"));
        contactSource.setDisabledLabel(StaticDataHelper.getLanguage("message_contacts-disabled"));
        contactSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_CONTACTS));
        contactSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_CONTACTS));
        contactSource.setUiSourceIndex(Customization.getInstance().getPreferredUiSourceItemIndex(SYNC_FLAG_CONTACTS));
        contactSource.setConfigStorageKey(CONFIG_KEY_SYNC_CONTACT);
        registerSource(contactSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_CONTACTS);

        SourceConfig config = initializeSourceConfig(true, SYNC_FLAG_CONTACTS,
                    CONFIG_KEY_SYNC_CONTACT, CONTACT, Customization.getInstance().getContactType(),
                    uri);

        /*
         * Check if upgraded application from an old version that not support
         * email attributes.
         */
        if(Configuration.OLD_CONFIG_VERSION.equals("20060721") ||
           Configuration.OLD_CONFIG_VERSION.equals("1")){
            // Restore the remote uri and the mime-type
            config.setRemoteUri(uri);
            config.setType(Customization.getInstance().getContactType());
            config.setLastAnchor(0);
        }

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.CONTACT_LIST,
                                                       PIM.READ_WRITE);

        boolean updateFix = false;
        String version[] = StaticDataHelper.getOSVersion();
        if (version.length > 1 && version[0].equals("4") && version[1].equals("5")) {
            Log.debug("Version is 4.5, use source with update fix");
            updateFix = true;
        } else { 
            Log.debug("Version is NOT 4.5, use standard source"); 
        }
        SyncSource syncSource;
        if (updateFix) {
            syncSource = new BBPIMSyncSourceWithUpdateFix(config, list,
                               HelperFactory.getContactHelper(list),
                               CONTACT_STORAGE_KEY,
                               CONTACTS_ADDITS_STORAGE_KEY);
        } else {
            syncSource = new BBPIMSyncSource(config, list,
                                    HelperFactory.getContactHelper(list),
                                    CONTACT_STORAGE_KEY,
                                    CONTACTS_ADDITS_STORAGE_KEY);
        }

        syncSource.setConfig(config);
        // Reset the underlying sync source
        contactSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(contactSource);
        contactSource.setAppManager(appManager);
    }

    public void setUpCalendarSource() throws Exception {
        Log.info("[AppSyncSourceManager.setUpCalendarSource]");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_calendar");
        AppSyncSource calendarSource = new AppSyncSource(name);
        calendarSource.setId(SYNC_FLAG_CALENDAR);
        calendarSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-calendar"));
        calendarSource.setDisabledLabel(StaticDataHelper.getLanguage("message_calendar-disabled"));
        calendarSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_CALENDAR));
        calendarSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_CALENDAR));
        calendarSource.setUiSourceIndex(Customization.getInstance().getPreferredUiSourceItemIndex(SYNC_FLAG_CALENDAR));
        calendarSource.setConfigStorageKey(CONFIG_KEY_SYNC_CALENDAR);
        registerSource(calendarSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_CALENDAR);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                                                     CONFIG_KEY_SYNC_CALENDAR,
                                                     CALENDAR, CALENDAR_TYPE,
                                                     uri);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.EVENT_LIST, PIM.READ_WRITE);

        EventMailListener listener = null;
        if (Customization.getInstance().invitesFiltering()) {
            listener = new EventMailListener(StaticDataHelper.getLanguage("message_blocked-invite"));
        }

        PIMItemHelper eventhelper = HelperFactory.getEventHelper(list, listener,
                Configuration.rangePast, Configuration.rangeFuture);

        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list,
                                         eventhelper, CALENDAR_STORAGE_KEY, CALENDAR_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        // Reset the underlying sync source
        calendarSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(calendarSource);
        calendarSource.setAppManager(appManager);
    }

    public void setUpTasksSource() throws Exception {
        Log.info("[AppSyncSourceManager.setUpTasksSource]");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_tasks");

        Log.debug("Setting up tasks source " + name);

        AppSyncSource tasksSource = new AppSyncSource(name);
        tasksSource.setId(SYNC_FLAG_TASKS);
        tasksSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-task"));
        tasksSource.setDisabledLabel(StaticDataHelper.getLanguage("message_tasks-disabled"));
        tasksSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_TASKS));
        tasksSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_TASKS));
        tasksSource.setUiSourceIndex(Customization.getInstance().getPreferredUiSourceItemIndex(SYNC_FLAG_TASKS));
        tasksSource.setConfigStorageKey(CONFIG_KEY_SYNC_TASK);
        registerSource(tasksSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_TASKS);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_TASKS,
                    CONFIG_KEY_SYNC_TASK, TASK, TASK_TYPE,
                    uri);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.TODO_LIST,
                                                                 PIM.READ_WRITE);
        PIMItemHelper helper = HelperFactory.getTaskHelper(list);

        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list, helper, TASK_STORAGE_KEY, TASKS_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        tasksSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(tasksSource);
        tasksSource.setAppManager(appManager);
    }

    public void setUpNotesSource() throws Exception {
        Log.info("Creating Note sync source");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_notes");
        AppSyncSource notesSource = new AppSyncSource(name);
        notesSource.setId(SYNC_FLAG_NOTES);
        notesSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-note"));
        notesSource.setDisabledLabel(StaticDataHelper.getLanguage("message_notes-disabled"));
        notesSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_NOTES));
        notesSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_NOTES));
        notesSource.setUiSourceIndex(Customization.getInstance().getPreferredUiSourceItemIndex(SYNC_FLAG_NOTES));
        notesSource.setConfigStorageKey(CONFIG_KEY_SYNC_NOTE);
        registerSource(notesSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_NOTES);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_NOTES,
                    CONFIG_KEY_SYNC_NOTE, NOTE, NOTE_TYPE, uri);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(BlackBerryPIM.MEMO_LIST, PIM.READ_WRITE);
        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list, HelperFactory.getNoteHelper(list),
                                         NOTE_STOREAGE_KEY, NOTE_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        notesSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(notesSource);
        notesSource.setAppManager(appManager);
    }

    public void setUpPhotosSource() throws Exception {
        Log.info("Creating Photos sync source");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_photos");
        AppSyncSource photosSource = new AppSyncSource(name);
        photosSource.setId(SYNC_FLAG_PHOTOS);
        photosSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-photo"));
        photosSource.setDisabledLabel(StaticDataHelper.getLanguage("message_photos-disabled"));
        photosSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_PHOTOS));
        photosSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_PHOTOS));
        photosSource.setUiSourceIndex(Customization.getInstance().getPreferredUiSourceItemIndex(SYNC_FLAG_PHOTOS));
        photosSource.setConfigStorageKey(CONFIG_KEY_SYNC_PHOTO);
        registerSource(photosSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_PHOTOS);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_PHOTOS,
                    CONFIG_KEY_SYNC_PHOTO, PHOTO, PHOTO_TYPE, uri);

        StringKeyValueFileStore cacheStore = new StringKeyValueFileStore("file:///store/home/user/photoscache.txt");
        String directory = "file:///store/home/user/pictures/";
        CacheTracker tracker = new PhotoTracker(cacheStore, directory);
        SyncSource syncSource;
        syncSource = new FileSyncSource(config, tracker, directory);
        syncSource.setConfig(config);

        photosSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        //ExternalAppManager appManager = new ExternalAppManager(photosSource);
        //photosSource.setAppManager(appManager);
    }

    public AppSyncSource getSource(int id) {
        AppSyncSource source = (AppSyncSource)idMap.get(new Integer(id));
        return source;
    }

    /*
     * Disable a given source. After this method returns the source is
     * disabled for syncs and the setting is propagated into the configuration
     */
    public void disableSource(int id) {
        Configuration.disableSource(id);
    }

    public Enumeration getEnabledSources() {

        Log.trace("[AppSyncSourceManager.getEnabledSources]");
        Vector result = new Vector();
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled()) {
                result.addElement(appSource);
            }
        }
        return result.elements();
    }

    public Enumeration getEnabledAndWorkingSources() {
        Log.trace("[AppSyncSourceManager.getEnabledAndWorkingSources]");
        Vector result = new Vector();
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled() && appSource.isWorking()) {
                result.addElement(appSource);
            }
        }
        return result.elements();
    }

    public Enumeration getRegisteredSources() {
        Log.trace("[AppSyncSourceManager.getRegisteredSources]");
        
        Vector result = new Vector();

        result.setSize(numberOfRegisteredSources());
 
        for(int i=0;i<sourcesList.size();++i) {
            Log.debug("[AppSyncSourceManager.getRegisteredSources]" + i);
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            result.setElementAt(appSource, i);
        }
        Log.debug("[AppSyncSourceManager.getRegisteredSources]Return");
            
        return result.elements();
    }

    /**
     * @param read
     * @param last
     * @param next
     * @param flag
     * @param name
     *            TODO
     * @param type
     *            TODO
     * @param uri
     *            TODO
     */
    private SourceConfig initializeSourceConfig(boolean read, int flag, String configKey,
                                                String name, String type, String uri) {

        long last = 0;
        long next = 0;
        boolean restore = false;
        SourceConfig config;

        Log.info("Initialize SourceConfig [" + name + "]");
        config = new SourceConfig(name, type, uri);

        // Handle special case
        if (CONTACT.equals(name) &&
            CONTACT_TYPE_VCARD.equals(Customization.getInstance().getContactType())) {
            // this is hacky, but the way the code is organized today we need
            // to change sync sources encoding for all non-SIF sources
            Log.info("Setting encoding to NONE for source " + name);
            config.setEncoding(com.funambol.syncml.spds.SyncSource.ENCODING_NONE);
        }

        // Read from persistent store?
        if (read) {
            try {
                Log.debug("Reading config from store " + configKey);

                byte[] byte_array = Configuration.getValue(configKey, new byte[0]);
                if (byte_array != null) {
                    Log.debug("Data Found");
                    DataInputStream temp = new DataInputStream(new ByteArrayInputStream(byte_array));
                    config.deserialize(temp);
                    temp.close();
                }
            } catch (final Exception e) {
                Log.error("Exception while initializating (reading) of SourceConfig ["
                           + name + "]");
            }
        }

        // Restore anchors?
        if (restore) {
            config.setLastAnchor(last);
            config.setNextAnchor(next);
        }
        // Save!
        Configuration.saveSourceConfig(configKey, config);
        return config;
    }
}


