/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.io.UnsupportedEncodingException;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.funambol.util.Base64;
import com.funambol.util.StringUtil;
import com.funambol.util.Log;

import com.funambol.blackberry.source.BBPIMSyncSourceWithUpdateFix;
import com.funambol.syncml.spds.SyncManager;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.protocol.SyncMLStatus;
import com.funambol.syncml.protocol.SyncMLCommand;
import com.funambol.util.XmlUtil;
import com.funambol.util.XmlException;
import com.funambol.util.ChunkedString;
import com.funambol.util.CodedException;

/**
 * This class is a specialization of the API Sync Manager. Its purpose is to
 * allow the client to perform a preprocess on messages coming from the server.
 * In particular the class scans incoming messages searching for update/delete
 * commands and build a list of items what will be affected by these commands.
 * Such a list is then handed to sync sources so that they can preload these
 * items.
 * In the current implementation this is only done if the sync source being
 * synchronized is a BBPIMSyncSourceWithUpdateFix, but the mechanism will be
 * extended to all sources.
 */
public class BBSyncManager extends SyncManager {

    public BBSyncManager(SyncConfig conf) {
        super(conf);
    }

    protected boolean processModifications(ChunkedString modifications)
            throws SyncException {

        // At the moment we limit this work for this specific case as we want to
        // minimize the impact on other platforms
        if (source instanceof BBPIMSyncSourceWithUpdateFix) {
            preprocessModificationsBody(modifications);
        }
        return super.processModifications(modifications);
    }

    protected void preprocessModificationsBody(ChunkedString modifications) {

        ChunkedString bodyTag = null;
        Log.info("[BBSyncManager.preprocessModificationsBody]");

        try {
            // Get message body
            bodyTag = XmlUtil.getTagValue(modifications, SyncML.TAG_SYNCBODY);
        } catch (XmlException e) {
            Log.error("[preprocessModification] error parsing message: " + e.toString());
            return;
        }


        if (alertCode != SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT &&
            alertCode != SyncML.ALERT_CODE_REFRESH_FROM_CLIENT) {

            try {

                Vector xmlBody = new Vector(1);
                Vector cmdTags = null;
                xmlBody.addElement(bodyTag);

                // If the server sends a <Sync> tag, the message contains
                // modifications, otherwise it contains only status.
                if (XmlUtil.getTag(bodyTag, SyncML.TAG_SYNC) != - 1) {
                    String[] cmdNames = {
                        SyncML.TAG_REPLACE,
                        SyncML.TAG_DELETE
                    };

                    // Process commands, one kind at a time, in the order
                    // defined above
                    Vector itemsKey = new Vector();
                    for (int c = 0; c < cmdNames.length; c++) {
                        int count = 0;
                        Log.debug("[preprocessModificationsBody] processing " + cmdNames[c] + " commands");
                        cmdTags = XmlUtil.getTagValues(xmlBody, cmdNames[c]);

                        for (int i = 0,  l = cmdTags.size(); i < l; i++) {
                            ChunkedString command = (ChunkedString) cmdTags.elementAt(i);

                            try {
                                Vector itemTags = XmlUtil.getTagValues(command, SyncML.TAG_ITEM);
                                int len = itemTags.size();

                                // Process items
                                SyncMLStatus status = null;
                                for (int j = 0; j < len; j++) {
                                    ChunkedString xmlItem = (ChunkedString) itemTags.elementAt(j);
                                    String key = XmlUtil.getTagValue(xmlItem, SyncML.TAG_LOC_URI).toString();
                                    itemsKey.addElement(key);
                                }
                            } catch (XmlException xe) {
                                Log.error("[preprocessCommand] Parse error: " + xe.toString());
                            }
                            command = null;
                        }
                        cmdTags = null;
                        Log.info(cmdNames[c] + ": " + count + " items processed");
                    }

                    if (source instanceof BBPIMSyncSourceWithUpdateFix) {
                        BBPIMSyncSourceWithUpdateFix bbSource = (BBPIMSyncSourceWithUpdateFix)source;
                        bbSource.prepareModifications(itemsKey);
                    }

                }
            } catch (Exception e) {
                Log.error("[processModification] error parsing command: " + e.toString());
            }
        }
    }
}


