/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;
import java.util.Enumeration;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.DeviceConfig;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;
import com.funambol.util.AppProperties;

/**
 * Configuration class for configuration details
 */
public class Configuration {

    // ------------------------------------------------------------ Constants
    public static final String CONF_KEY_VERSION                = "VERSION";

    public static final String CONF_KEY_LOG_LEVEL              = "LOG_LEVEL";

    public static final String CONF_KEY_SYNC_URL               = "SYNC_URL";

    public static final String CONF_KEY_USERNAME               = "USERNAME";

    public static final String CONF_KEY_PASSWORD               = "PASSWORD";

    public static final String CONF_KEY_SYNC_SOURCE            = "SYNC_SOURCE";
    public static final String CONF_KEY_SYNC_URI               = "SYNC_SOURCE_URI";

    public static final String CONF_KEY_POLL_PIM_FLAG          = "POLL_PIM";

    public static final String CONF_KEY_POLL_PIM_TIME          = "POLL_PIM_TIME";

    public static final String CONF_KEY_SYNC_RANGE_PAST        = "RANGE_PAST";

    public static final String CONF_KEY_SYNC_RANGE_FUTURE      = "RANGE_FUTURE";

    public static final String CONF_KEY_SYNC_TYPE              = "SYNC_TYPE";

    public static final String CONF_KEY_BLOCK_INCOMING_INVITES = "BLOCK_INVITES";

    public static final String UPDATE_URL = "UPDATE_URL";
    public static final String UPDATE_TYPE = "UPDATE_TYPE";
    public static final String AVAILABLE_VERSION ="AVAILABLE_VERSION";
    public static final String LAST_UPDATE_CHECK = "LAST_UPDATE_CHECK";
    public static final String CHECK_INTERVAL   = "CHECK_INTERVAL";
    public static final String REMINDER_INTERVAL = "REMINDER_INTERVAL";
    public static final String LAST_REMINDER = "LAST_REMINDER";
    public static final String SKIP_UPDATE = "SKIP_UPDATE";

    

    public static final String MAXMSGSIZE                      = "16000";

    public static final String CONFIG_VERSION                  = "2";

    public static String       OLD_CONFIG_VERSION              = "1";

    public static String       PROTOCOL;

    public static String       RETRY;

    public static int          logLevel                        = 0;              // disabled

    public static final int    CONF_OK                         = 0;

    public static final int    CONF_NOTSET                     = -1;

    public static final int    CONF_INVALID                    = -2;

    public static String       syncUrl;

    public static String       username;

    public static String       password;

    public static boolean      enablePollingPIM;

    public static int          pollingPIMInterval;

    public static int          rangePast;

    public static int          rangeFuture;

    public static int          syncType;

    public static boolean      blockIncomingInvites            = false;

    public static boolean      initialized                     = false;

    private static boolean     loaded                          = false;

    private static Hashtable   appSourceConfigs                = new Hashtable();

    //update parameters
    public static String    downloadUrl=" ";;
    public static String    updatetype=" ";
    public static String    availableVersion=" ";;
    public static long      lastUpdateCheck = 0L;
    public static long      checkInterval = 24 * 60 * 60 * 1000; // 24hours in millisecs
    public static long      reminderInterval;
    public static long      lastReminder;
    public static boolean   skip = false;

    // These constants are only needed for configuration migration
    private static final String CONF_KEY_SYNC_CONTACT_FLAG       = "SYNC_CONTACT";
    private static final String CONF_KEY_SYNC_CONTACT_URI        = "CONTACT_URI";
    private static final String CONF_KEY_SYNC_CAL_FLAG           = "SYNC_CAL";
    private static final String CONF_KEY_SYNC_CAL_URI            = "CAL_URI";
    private static final String CONF_KEY_SYNC_NOTE_FLAG          = "SYNC_NOTE";
    private static final String CONF_KEY_SYNC_NOTE_URI           = "NOTE_URI";
    private static final String CONF_KEY_SYNC_TASK_FLAG          = "SYNC_TASK";
    private static final String CONF_KEY_SYNC_TASK_URI           = "TASK_URI";


    private static void copyDefaults() {

        Log.info("[Configuration.copyDefaults]");

        // Jad has precedence over Customization as it allows a per user
        // settings
        AppProperties properties = new AppProperties(null);
        syncUrl  = properties.get(AppProperties.URL_ATTR);
        username = properties.get(AppProperties.USER_ATTR);
        password = properties.get(AppProperties.PASSWORD_ATTR);
        String checkIntervalprop = properties.get(AppProperties.AUTOUPDATE_CHECK_INTERVAL);
        String reminderIntervalprop = properties.get(AppProperties.AUTOUPDATE_REMINDER_INTERVAL);

        if (syncUrl == null) {
            syncUrl  = Customization.getInstance().getServerUriDefault();
        }

        if (username == null) {
            username = Customization.getInstance().getUserDefault();
        }

        if (password == null) {
            password = Customization.getInstance().getPasswordDefault();
        }
        
        if(checkIntervalprop == null){
            checkInterval = Customization.getInstance().getCheckUpdtIntervalDefault();
        }else{
            checkInterval = Long.parseLong(checkIntervalprop);
        }

        if(reminderIntervalprop == null){
            reminderInterval = Customization.getInstance().getReminderUpdtIntervalDefault();
        }else{
            reminderInterval = Long.parseLong(reminderIntervalprop);
        }

        enablePollingPIM = StaticDataHelper.getEnablePollingPIMDefault();
        pollingPIMInterval = StaticDataHelper.getPollingPIMIntervalDefault();

        rangePast = StaticDataHelper.getRangePastDefault();
        rangeFuture = StaticDataHelper.getRangeFutureDefault();

        syncType = StaticDataHelper.getSyncTypeDefault();
        blockIncomingInvites = StaticDataHelper.getBlockIncomingInvitesDefault();
    
        initSource(null);
    }

    /**
     * This must be called once at the beginning of the application.
     **/
    public static void init(Class c) throws Exception {
        if (!initialized) {
            StaticDataHelper.init(c, StaticDataHelper.XML_LANGUAGE_FILE);
            copyDefaults();
        }
        initialized = true;
    }

    // ------------------------------------------------------------ Public
    /**
     * Load the current config from the persistent store.
     * 
     * @return: <li><b>CONF_OK</b>: if all the data were present in the store.
     *          <li><b>CONF_NOTSET</b>: if the store is not present.
     *          Configuration remains untouched. <li><b>CONF_INVALID</b>: if the
     *          store does not contain valid data. Configuration is reverted to
     *          default. Note: if a parameter is not present in the store, the
     *          current value is kept for it.
     */
    public static int load() {

        if (!loaded) {

            Log.info("Loading Configuration data from persistent storage");
            final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
            try {
                final ConfigData contents = (ConfigData) pobj.getContents();
                if (contents == null) {
                    Log.error("Configuration not set, using current values.");
                    copyDefaults();
                    return CONF_NOTSET;
                }
                if (contents.get(CONF_KEY_VERSION) == null) {
                    Log.error("Configuration invalid: no conf key version set.");
                    PersistentStore.destroyPersistentObject(Constants.CONFIG_KEY);
                    copyDefaults();
                    return CONF_INVALID;
                }

                OLD_CONFIG_VERSION = contents.getValue(Configuration.CONF_KEY_VERSION,
                                                       CONFIG_VERSION);
                // Migrate the configuration if needed
                migrateConfig(contents);

                syncUrl = contents.getValue(CONF_KEY_SYNC_URL, syncUrl);
                if (Customization.getInstance().stripPortFromSyncAddress()) {
                    syncUrl = StringUtil.removePortFromUrl(syncUrl, "http");
                }

                username = contents.getValue(CONF_KEY_USERNAME, username);
                password = contents.getValue(CONF_KEY_PASSWORD, password);

                Log.debug("[Configuration.load]Init Registered Sources");
                initSource(contents);

                Log.debug("[Configuration.load]values");
                enablePollingPIM = contents.getValue(Configuration.CONF_KEY_POLL_PIM_FLAG,
                        enablePollingPIM);
                pollingPIMInterval = contents.getValue(Configuration.CONF_KEY_POLL_PIM_TIME,
                        pollingPIMInterval);

                rangePast = contents.getValue(Configuration.CONF_KEY_SYNC_RANGE_PAST, rangePast);
                rangeFuture = contents.getValue(Configuration.CONF_KEY_SYNC_RANGE_FUTURE,
                        rangeFuture);

                syncType = contents.getValue(Configuration.CONF_KEY_SYNC_TYPE, syncType);

                logLevel = contents.getValue(Configuration.CONF_KEY_LOG_LEVEL, logLevel);

                blockIncomingInvites = contents.getValue(
                        Configuration.CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);

                downloadUrl = contents.getValue(Configuration.UPDATE_URL, downloadUrl);
                updatetype = contents.getValue(Configuration.UPDATE_TYPE, updatetype);
                availableVersion = contents.getValue(Configuration.AVAILABLE_VERSION, availableVersion);
                lastUpdateCheck = contents.getValue(Configuration.LAST_UPDATE_CHECK, lastUpdateCheck);
                checkInterval = contents.getValue(Configuration.CHECK_INTERVAL, checkInterval);
                reminderInterval = contents.getValue(Configuration.REMINDER_INTERVAL, reminderInterval);
                lastReminder = contents.getValue(Configuration.LAST_REMINDER, lastReminder);
                skip = contents.getValue(Configuration.SKIP_UPDATE, skip);
                
                // TODO FIXME
                //loaded = true;

                return CONF_OK;
            } catch (final ClassCastException cce) {
                Log.error("ClassCastException - Configuration invalid: reverting to default.");
                PersistentStore.destroyPersistentObject(Constants.CONFIG_KEY);
                copyDefaults();
                return CONF_INVALID;
            }
        }

        return CONF_OK;
    }

    /**
     * Save the current config to the persistent store.
     */
    public static int save(Controller controller) {
        Log.info("Saving Configuration data to persistent storage");
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null) {
            contents = new ConfigData();
        }

        // This field can be used to check which version of the sw
        // saved the data. If not present, it is assumed it's an
        // old version and the saved config is discarded.
        contents.setValue(Configuration.CONF_KEY_VERSION, CONFIG_VERSION);
        contents.setValue(Configuration.CONF_KEY_LOG_LEVEL, logLevel);

        contents.setValue(Configuration.CONF_KEY_SYNC_URL, syncUrl);
        contents.setValue(Configuration.CONF_KEY_USERNAME, username);
        contents.setValue(Configuration.CONF_KEY_PASSWORD, password);

        Log.debug("Saving sources configs");

        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            int    id   = appSource.getId();
            String enabledKey = Configuration.CONF_KEY_SYNC_SOURCE + "_" + id;
            String uriKey     = Configuration.CONF_KEY_SYNC_URI    + "_" + id;
            AppSyncSourceConfig sc = (AppSyncSourceConfig)appSourceConfigs.get(new Integer(id));

            Log.debug("sc="+sc);

            boolean enabled = sc.getEnabled();
            String  uri     = sc.getUri();

            contents.setValue(enabledKey, enabled);
            contents.setValue(uriKey, uri);
        }

        contents.setValue(Configuration.CONF_KEY_POLL_PIM_FLAG, enablePollingPIM);
        contents.setValue(Configuration.CONF_KEY_POLL_PIM_TIME, pollingPIMInterval);

        contents.setValue(Configuration.CONF_KEY_SYNC_RANGE_PAST, rangePast);
        contents.setValue(Configuration.CONF_KEY_SYNC_RANGE_FUTURE, rangeFuture);

        contents.setValue(Configuration.CONF_KEY_SYNC_TYPE, syncType);

        contents.setValue(Configuration.CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);
        
        contents.setValue(Configuration.UPDATE_URL, downloadUrl);
        contents.setValue(Configuration.UPDATE_TYPE, updatetype);
        contents.setValue(Configuration.AVAILABLE_VERSION, availableVersion);
        contents.setValue(Configuration.LAST_UPDATE_CHECK, lastUpdateCheck);
        contents.setValue(Configuration.CHECK_INTERVAL, checkInterval);
        contents.setValue(Configuration.REMINDER_INTERVAL, reminderInterval);
        contents.setValue(Configuration.LAST_REMINDER, lastReminder);
        contents.setValue(Configuration.SKIP_UPDATE, skip);

        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
            if (controller != null) {
                controller.reapplyConfiguration();
            }
        }
        
        return CONF_OK;
    }

    public static int append(String key, byte[] value) {
        Log.trace("[Configuration.append] " + key);
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null) {
            contents = new ConfigData();
        }
        contents.setValue(key, value);
        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }

    public static int append(String key, String value) {
        Log.trace("[Configuration.append] " + key);
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null) {
            contents = new ConfigData();
        }
        contents.setValue(key, value);
        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }

    public static byte[] getValue(String key, byte[] value) {
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        try {
            final ConfigData contents = (ConfigData) pobj.getContents();
            if (contents == null) {
                return null;
            }
            return contents.getValue(key, value);
        } catch (final ClassCastException e) {
        }
        return null;
    }

    private static DeviceConfig devconf;

    public static int getSyncMode() {
        int syncMode = 0;
        switch (Configuration.syncType) {
            case StaticDataHelper.SYNC_TYPE_TWO_WAY:
            default:
                syncMode = SyncML.ALERT_CODE_FAST;
                break;

            case StaticDataHelper.SYNC_TYPE_ONE_WAY_TO_CLIENT:
                syncMode = SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER;
                break;

            case StaticDataHelper.SYNC_TYPE_ONE_WAY_TO_SERVER:
                syncMode = SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT;
                break;
        }
        return syncMode;
    }

    public static SyncConfig getSyncConfig() {
        SyncConfig syncConfig = new SyncConfig();

        // TODO set before this runs
        syncConfig.syncUrl = Configuration.syncUrl;
        syncConfig.lastServerUrl = Configuration.syncUrl;
        syncConfig.userName = Configuration.username;
        syncConfig.password = Configuration.password;
        
        // Remember to update the blackberry synclet pattern (server side) when changing the user agent
        syncConfig.userAgent = "Funambol BlackBerry Plug-in " + BBVersion.VERSION;
        syncConfig.deviceConfig = getDeviceConfig();
        syncConfig.forceCookies = false;

        return syncConfig;
    }

    /**
     * Helper function to build the DevConf
     * 
     * @return A DeviceConfig
     */
    private static DeviceConfig getDeviceConfig() {
        if (devconf != null) {
            return devconf;
        }
        devconf = new DeviceConfig();
        devconf.man = DeviceInfo.getManufacturerName();
        devconf.mod = DeviceInfo.getDeviceName();
        devconf.swv = Constants.VER;
        devconf.hwv = "" + DeviceInfo.getPlatformVersion();
        devconf.devID = "fbb-" + DeviceInfo.getDeviceId();
        devconf.maxMsgSize = 32 * 1024;
        devconf.loSupport = false;
        devconf.utc = true;
        devconf.nocSupport = true;
        return devconf;
    }

    /**
     * Saves a syncsource config to storage
     * 
     * @param storageKey
     * @param config
     */
    public static void saveSourceConfig(final String storageKey, SourceConfig config) {
        Log.debug("Storing SourceConfig [" + config.getName() + "]");
        try {
            ByteArrayOutputStream buff = new ByteArrayOutputStream(512);
            DataOutputStream temp = new DataOutputStream(buff);
            config.serialize(temp);
            append(storageKey, buff.toByteArray());
            temp.close();
        } catch (final Exception e) {
            Log.error("Exception while storing SourceConfig [" + config.getName()
                       + "] " + e.toString());
        }
    }

    public static boolean isSourceEnabled(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            return sc.getEnabled();
        }
        // We shall grab the default value from the customization here
        return Customization.getInstance().isSourceEnabledByDefault(id);
    }

    public static String getSourceUri(int id) {
        String uri = null;
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            uri = sc.getUri();
        }
        if (uri == null) {
            // We shall grab the default value from the customization here
            uri = Customization.getInstance().getDefaultSourceUri(id);
            // This should never happen, but just in case...
            if (uri == null) {
                uri = "undefined";
            }
        }
        return uri;
    }

    public static void setSourceConfig(int id, boolean enabled, String uri) {
        AppSyncSourceConfig sc = new AppSyncSourceConfig(id, enabled, uri);
        appSourceConfigs.put(new Integer(id), sc);
    }


    public static void disableSource(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        // The source config has never been set, so we use the default uri
        if (sc == null) {
            String uri = getSourceUri(id);
            sc = new AppSyncSourceConfig(id, false, uri);
            Log.debug("[Configuration.load]AppSyncSourceConfig");
            appSourceConfigs.put(new Integer(id), sc);
        }
        sc.setEnabled(false);
    }

    private static void initSource(final ConfigData contents) {
        // Try to load the config for each registered source
        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) sources.nextElement();
            int id = appSource.getId();
            String enabledKey = CONF_KEY_SYNC_SOURCE + "_" + id;
            String uriKey = CONF_KEY_SYNC_URI + "_" + id;

            boolean enabled = isSourceEnabled(id);
            String uri = getSourceUri(id);
            if (contents != null) {
                enabled = contents.getValue(enabledKey, enabled);
                uri = contents.getValue(uriKey, uri);
            }


            AppSyncSourceConfig sc = new AppSyncSourceConfig(id, enabled, uri);
            Log.debug("[Configuration.load]AppSyncSourceConfig");
            appSourceConfigs.put(new Integer(id), sc);
        }
    }

    /**
     * This method migrates old configuration to the new one
     * Currently we support upgrading from 7.0 and 7.1 to 7.2.
     */
    private static void migrateConfig(ConfigData contents) {
        Log.debug("migrate config: " + contents + " -- " + OLD_CONFIG_VERSION);

        if (OLD_CONFIG_VERSION.equals("20060721") || OLD_CONFIG_VERSION.equals("1")) {
            Log.debug("Configuration needs to be migrated");

            // This is an old version, upgrading is necessary
            boolean syncContact  = true;
            boolean syncCalendar = true;
            boolean syncTask     = true;
            boolean syncNote     = true;
            String contactSourceUri = Customization.getInstance().getDefaultContactUri(); //"card";
            String calendarSourceUri = Customization.getInstance().getDefaultCalendarUri(); //"scal";
            String taskSourceUri = Customization.getInstance().getDefaultTaskUri(); //"staks";
            String noteSourceUri = Customization.getInstance().getDefaultNoteUri(); //"snote";

            String oldContactSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_CONTACT_URI,
                                                  contactSourceUri);

            syncContact = contents.getValue(Configuration.CONF_KEY_SYNC_CONTACT_FLAG,
                                            syncContact);
            contactSourceUri = Customization.getInstance().getDefaultSourceUri(AppSyncSourceManager.SYNC_FLAG_CONTACTS);
            
            syncCalendar = contents.getValue(Configuration.CONF_KEY_SYNC_CAL_FLAG,
                                             syncCalendar);
            calendarSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_CAL_URI,
                                                  calendarSourceUri);
            
            syncTask = contents.getValue(Configuration.CONF_KEY_SYNC_TASK_FLAG, syncTask);
            taskSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_TASK_URI,
                                              taskSourceUri);
            
            syncNote = contents.getValue(Configuration.CONF_KEY_SYNC_NOTE_FLAG, syncNote);
            noteSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_NOTE_URI,
                                              noteSourceUri);
            
            contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                              syncContact);
            if(oldContactSourceUri.indexOf("scard")!= -1){
                contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                                 contactSourceUri);
            }else{
                contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                                 oldContactSourceUri);
            }

            contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                              syncCalendar);
            contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                              calendarSourceUri);

            contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_TASKS,
                              syncTask);
            contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_TASKS,
                              taskSourceUri);

            contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_NOTES,
                              syncNote);
            contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_NOTES,
                              noteSourceUri);
        }
    }
}
