/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;

public class Customization {

    // The XML Language (e.g. en_us, de_de)
    private static final String   XML_LANGUAGE            = "en_us";
    
    // Icons/images customization
    private static final String   IMAGE_CONTACTS          = "contacts.png";
    private static final String   IMAGE_CALENDAR          = "calendar.png";
    private static final String   IMAGE_TASKS             = "tasks.png";
    private static final String   IMAGE_NOTES             = "notes.png";
    private static final String   IMAGE_PHOTOS            = "photos.png";
    private static final String   IMAGE_CONTACTS_DISABLED = "contacts_grey.png";
    private static final String   IMAGE_CALENDAR_DISABLED = "calendar_grey.png";
    private static final String   IMAGE_TASKS_DISABLED    = "tasks_grey.png";
    private static final String   IMAGE_NOTES_DISABLED    = "notes_grey.png";
    private static final String   IMAGE_PHOTOS_DISABLED   = "photos_grey.png";
    private static final String   IMAGE_ALL               = "sync_logo.png";
    private static final String   IMAGE_ALL_BG            = "bg_button_dark.png";
    private static final String   IMAGE_ALL_BG_HIGHLIGHT  = "bg_button_darkblue.png";
    private static final String   IMAGE_BG                = "bg_button_light.png";
    private static final String   IMAGE_BG_HIGHLIGHT      = "bg_button_blue.png";
    private static final String   IMAGE_ERROR             = "alert.png";
    private static final String   IMAGE_OK                = "complete.png";
    private static final String   IMAGE_CANCEL            = "cancel.png";
    private static final String   IMAGE_WARNING           = "cancel.png";

    // Settings customization
    private static final String   SERVER_URI              = "http://my.funambol.com/sync";
    private static final String   USERNAME                = "";
    private static final String   PASSWORD                = "";

    public static boolean EVENT_RANGE_LIMITED = true;
    public static String  PREFERRED_FONT      = "Verdana";

    // About customization
    private static final String   ABOUT_COPYRIGHT_DEFAULT = "Copyright (C) 2009 Funambol, Inc.";
    private static final String   ABOUT_SITE_DEFAULT      = "www.funambol.com";
    
    /** Set this parameter if a fix apn is to be used from your application*/
    private static final String   FIXED_APN               = null;

    // Log customization
    private static boolean LOG_ENABLED         = true;
    private static boolean SEND_LOG_ENABLED    = false;
    private static String  LOG_TITLE           = "Funambol-SyncClient";
    private static String  LOG_FILE_NAME       = "synclog.txt";
    private static String  LOG_PATH            = "file:///store/home/user";

    final private static long CHECK_UPDATE_INTERVAL = (long)24 * (long)60 * (long)60 * (long)1000;   // 1 day in milliseconds
    final private static long REMINDER_UPDATE_INTERVAL = (long)2 * (long)60 * (long)60 * (long)1000; // 2 hours in seconds

    public static boolean LOG_ENABLED_IN_SETTINGS_SCREEN = true;
    
    /** 
     * This fields values must change accordingly to the number of registered 
     * sources
     */
    private static final int   UI_CONTACTS_ITEM_INDEX     = 0;
    private static final int   UI_CALENDARS_ITEM_INDEX    = 1;
    private static final int   UI_TASKS_ITEM_INDEX        = 2;
    private static final int   UI_NOTES_ITEM_INDEX        = 3;
    private static final int   UI_PICTURES_ITEM_INDEX     = 4;
    private static final int   UI_MAILS_ITEM_INDEX        = 5;

    private final String   CONTACTS_DEFAULT_URI    = "card";
    private final String   CALENDARS_DEFAULT_URI   = "scal";
    private final String   TASKS_DEFAULT_URI       = "stask";
    private final String   NOTES_DEFAULT_URI       = "snote";
    private final String   PICTURES_DEFAULT_URI    = "briefcase";
    private final String   MAILS_DEFAULT_URI       = "email";
    
    private static Customization instance = null;

    private Hashtable sourcesUri          = new Hashtable();
    private Hashtable activeSources       = new Hashtable();
    private Hashtable enabledSources      = new Hashtable();
    private Hashtable sourcesIcon         = new Hashtable();
    private Hashtable sourcesDisabledIcon = new Hashtable();
    private Hashtable uiSourcesItemIndex  = new Hashtable();
    
    private int[] pollingPimIntervalChoices = {
        5,
        15,
        30,
        60,
        120,
        240,
        480,
        720,
        1440
    };
    
    private Customization() {
        initSourcesInfo();
    }

    public static Customization getInstance() {
        if (instance == null) {
            instance = new Customization();
        }
        return instance;
    }

    public boolean syncAllOnMainScreenRequired() {
        return false;
    }
    
    /**
     * Get the default url connection parameters. To be used in case of fixed 
     * apn to be passed to the API connection manager instance
     * @return String representing the default parameters
     */
    public String getFixedApn() {
        return FIXED_APN;
    }
    
    /**
     * Specifies if the port must be stripped out of the sync url
     */
    public boolean stripPortFromSyncAddress() {
        return false;
    }

    /**
     * Specifies if the login screen must be presented when the application
     * starts and/or credentials are invalid
     */
    public boolean loginScreenRequired() {
        return false;
    }

    /**
     * Specifies if the sync url is editable in both the login and setting
     * screens
     */
    public boolean syncUriEditable() {
        return true;
    }

    public boolean isRangePastVisible() {
        return true;
    }
    
    public boolean isRangeFutureVisible() {
        return true;
    }
    
    public boolean isBlockInvitesFieldVisible() {
        return true;
    }
    
    public boolean isSourceUriVisible() {
        return true;
    }
    
    
    /**
     * Specifies if the attendees filtering is enabled or disabled
     */
    public boolean invitesFiltering() {
        return false;
    }

    /**
     * Specifies if contact sync is enabled or disabled
     */
    public boolean isSourceActive(int id) {
        Boolean active = (Boolean)activeSources.get(new Integer(id));
        return active.booleanValue();
    }

    /**
     * Specifies if contact sync is enabled or disabled
     */
    public boolean isSourceEnabledByDefault(int id) {
        Boolean enabled = (Boolean)enabledSources.get(new Integer(id));
        return enabled.booleanValue();
    }


    public String getSourceIcon(int id) {
        String icon = (String)sourcesIcon.get(new Integer(id));
        return icon;
    }

    public String getSourceDisabledIcon(int id) {
        String icon = (String)sourcesDisabledIcon.get(new Integer(id));
        return icon;
    }

    /**
     * Specifies if slow sync needs to be confirmed by the user
     */
    public boolean confirmSlowSync() {
        return false;
    }

    /**
     * Specifies if the user must be warned on large amount of deletes
     */
    public boolean warnOnDeletes() {
        return false;
    }

    /**
     * Specifies if photos must be synchronized in contact sync
     */
    public boolean syncContactPhoto() {
        return true;
    }

    /**
     * Returns the type of format that must be used to sync contacts
     * @return the vcard type as defined in AppSyncSourceManager
     */
    public String getContactType() {
        return AppSyncSourceManager.CONTACT_TYPE_VCARD;
    }

    //Used for MailTrust update
    public boolean checkForUpdates() {
        return false;
    }

    //Used for Funambol update
    public boolean enableUpdaterManager() {
        return false;
    }

    public boolean syncCompanyDirectory() {
        return false;
    }

    public boolean disablePollOn8830() {
        return false;
    }

    public boolean showTitleWithVersion() {
        return false;
    }

    public boolean rolloverIcon() {
        return false;
    }

    public String getSyncAllIcon() {
        return IMAGE_ALL;
    }

    public String getSyncAllBackground() {
        return IMAGE_ALL_BG;
    }

    public String getSyncAllHighlightedBackground() {
        return IMAGE_ALL_BG_HIGHLIGHT;
    }

    public String getButtonBackground() {
        return IMAGE_BG;
    }

    public String getButtonHighlightedBackground() {
        return IMAGE_BG_HIGHLIGHT;
    }

    public String getDefaultSourceUri(int id) {
        String uri = (String)sourcesUri.get(new Integer(id));
        return uri;
    }

    public int getPreferredUiSourceItemIndex(int id) {
        int index = ((Integer)uiSourcesItemIndex.get(new Integer(id))).intValue();;
        return index;
    }

    public String getOkIcon() {
        return IMAGE_OK;
    }

    public String getErrorIcon() {
        return IMAGE_ERROR;
    }

    public String getCancelledIcon() {
        return IMAGE_CANCEL;
    }

    public String getWarningIcon() {
        return IMAGE_WARNING;
    }

    public boolean gotoMenuEnabled() {
        return true;
    }

    public boolean logEnabled() {
        return LOG_ENABLED;
    }

    public boolean isLogEnabledInSettingsScreen() {
        return LOG_ENABLED_IN_SETTINGS_SCREEN;
    }


    public boolean sendLogEnabled() {
        return SEND_LOG_ENABLED;
    }

    public String getServerUriDefault() {
        return SERVER_URI;
    }

    public String getUserDefault() {
        return USERNAME;
    }

    public String getPasswordDefault() {
        return PASSWORD;
    }

    public long getCheckUpdtIntervalDefault(){
        return CHECK_UPDATE_INTERVAL;
    }

    public long getReminderUpdtIntervalDefault(){
        return REMINDER_UPDATE_INTERVAL;
    }

    public String getLogAppTitle() {
        return LOG_TITLE;
    }

    public boolean enableFileLogging() {
        return true;
    }

    public String getLogFileDirectory() {
        return LOG_PATH;
    }

    public String getLogFileName() {
        return LOG_FILE_NAME;
    }

    public boolean useDefaultFont() {
        return true;
    }

    public String getPreferredFont() {
        return PREFERRED_FONT;
    }
    
    /** About customization **/
    public String getAboutCopyright() {
        return ABOUT_COPYRIGHT_DEFAULT;
    }
    public String getAboutSite() {
        return ABOUT_SITE_DEFAULT;
    }
    public boolean showAboutLicence() {
        return true;
    }
    public boolean showPoweredBy() {
        return false;
    }
    
    public boolean hideHelpScreen() {
        return false;
    }

    public boolean reportStatusInSyncAll() {
        return false;
    }
    
    public String getXMLLanguage() {
        return XML_LANGUAGE;
    }

    public int[] getPollingPimIntervalChoices() {
        return pollingPimIntervalChoices;
    }
    
    public String getDefaultContactUri() {
        return CONTACTS_DEFAULT_URI;
    }
    
    public String getDefaultCalendarUri() {
        return CALENDARS_DEFAULT_URI;
    }
    
    public String getDefaultTaskUri() {
        return TASKS_DEFAULT_URI;
    }
    
    public String getDefaultNoteUri() {
        return NOTES_DEFAULT_URI;
    }
    
    private void initSourcesInfo() {
        int id;

        id = AppSyncSourceManager.SYNC_FLAG_CONTACTS;
        sourcesUri.put(new Integer(id), CONTACTS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(true));
        enabledSources.put(new Integer(id), new Boolean(true));
        sourcesIcon.put(new Integer(id), IMAGE_CONTACTS);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_CONTACTS_DISABLED);
        uiSourcesItemIndex.put(new Integer(id), new Integer(UI_CONTACTS_ITEM_INDEX));
        
        id = AppSyncSourceManager.SYNC_FLAG_CALENDAR;
        sourcesUri.put(new Integer(id), CALENDARS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(true));
        enabledSources.put(new Integer(id), new Boolean(true));
        sourcesIcon.put(new Integer(id), IMAGE_CALENDAR);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_CALENDAR_DISABLED);
        uiSourcesItemIndex.put(new Integer(id), new Integer(UI_CALENDARS_ITEM_INDEX));
        
        id = AppSyncSourceManager.SYNC_FLAG_TASKS;
        sourcesUri.put(new Integer(id), TASKS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(true));
        enabledSources.put(new Integer(id), new Boolean(true));
        sourcesIcon.put(new Integer(id), IMAGE_TASKS);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_TASKS_DISABLED);
        uiSourcesItemIndex.put(new Integer(id), new Integer(UI_TASKS_ITEM_INDEX));
        
        id = AppSyncSourceManager.SYNC_FLAG_NOTES;
        sourcesUri.put(new Integer(id), NOTES_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(true));
        enabledSources.put(new Integer(id), new Boolean(true));
        sourcesIcon.put(new Integer(id), IMAGE_NOTES);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_NOTES_DISABLED);
        uiSourcesItemIndex.put(new Integer(id), new Integer(UI_NOTES_ITEM_INDEX));
        
        id = AppSyncSourceManager.SYNC_FLAG_PHOTOS;
        sourcesUri.put(new Integer(id), PICTURES_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(false));
        enabledSources.put(new Integer(id), new Boolean(false));
        sourcesIcon.put(new Integer(id), IMAGE_PHOTOS);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_PHOTOS_DISABLED);
        uiSourcesItemIndex.put(new Integer(id), new Integer(UI_PICTURES_ITEM_INDEX));

        id = AppSyncSourceManager.SYNC_FLAG_MAIL;
        sourcesUri.put(new Integer(id), MAILS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(false));
        enabledSources.put(new Integer(id), new Boolean(false));
        sourcesIcon.put(new Integer(id), IMAGE_NOTES);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_NOTES_DISABLED);
        uiSourcesItemIndex.put(new Integer(id), new Integer(UI_MAILS_ITEM_INDEX));
    }
    
}

