/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.pim.PIM;

import net.rim.blackberry.api.pdap.BlackBerryPIM;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.ui.UiApplication;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.helper.EventHelper;
import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.blackberry.helper.HelperFactory;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.source.BBPIMSyncListener;
import com.funambol.blackberry.source.BBPIMSyncSource;
import com.funambol.blackberry.source.BBPIMSyncSourceWithUpdateFix;
import com.funambol.blackberry.source.Poller;
import com.funambol.blackberry.ui.rim.RimControllerDataFactory;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Appender;
import com.funambol.util.FileAppender;
import com.funambol.util.ConsoleAppender;
import com.funambol.util.MemoryAppender;
import com.funambol.util.Log;
import com.funambol.util.MultipleAppender;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;
import com.funambol.util.appenders.BlackberryEventLogAppender;
import com.funambol.util.PluginObjectWrapperHanlder;
import com.funambol.storage.BlackberryRecordStore;
import com.funambol.util.ConnectionManager;


/**
 * Provides the user interface with the configuration settings, mail
 * synchronization capabilities (polling included), and contacts and calendar
 * events synchronization
 * 
 */
public class SyncClient extends UiApplication {

    // Used to select which source to sync in synchronize()

    private Poller                 pollerPIM = null;

    private final Updater          updater;

    private final Controller       controller;

    private static Appender        logAppender  = null;
    private MemoryAppender         memoryAppender = null;

    /**
     * The default constructor. It opens internally the BlackBerry PIM lists
     * (contacts and calendar events), adds a listener to them and to the mail
     * folders, and pushes the home screen to allow user configuration
     */
    public SyncClient() throws Exception {

        if (Customization.getInstance().enableFileLogging()) {
            String logFilePath = Customization.getInstance().getLogFileDirectory();
            String logFileName = Customization.getInstance().getLogFileName();
            Appender fileAppender = new FileAppender(logFilePath, logFileName);
            Log.initLog(fileAppender);
            Log.setLogLevel(Log.TRACE);
            /*
            Log.initLog(new ConsoleAppender());
            Log.setLogLevel(Log.TRACE);
            */
        } else {
            MultipleAppender appenders = new MultipleAppender();
            String appName = Customization.getInstance().getLogAppTitle();
            Appender eventAppender = new BlackberryEventLogAppender(appName);
            eventAppender.initLogFile();
            appenders.addAppender(eventAppender);
            memoryAppender = new MemoryAppender();
            appenders.addAppender(memoryAppender);
            logAppender = appenders;
            Log.initLog(logAppender, Log.TRACE);
        }

        /*
         * get configurations contents from the store. The class configuration
         * initializes also StaticDataHelper defaults.
         */
        Configuration.init(this.getClass());
        Configuration.load();
        if (!Configuration.initialized) {
            throw new Exception(); // TODO
        }

        // Create syncsource configs
        initializeSources();

        // Set running log level as startup phase is over
        //Log.setLogLevel(Configuration.logLevel);

        //Set the Connection parameters if the customization file has one fixed 
        //apn  
        ConnectionManager.getInstance().setFixedApn(Customization.getInstance().getFixedApn());

        ///////////////////////////
        //// TEMP TODO FIXME
        Configuration.load();
        ///////////////////////////

        // Initialize the BlackBerry Record Store for this application
        PluginObjectWrapperHanlder wrapper = new PluginObjectWrapperHanlder();
        BlackberryRecordStore.init(wrapper, "Funambol_PIM_Sync_Client");

        controller = new Controller(this, new RimControllerDataFactory());

        // The controller must be registered in each source
        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            // TODO FIXME, this must change. It is only necessary to stop the
            // sync. Once the SyncManager will be stoppable the sync source
            // will not need the controller any longer
            SyncSource source = appSource.getSyncSource();
            if (source != null) {
                if (source instanceof BBPIMSyncSource) {
                    BBPIMSyncSource bbPIMSyncSource = (BBPIMSyncSource)source;
                    bbPIMSyncSource.setController(controller.getSynchronizationController());
                }
                // Now register the listener into the source
                BBPIMSyncListener lis = new BBPIMSyncListener(controller.getSyncStatusController(),
                        controller.getSynchronizationController(),
                        appSource);
                source.setListener(lis);
            }
        }

        // Create the updater
        updater = new Updater();

        controller.reapplyConfiguration();

        /*
         * The UI engine must be accessed holding the event lock object shared
         * with the application's main thread, otherwise an
         * IllegalStateException is thrown (the event lock is held here where
         * the screen is pushed in the foreground)
         */

        Log.debug("[SyncClient]eventLock");
        synchronized (getEventLock()) {
            if (Customization.getInstance().loginScreenRequired()) {
                if (StringUtil.isNullOrEmpty(Configuration.username)
                        || StringUtil.isNullOrEmpty(Configuration.password)) {
                    Log.debug("[SyncClient]login screen");
                    controller.getScreenDisplayController().showLoginScreen(true);
                } else {
                    Log.debug("[SyncClient]home screen");
                    controller.getScreenDisplayController().showHomeScreen(true);
                }
            } else {
                Log.debug("[SyncClient]home screen");
                controller.getScreenDisplayController().showHomeScreen(true);
            }
        }
        Log.debug("[SyncClient]End");

        // TODO FIXME
        //controller.checkForUpdate();
    }

    private void initializeSources() {

        /**
         * Contacts
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_CONTACTS)) {
            try {
                AppSyncSourceManager.getInstance().setUpContactsSource();
            } catch (Throwable t) {
                Log.error("Coud not create contacts sync source, disabling contacts "
                          + t.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_CONTACTS);
            }
        }

        /**
         * Calendar
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_CALENDAR)) {
            try {
                AppSyncSourceManager.getInstance().setUpCalendarSource();
            } catch (Throwable t) {
                Log.error("Coud not create calendar sync source, disabling calendar "
                          + t.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_CALENDAR);
            }
        }

        /**
         * Tasks
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_TASKS)) {
            try {
                AppSyncSourceManager.getInstance().setUpTasksSource();
            } catch (Throwable t) {
                Log.error("Coud not create tasks sync source, disabling tasks "
                          + t.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_TASKS);
            }
        }

        /**
         * Note
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_NOTES)) {
            try {
                AppSyncSourceManager.getInstance().setUpNotesSource();
            } catch (Throwable t) {
                Log.error("Coud not create notes sync source, disabling notes"
                          + t.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_NOTES);
            }
        }

        /**
         * Photos
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_PHOTOS)) {
            try {
                AppSyncSourceManager.getInstance().setUpPhotosSource();
            } catch (Exception e) {
                Log.error("Coud not create photos sync source, disabling photos"
                          + e.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_PHOTOS);
            }
        }
    }

    public SourceConfig getSourceConfig(int type) {
        AppSyncSource   appSource = AppSyncSourceManager.getInstance().getSource(type);
        SyncSource source = appSource.getSyncSource();
        if (source != null) {
            return source.getConfig();
        }
        return null;
    }

    protected SourceConfig setSourceConfig(int type, SourceConfig config) {
        AppSyncSource   appSource = AppSyncSourceManager.getInstance().getSource(type);
        SyncSource source = appSource.getSyncSource();
        if (source != null) {
            source.setConfig(config);
        } else {
            Log.error("Unable to set source config for type " + type);
        }
        return null;
    }

    /**
     * Immediately disable autosync
     */
    public void disablePollingPIM() {
        if (pollerPIM != null && pollerPIM.isAlive()) {
            pollerPIM.disable();
            pollerPIM = null;
        }
    }

    /**
     * Enable autosync. This will reset the timer
     */
    public void enablePollingPIM() {
        disablePollingPIM();

        pollerPIM = new Poller(controller.getSynchronizationController(),
                Configuration.pollingPIMInterval, Configuration.enablePollingPIM);
        pollerPIM.start();
    }

    public void setEnableBlockInvites(boolean enabled) {
        AppSyncSource   appSource = AppSyncSourceManager.getInstance()
                                    .getSource(AppSyncSourceManager.SYNC_FLAG_CALENDAR);
        if (appSource != null) {
            BBPIMSyncSource ss = (BBPIMSyncSource) appSource.getSyncSource();
            if (ss != null) {
                ((EventHelper) ss.getHelper()).setIncomingMailListenerStatus(enabled);
            }
        }
    }

    public Updater getUpdater() {
        return updater;
    }

    public String getLogContent() {
        // Just for safety we check if the memoryAppender is initialized.
        // It it is not, then we have a logical error, but at least we avoid a
        // NPE
        if (memoryAppender != null) {
            return memoryAppender.getLogData();
        } else {
            return "";
        }
    }

    public static Appender getAppender() {
        return logAppender;
    }

}
