/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;

import net.rim.device.api.system.RadioInfo;

import com.funambol.util.CodedException;
import com.funambol.util.HttpTransportAgent;
import com.funambol.util.Log;
import com.funambol.util.SimpleXMLParser;
import com.funambol.util.StaticDataHelper;

public class Updater {

    // Used to select which source to sync in synchronize()

    private boolean updateFlag     = false;

    private boolean updateRequired = false;

    private String  updateOTA      = null;

    private String  updateURL      = null;

    /**
     * 
     * @return true if there is a required update
     */
    public boolean checkUpdate() {
        final String UPDATE_ROOT_TAG = "sync";
        final String BB_UPDATE_TAG = "blackberry";
        final String VERSION_TAG = "version";
        final String REQUIRED_TAG = "required";
        final String URL_TAG = "url";
        final String OTA_TAG = "ota";
        final String VER_SEP = ".";

        if (RadioInfo.getState() == RadioInfo.STATE_OFF
                || RadioInfo.getSignalLevel() == RadioInfo.LEVEL_NO_COVERAGE) {
            return false;
        }

        // TODO Strip all trailing directories from sync server uri
        // and append update.xml
        String url = StaticDataHelper.getLanguage("update_xml");

        try {
            HttpTransportAgent transport = new HttpTransportAgent(
                    url,
                    Configuration.getSyncConfig().userAgent,
                    "UTF-8",
                    false, false);
            
            String response = transport.sendMessage("");

            SimpleXMLParser parser = new SimpleXMLParser();
            parser.addHashtableValueTag(BB_UPDATE_TAG);

            Hashtable output = parser.parseData(UPDATE_ROOT_TAG, response);
            Hashtable bb = (Hashtable) output.get(BB_UPDATE_TAG);
            if (bb != null) {
                String version = Constants.VER;
                String fversion = (String) bb.get(VERSION_TAG);

                // String required = (String)bb.get(REQUIRED_TAG);

                int vpos = 0;
                int vfpos = 0;
                do {
                    vpos = version.indexOf(VER_SEP, 0);
                    vfpos = fversion.indexOf(VER_SEP, 0);
                    if (vpos < 0 && version.length() > 0) {
                        vpos = version.length();
                    }
                    if (vfpos < 0 && fversion.length() > 0) {
                        vfpos = fversion.length();
                    }

                    if (vpos > 0 && vfpos > 0) {
                        int val = Integer.parseInt(version.substring(0, vpos));
                        int fval = Integer.parseInt(fversion.substring(0, vfpos));
                        if (val < fval) {
                            updateFlag = true;
                            break;
                        } else if (val > fval) {
                            updateFlag = false;
                            break;
                        }
                        if (vpos < version.length()) {
                            version = version.substring(vpos + 1);
                        } else {
                            vpos = -1;
                        }
                        if (vfpos < fversion.length()) {
                            fversion = fversion.substring(vfpos + 1);
                        } else {
                            vfpos = -1;
                        }
                    }
                } while (vpos > 0 && vfpos > 0);

                if (updateFlag) {
                    updateOTA = (String) bb.get(OTA_TAG);
                    updateURL = (String) bb.get(URL_TAG);
                    updateRequired = !((String) bb.get(REQUIRED_TAG)).equals("0");
                }
            } else {
                Log.info("BlackBerry info set is null");
            }
            output = null;
            bb = null;
        } catch (ClassCastException cce) {
            Log.info(cce.toString());
        } catch (CodedException ce) {
            Log.info(ce.toString());
        } catch (Throwable t) {
            Log.info(t.toString());
        }

        Log.info("updateFlag [" + updateFlag + "] updateOTA [" + updateOTA
                + "] updateURL [" + updateURL + "] updateRequired [" + updateRequired + "]");

        if (updateFlag && updateRequired) {
            return true;
        }
        return false;
    }

    /**
     * Returns the current status of the update flag
     * 
     * @return True if there is an update available.
     */
    public boolean isUpdate() {
        return updateFlag;
    }

    /**
     * 
     * @param ota
     * @return
     */
    public String getUpdateUrl(boolean ota) {
        if (ota) {
            return updateOTA;
        } else {
            return updateURL;
        }
    }

}
