/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */



package com.funambol.blackberry.controller;

import net.rim.device.api.i18n.Locale;
import net.rim.device.api.system.Branding;
import net.rim.device.api.system.DeviceInfo;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.Updater;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.ui.AboutScreen;
import com.funambol.blackberry.ui.ConfigurationScreen;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.ui.HelpScreen;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.blackberry.ui.LoginScreen;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.util.ErrorLog;
import com.funambol.util.HttpTransportAgent;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;

public class Controller {

    private final SyncClient     client;

    private final DisplayManager displayManager;

    // Constructor
    public Controller(SyncClient client, ControllerDataFactory fact) {

        fact.setController(this);

        errors = new ErrorLog();

        this.displayManager = fact.getDisplayManager();

        HomeScreen homeScreen = fact.getHomeScreen();
        ConfigurationScreen configScreen = fact.getConfigurationScreen();
        AboutScreen aboutScreen = fact.getAboutScreen();
        HelpScreen helpScreen = fact.getHelpScreen();
        LoginScreen loginScreen = fact.getLoginScreen();

        this.client = client;

        syncStatusController = new SyncStatusController(homeScreen);
        screenDisplayController = new ScreenDisplayController(displayManager, homeScreen,
                configScreen, aboutScreen, helpScreen, loginScreen);
        synchronizationController = new SynchronizationController(this, client);
        homeScreenController = new HomeScreenController(this, syncStatusController, homeScreen);
        dialogController = new DialogController(displayManager);

        homeScreen.initialize(homeScreenController);
    }

    private SyncStatusController      syncStatusController;

    private SynchronizationController synchronizationController;

    private ScreenDisplayController   screenDisplayController;

    private HomeScreenController      homeScreenController;

    private DialogController          dialogController;

    private final ErrorLog            errors;

    /**
     * @return the syncStatusController
     */
    public SyncStatusController getSyncStatusController() {
        return syncStatusController;
    }

    /**
     * @return the ScreenDisplayController
     */
    public ScreenDisplayController getScreenDisplayController() {
        return screenDisplayController;
    }

    /**
     * @return the HomeScreenController
     */
    public HomeScreenController getHomeScreenController() {
        return homeScreenController;
    }

    /**
     * @return the SynchronizationController
     */
    public SynchronizationController getSynchronizationController() {
        return synchronizationController;
    }

    /**
     * @return the DialogController;
     */
    public DialogController getDialogController() {
        return dialogController;
    }

    // Misc interface functions

    public void toForeground() {
        displayManager.toForeground();
    }

    public void toBackground() {
        displayManager.toBackground();
    }

    public boolean isUpdate() {
        return client.getUpdater().isUpdate();
    }

    public void reapplyConfiguration() {
        homeScreenController.updateHomeScreen();

        // TODO FIXME: what is this for?
        //client.initializeSourceConfigs(false);
        //////////////////////////////

        if (Configuration.enablePollingPIM) {
            client.enablePollingPIM();
        } else {
            client.disablePollingPIM();
        }

        // TODO FIXME
        Log.error("Setting log level to: " + Configuration.logLevel);
        Log.setLogLevel(Configuration.logLevel);

        client.setEnableBlockInvites(Configuration.blockIncomingInvites);
    }

    /**
     * Called when user selected Download Update from the menu
     */
    public void promptUpdate() {
        Updater updater = client.getUpdater();
        updater.checkUpdate();
        if (updater.isUpdate()) {
            synchronized (SyncClient.getEventLock()) {
                boolean result = dialogController.askYesNoQuestion(StaticDataHelper
                        .getLanguage("dialog_update"), true);
                if (result) {
                    displayManager.loadBrowser(updater.getUpdateUrl(true));
                } else {
                    dialogController.showMessage(StaticDataHelper
                            .getLanguage("message_update-location")
                            + ":\n" + updater.getUpdateUrl(true));
                }
            }
        }
    }

    public boolean checkForUpdate() {

        if (Customization.getInstance().checkForUpdates()) {
            if (client.getUpdater().checkUpdate()) {
                getDialogController().showMessage(
                        StaticDataHelper.getLanguage("message_update-required"));
                toForeground();
                return true;
            } else if (client.getUpdater().isUpdate()) {
                getDialogController().showMessage(
                        StaticDataHelper.getLanguage("message_update-available"));
                toForeground();
            }
        }
        return false;
    }

    public SourceConfig getSourceConfig(int x) {
        return client.getSourceConfig(x);
    }

    public int numErrors() {
        return errors.numErrors();
    }

    public void clearErrors() {
        errors.clearErrors();
    }

    public String getErrors() {
        return errors.getErrors();
    }

    public void error(String message) {
        Log.error(message);
        errors.addError(message);
    }

    public void sendLogs() {

        Thread sendLogThread = new Thread() {

            public void run() {

                // We need to temporarily disable logging while we do this, so
                // that
                // we dont capture the stuff logged during the send log

                int oldLevel = Log.getLogLevel();
                Log.setLogLevel(Log.DISABLED);

                String logData = client.getLogContent();

                String username = Configuration.username;

                String result = "";

                if (logData == null) {
                    result = StaticDataHelper.getLanguage("sendlog_no-logdata");
                } else if (username == null) {
                    result = StaticDataHelper.getLanguage("sendlog_no-username");
                } else {

                    HttpTransportAgent http = new HttpTransportAgent(StaticDataHelper
                            .getLanguage("sendlog_post-url"),
                            Configuration.getSyncConfig().userAgent, "UTF-8", false, false);
                    StringBuffer message = new StringBuffer();
                    message.append("Username:").append(username).append("\n");
                    message.append("Client:").append("Blackberry").append("\n");
                    message.append("Device:").append(Configuration.getSyncConfig().deviceConfig.devID).append("\n");
                    message.append("OS:").append(StaticDataHelper.getOS()).append("\n");
                    message.append("Locale:").append(Locale.getDefaultForSystem().getCountry())
                            .append("\n");
                    message.append("Model:").append(DeviceInfo.getDeviceName()).append("\n");
                    message.append("Vendor:").append(Branding.getVendorId()).append("\n");
                    message.append("Manufacturer:").append(DeviceInfo.getManufacturerName())
                            .append("\n");
                    message.append("\n");
                    message.append(logData);
                    try {
                        result = http.sendMessage(message.toString());
                    } catch (Exception e) {
                        result = StaticDataHelper.getLanguage("sendlog_cannot-send");
                    }
                }

                Log.setLogLevel(oldLevel);

                final String alert = result;

                getDialogController().showMessage(alert);
            }
        };

        sendLogThread.start();
    }
}
