/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Enumeration;
import net.rim.device.api.i18n.DateFormat;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.SyncListener;
import com.funambol.util.Log;

public class HomeScreenController {

    Controller           controller;

    SyncStatusController syncStatusController;

    HomeScreen           homeScreen;

    SyncClient           client;

    HomeScreenController(Controller controller, SyncStatusController syncStatusController,
            HomeScreen homeScreen) {
        this.controller = controller;
        this.homeScreen = homeScreen;
        this.syncStatusController = syncStatusController;
    }

    public void setEnabled(int syncMask, boolean enabled) {
        homeScreen.setEnabled(syncMask, enabled);

    }

    public void syncStarted(int syncMask) {
        lockHomeScreen(syncMask);
        homeScreen.syncStarted(syncMask);
    }

    public void sourceStarted(int syncMask) {
        homeScreen.sourceStarted(syncMask);
    }

    public void sourceEnded(int syncMask) {
        homeScreen.sourceEnded(syncMask);
    }

    public void syncEnded() {
        homeScreen.syncEnded();
        unlockHomeScreen();
    }

    public void redraw() {
        homeScreen.redraw();
    }

    private void lockHomeScreen(int syncMask) {
        setEnabled(0, false);
        setEnabled(syncMask, true);
        setEnabled(~syncMask, false);
        redraw();
    }

    public void updateHomeScreen() {

        Log.trace("[HomeScreenController.updateHomeScreen]");

        if (controller.getSynchronizationController().isSynchronizing()) {
            return;
        }
        
        unlockHomeScreen();

        long anchor;
        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            int           id        = appSource.getId();

            Log.trace("Setting status for " + id);

            if (!appSource.isEnabled()) {
                Log.trace("disabled");
                syncStatusController.setSyncDisabled(id, StaticDataHelper
                            .getLanguage("home_disabled"));
            } else {
                Log.trace("enabled");
                if (appSource.getLastSyncStatus() == SyncListener.SUCCESS) {
                    Log.debug("Last sync was successfull");
                    anchor = appSource.getSyncSource().getNextAnchor();
                    if (anchor > 0) {
                        syncStatusController.setSyncStatus(id, StaticDataHelper
                                        .getLanguage("home_last-sync")
                                        + " " + createLastSyncedString(anchor));
                    } else {
                        syncStatusController.setSyncStatus(id, StaticDataHelper
                                        .getLanguage("home_unsynchronized"));
                    }
                }
            }
        }

        redraw();
    }

    private void unlockHomeScreen() {
        boolean flag;

        setEnabled(0, true);

        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            setEnabled(appSource.getId(), appSource.isEnabled());
        }

        redraw();
    }

    public void syncMenuSelected(int syncFlag) {

        controller.getSynchronizationController().synchronize(Constants.SYNC_TYPE_MANUAL, syncFlag);

    }

    public void syncMenuSelected() {
        syncMenuSelected(AppSyncSourceManager.SYNC_FLAG_ALL);

    }

    public void refreshMenuSelected() {

        if (!controller.getDialogController().promptNext(
                StaticDataHelper.getLanguage("dialog_refresh_warn"))) {
            return;
        }

        int direction = controller.getDialogController().showRefreshDirectionDialog();
        if (direction > 0) {
            int mask = controller.getDialogController().showRefreshTypeDialog(direction);
            if (mask > 0) {
                controller.getScreenDisplayController().showHomeScreen();
                controller.getSynchronizationController().refresh(mask, direction);
            }
        }

    }

    public void cancelMenuSelected() {

        controller.getSynchronizationController().cancelSync();

    }

    public void updateMenuSelected() {
        controller.promptUpdate();

    }

    public void helpMenuSelected() {
        controller.getScreenDisplayController().showHelpScreen();
    }

    public void aboutMenuSelected() {
        controller.getScreenDisplayController().showAboutScreen();
    }

    public void quitMenuSelected() {
        controller.toBackground();
    }

    public void configMenuSelected() {

        controller.getScreenDisplayController().showConfigScreen();
    }

    public void errorMenuSelected() {
        controller.getDialogController().showMessage(getErrors());
    }

    public boolean isUpdate() {
        return controller.isUpdate();
    }

    public boolean isSynchronizing() {
        return controller.getSynchronizationController().isSynchronizing();
    }

    public int numErrors() {
        return controller.numErrors();
    }

    public void sendLogs() {
        controller.sendLogs();
    }

    private String createLastSyncedString(long anchor) {
    
        StringBuffer sb = new StringBuffer();
        long now = System.currentTimeMillis();
        long aday = 24 * 60 * 60 * 1000;
        long yesterday = now - aday;
        DateFormat formatDateShort = DateFormat.getInstance(DateFormat.DATE_SHORT);
        String todayDate = formatDateShort.formatLocal(now);
        String yesterdayDate = formatDateShort.formatLocal(yesterday);
        String anchorDate = formatDateShort.formatLocal(anchor);
    
        if (anchorDate.equals(todayDate)) {
            sb.append(StaticDataHelper.getLanguage("word_today"));
        } else if (anchorDate.equals(yesterdayDate)) {
            sb.append(StaticDataHelper.getLanguage("word_yesterday"));
        } else {
            sb.append(DateFormat.getInstance(DateFormat.DATE_LONG).formatLocal(anchor));
        }
    
        sb.append(" ").append(StaticDataHelper.getLanguage("word_at")).append(" ");
    
        String time = DateFormat.getInstance(DateFormat.TIME_MEDIUM).formatLocal(anchor);
        int delimiter = time.indexOf(":");
    
        // Just to be safe
        if (delimiter >= 0) {
            String hour = time.substring(0, delimiter);
            String minute = time.substring(delimiter + 1);
            // Time can be in 12 or 24 hours. In 12 hours mode the time
            // contains already the ampm info, so we shall not re-add it
            String ampm = "";
            if (!minute.endsWith("M")) {
                if (Integer.parseInt(hour) > 12) {
                    ampm = StaticDataHelper.getLanguage("word_pm");
                    hour = "" + (Integer.parseInt(hour) - 12);
                } else {
                    // To remove possible leading '0'
                    ampm = StaticDataHelper.getLanguage("word_am");
                    hour = "" + (Integer.parseInt(hour));
                }
            }
            sb.append(hour).append(":").append(minute).append(" ").append(ampm);
    
        } else {
            sb.append(time);
        }
    
        return sb.toString();
    }

    private String getErrors() {
        return controller.getErrors();
    }
}
