package com.funambol.blackberry.controller;

import com.funambol.blackberry.ui.AboutScreen;
import com.funambol.blackberry.ui.ConfigurationScreen;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.ui.HelpScreen;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.blackberry.ui.LoginScreen;
import com.funambol.blackberry.ui.Screen;
import com.funambol.util.Log;

public class ScreenDisplayController {

    private final DisplayManager      displayManager;

    private final HomeScreen          homeScreen;

    private final ConfigurationScreen configScreen;

    private final AboutScreen         aboutScreen;

    private final HelpScreen          helpScreen;

    private final LoginScreen         loginScreen;

    ScreenDisplayController(DisplayManager manager, HomeScreen homeScreen,
            ConfigurationScreen configScreen, AboutScreen aboutScreen, HelpScreen helpScreen,
            LoginScreen loginScreen) {
        this.displayManager = manager;
        this.homeScreen = homeScreen;
        this.configScreen = configScreen;
        this.aboutScreen = aboutScreen;
        this.helpScreen = helpScreen;
        this.loginScreen = loginScreen;
    }

    // Show functions
    public void showScreen(Screen scr) {
        showScreen(scr, false);
    }

    public void showScreen(Screen scr, boolean donotwait) {
        try {
            displayManager.showScreen(scr, donotwait);
        } catch (Exception e) {
            Log.error("Could not show screen " + scr.getClass().getName());
        }
    }

    public void showAboutScreen() {
        showScreen(aboutScreen);
    }

    public void showConfigScreen() {
        showScreen(configScreen);
    }

    public void showHelpScreen() {
        showScreen(helpScreen);
    }

    public void showHomeScreen() {
        showHomeScreen(false);
    }

    public void showHomeScreen(boolean donotwait) {
        showScreen(homeScreen, donotwait);
    }

    public void showLoginScreen() {
        showLoginScreen(false);
    }

    public void showLoginScreen(boolean donotwait) {
        showScreen(loginScreen, donotwait);
    }

    // Hide functions
    public void hideScreen(Screen scr) {
        try {
            displayManager.hideScreen(scr);
        } catch (Exception e) {
            Log.error("Could not hide screen " + scr.getClass().getName());
        }
    }

    public void hideAboutScreen() {
        hideScreen(aboutScreen);
    }

    public void hideConfigScreen() {
        hideScreen(configScreen);
    }

    public void hideHelpScreen() {
        hideScreen(helpScreen);
    }

    public void hideHomeScreen() {
        hideScreen(homeScreen);
    }

    public void hideLoginScreen() {
        hideScreen(loginScreen);
    }
}
