/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Vector;
import java.util.Enumeration;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.SyncEngine;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.Customization;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.util.ConnectionListener;
import com.funambol.util.Log;
import com.funambol.util.SyncListener;
import com.funambol.util.StaticDataHelper;

public class SynchronizationController implements ConnectionListener {

    private Controller controller;

    private SyncClient client;

    private SyncEngine engine;

    private boolean    synchronizing;

    // Used to select which source to sync in synchronize()

    private boolean    doCancel        = false;

    private String     syncType        = null;

    private AppSyncSource currentSource = null;

    private boolean    showTCPAlert;

    private boolean    logConnectivityError;

    private int        slowAborted;

    SynchronizationController(Controller controller, SyncClient client) {
        this.controller = controller;
        this.client = client;
        this.engine = new SyncEngine(this);;
    }

    public SyncStatusController getSyncStatusController() {
        return controller.getSyncStatusController();
    }

    public boolean isSynchronizing() {
        return synchronizing;
    }

    public void cancelSync() {
        setCancel(true);

        if (isSynchronizing() && currentSource != null) {
            int id = currentSource.getId();
            controller.getSyncStatusController().setSyncStatus(id,
                    StaticDataHelper.getLanguage("status_cancelling"));
        }
    }

    private int getMaskFromSources(Vector sources) {
        int mask = 0;
        for (int x = 0; x < sources.size(); x++) {
            AppSyncSource appSource = (AppSyncSource) sources.elementAt(x);
            mask |= appSource.getId();
        }
        return mask;
    }

    public synchronized void refresh(int syncMask, int direction) {

        if (synchronizing) {
            return;
        }

        int syncMode = 0;
        switch (direction) {
            case Constants.REFRESH_FROM_SERVER:
                syncMode = SyncML.ALERT_CODE_REFRESH_FROM_SERVER;
                break;
            case Constants.REFRESH_TO_SERVER:
                syncMode = SyncML.ALERT_CODE_REFRESH_FROM_CLIENT;
                break;
            default:
                return;
        }

        Enumeration sources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        Vector syncSources = new Vector();
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) sources.nextElement();
            if ((syncMask & appSource.getId()) != 0) {
                SyncSource source = appSource.getSyncSource();
                source.getConfig().setSyncMode(syncMode);
                syncSources.addElement(appSource);
            }
        }

        synchronizing = engine.synchronize(syncSources);

        if (!synchronizing) {
            controller.getHomeScreenController().syncEnded();
        }
    }

    public synchronized void synchronize(String syncType, int syncMask) {

        Log.info("[SynchronizationController.synchronize]" + syncType + "," + syncMask);

        if (synchronizing) {
            Log.info("A sync is already in progress");
            return;
        }

        this.syncType = syncType;
        int syncMode = 0;

        Enumeration sources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        Vector      syncSources = new Vector();
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) sources.nextElement();
            // Filter out sources
            if ((syncMask & appSource.getId()) != 0) {
                SyncSource source = appSource.getSyncSource();
                syncMode = Configuration.getSyncMode();
                // If this source has no config set, then we cannot force a sync
                // mode, but this is a logical error
                if (source.getConfig() != null) {
                    source.getConfig().setSyncMode(syncMode);
                    syncSources.addElement(appSource);
                } else {
                    Log.error("Source has no config, cannot set sync mode");
                }
            }
        }

        slowAborted = -1;

        synchronizing = engine.synchronize(syncSources);

        if (!synchronizing) {
            controller.getHomeScreenController().syncEnded();
        }
    }

    public boolean confirmSlowSync(int sourceMask) {

        Enumeration sources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        String sourceNames = getListOfSourceNames(sources).toLowerCase();

        StringBuffer question = new StringBuffer();
        question.append(StaticDataHelper.getLanguage("dialog_slow-text1")).append(" ").append(
                sourceNames).append(StaticDataHelper.getLanguage("dialog_slow-text2"));

        return controller.getDialogController().askYesNoQuestion(question.toString(), true, 20000);
    }

    public void abortSlow(int sourceMask) {
        slowAborted = sourceMask;
    }

    public boolean isAbortedSlow(AppSyncSource appSource) {
        return (appSource.getId() == slowAborted);
    }

    public boolean confirmDeletes(Enumeration sourceNameList) {

        String sourceNames = getListOfSourceNames(sourceNameList).toLowerCase();

        Log.info("Prompting user for delete confirmation");

        String message = StaticDataHelper.getLanguage("dialog_delete1") + " " + sourceNames
                + StaticDataHelper.getLanguage("dialog_delete2");

        boolean result = controller.getDialogController().askYesNoQuestion(message, false);

        if (result) {
            Log.info("Continuing with sync");
        } else {
            Log.info("User opted to cancel sync - " + sourceNames);
        }

        return result;
    }

    public SyncConfig getSyncConfig() {
        return Configuration.getSyncConfig();
    }

    // ConnectionListener implementation
    
    public boolean isConnectionConfigurationAllowed(final String apn) { 
        String message = StaticDataHelper.getLanguage("message_APN-question-1") + " " + apn + " "
                + StaticDataHelper.getLanguage("message_APN-question-2");
        return controller.getDialogController().askAcceptDenyQuestion(message, true);
    }

    public void noCredentials() {
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_login-required"));

    }

    public void noSources() {
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_nothing-to-sync"));

    }

    public void noConnection() {
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_radio-off"));

    }

    public void noSignal() {
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_no-signal"));

    }

    public void setCancel(boolean value) {
        doCancel = value;
    }

    /**
     * Check if the current sync should be cancelled
     * 
     * @return
     */
    public boolean isCancelled() {
        return doCancel;
    }

    public boolean syncStarted(Vector sources) {
        int syncMask = getMaskFromSources(sources);

        controller.getHomeScreenController().syncStarted(syncMask);

        if (Customization.getInstance().checkForUpdates()) {
            controller.getSyncStatusController().setSyncStatus(
                    StaticDataHelper.getLanguage("home_check-update"));
            boolean isRequired = controller.checkForUpdate();
            controller.getSyncStatusController().setSyncStatus(null);
            if (isRequired) {
                return false;
            }
        }

        return true;

    }

    public void syncEnded(Vector failedSources, String errorMessage) {
        syncType = null;
        synchronizing = false;

        controller.getHomeScreenController().syncEnded();
        controller.getHomeScreenController().updateHomeScreen();
        if(Customization.getInstance().enableUpdaterManager()){
            UpdaterManager upm = UpdaterManager.getInstance();
            upm.setController(controller);
            upm.check();

        }
    }

    /*
    public void syncFailed(Vector failedSources) {
        syncFailed(failedSources, null);
    }
    */

    public void syncComplete(Vector sources, boolean hadErrors) {
        if (hadErrors) {

            controller.getScreenDisplayController().showHomeScreen();
            controller.toForeground();

            if (showTCPAlert) {
                Log.debug("showing tcp settings alert!");
                controller.getDialogController().showMessage(
                        StaticDataHelper.getLanguage("message_enter-TCP-settings"));
            }
        }

        if (logConnectivityError) {
            if (Constants.SYNC_TYPE_MANUAL.equals(syncType)) {
                Log.error("Manual sync: Connection attempts failed.");
                controller.error(StaticDataHelper.getLanguage("message_connectivity"));
                logConnectivityError = false;
            } else if (Constants.SYNC_TYPE_SCHEDULED.equals(syncType)) {

                if (!engine.retry()) {
                    controller.error(StaticDataHelper.getLanguage("message_connectivity"));
                    logConnectivityError = false;
                }
            }
        }
    }

    public void sourceStarted(AppSyncSource appSource) {
        currentSource = appSource;
        int id = appSource.getId();
        controller.getHomeScreenController().sourceStarted(id);
    }

    public void sourceEnded(AppSyncSource appSource) {

        slowAborted = -1;
        saveSourceConfig(appSource);
        currentSource = null;

    }

    public void sourceCancelled(AppSyncSource appSource) {

        Vector sources = new Vector();
        sources.addElement(appSource);
        appSource.setLastSyncStatus(SyncListener.CANCELLED);
        //lastFailedMask |= getMaskFromSources(sources);

        controller.getSyncStatusController().setSyncFailed(appSource.getId(),
                StaticDataHelper.getLanguage("status_cancelled"));

    }

    public void sourceFailed(AppSyncSource appSource, SyncException e) {

        Log.trace("SynchronizationController.sourceFailed");

        int code = e.getCode();
        if (   code == SyncException.READ_SERVER_RESPONSE_ERROR
            || code == SyncException.WRITE_SERVER_REQUEST_ERROR
            || code == SyncException.CONN_NOT_FOUND) {

            logConnectivityError = true;
        } else if (code == SyncException.CONNECTION_BLOCKED_BY_USER) {
            showTCPAlert = true;
        }
    }

    public String getRemoteUri(AppSyncSource appSource) {
        SourceConfig config = appSource.getSyncSource().getConfig();
        return config.getRemoteUri();
    }

    public void serverOperationFailed() {

        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_not-send-to-server"));
        controller.error(StaticDataHelper.getLanguage("message_not-send-to-server"));

    }

    public void clearErrors() {
        controller.clearErrors();
    }

    public void error(String message) {
        controller.error(message);
    }

    /*
    public boolean didLastSyncFail(int syncMask) {
        return (lastFailedMask & syncMask) != 0;
    }
    */

    private String getListOfSourceNames(Enumeration sourceNameList) {
        StringBuffer sourceNames = new StringBuffer();

        int x = 0;
        AppSyncSource appSource = (AppSyncSource)sourceNameList.nextElement();

        while(appSource != null) {

            String name = appSource.getName();
            appSource = (AppSyncSource)sourceNameList.nextElement();

            if (x > 0) {
                sourceNames.append(", ");
                if (appSource == null) {
                    sourceNames.append(StaticDataHelper.getLanguage("dialog_and").toLowerCase());
                }
            }

            sourceNames.append(name);
        }

        return sourceNames.toString();
    }

    private void saveSourceConfig(AppSyncSource appSource) {

        SyncSource source = appSource.getSyncSource();
        String sscKey = appSource.getConfigStorageKey();
        if (sscKey != null) {
            Configuration.saveSourceConfig(sscKey, source.getConfig());
        } else {
            Log.error("Unable to save source - no key!");
        }
    }
    

    public void connectionOpened() {
        Log.debug("Connection opened");
    }

    public void requestWritten() {
        Log.debug("Request written");
    }

    public void responseReceived() {
        Log.debug("Response received");
    }

    public void connectionClosed() {
        Log.debug("Connection closed");
    }

    public void connectionConfigurationChanged() {
        Log.debug("Configuration changed");
    } 
}
