/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.SyncEngine;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.util.ThreadPool;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.AppProperties;
import com.funambol.util.StaticDataHelper;
import com.funambol.updater.Updater;
import com.funambol.updater.BasicUpdaterConfig;
import com.funambol.updater.UpdaterListener;
import java.util.Date;
import net.rim.blackberry.api.browser.Browser;



/**
 * This class is the client interface to the automatic update
 * library.
 */
public class UpdaterManager implements UpdaterListener {

    private static String componentName  = "bbpim";
    private Updater updater;
    private static UpdaterManager instance = null;
    private UpdaterManagerConfig config;
    private long checkInterval = (long)24 * (long)60 * (long)60 * (long)1000;   // 1 day in milliseconds
    private long reminderInterval = (long)2 * (long)60 * (long)60 * (long)1000; // 2 hours in seconds
    private Controller controller;

    private final int UPDATE_NOW_VALUE = 0;
    private final int UPDATE_LATER_VALUE = 1;
    private final int SKIP_UPDATE_VALUE = 2;

    private final String UPDATE_NOW_LABEL = StaticDataHelper.getLanguage("update_command-now");
    private final String UPDATE_LATER_LABEL = StaticDataHelper.getLanguage("update_command-later");
    private final String SKIP_UPDATE_LABEL = StaticDataHelper.getLanguage("update_command-skip");

    private final String UPDATE_POPUP_MESSAGE = StaticDataHelper.getLanguage("message_update-available");
    

    public static UpdaterManager getInstance() {
        if (instance == null) {
            instance = new UpdaterManager();
        }
        return instance;
    }

    public void setController(Controller controller){
        this.controller = controller;
    }

    class UpdaterThread extends Thread {
        public UpdaterThread() {
        }

        public void run() {
            updater.check();
        }
    }

    class UpdaterManagerConfig extends BasicUpdaterConfig {

       

        public void load() {
            try {
                Configuration.load();
            } catch (Exception ce) {
                Log.error("Loading updater config failed " + ce);
                // We shall create a new config
                String server = Customization.getInstance().getServerUriDefault();
                server = StringUtil.extractAddressFromUrl(server);
                
                Log.debug("Server url: "+server);
                super.setUrl(server);
                checkInterval = Customization.getInstance().getCheckUpdtIntervalDefault();
                setCheckInterval(checkInterval);
                reminderInterval = Customization.getInstance().getReminderUpdtIntervalDefault();
                setReminderInterval(reminderInterval);
    
            }
            String server = Configuration.syncUrl;
            server = StringUtil.extractAddressFromUrl(server);

            Log.debug("Server url: "+server);
            setUrl(server);
            setDownloadUrl(Configuration.downloadUrl);
            setType(Configuration.updatetype);
            setAvailableVersion(Configuration.availableVersion);
            setLastCheck(Configuration.lastUpdateCheck);
            setCheckInterval(Configuration.checkInterval);
            setReminderInterval(Configuration.reminderInterval);
            setLastReminder(Configuration.lastReminder);
            setSkip(Configuration.skip);
       
        }

        public void save() {
            // save the configuration
            try {

                Configuration.downloadUrl = getDownloadUrl();
                Configuration.updatetype = getType();
                Configuration.availableVersion = getAvailableVersion();
                Configuration.lastUpdateCheck = getLastCheck();
                Configuration.checkInterval = getCheckInterval();
                Configuration.reminderInterval = getReminderInterval();
                Configuration.lastReminder = getLastReminder();
                Configuration.skip = getSkip();
                Configuration.save(controller);
            } catch (Exception ce) {
                Log.error("Saving updater config failed " + ce);
            }
        }
    }

    private UpdaterManager() {
        String version = BBVersion.VERSION;
        config = new UpdaterManagerConfig();
        config.load();
        String [] osVersion = StaticDataHelper.getOSVersion();
        int os = Integer.parseInt(osVersion[0] + osVersion[1]);
        if(os > 46){
            componentName = componentName+"_os47";
        }
        
        updater = new Updater(config, version, componentName);
        updater.setListener(this);
    }

    public void check() {
        // The update check is executed in a separate thread
        UpdaterThread updThread = new UpdaterThread();
        updThread.setPriority(Thread.MIN_PRIORITY);
        updThread.start();
    }

    public void mandatoryUpdateAvailable(String version) {
        // In this client an update is available only if we have a new
        // version and a valid url.
        String url = getDownloadUrl();
        if (url != null && url.length() > 0) {
            
            DialogController dialControll = controller.getDialogController();
            int selection = dialControll.promptSelection(UPDATE_POPUP_MESSAGE,
                                                    new String[]{UPDATE_NOW_LABEL,UPDATE_LATER_LABEL},
                                                    new int[]{UPDATE_NOW_VALUE,UPDATE_LATER_VALUE},
                                                    UPDATE_NOW_VALUE,
                                                    true,
                                                    DisplayManager.NO_LIMIT);
            if (selection == UPDATE_NOW_VALUE){
                this.update();
            }else{
                //selected UPDATE_LATER_VALUE
                this.userSelectedLater();
            }
        }
    }

    public void optionalUpdateAvailable(String version) {
        // In this client an update is available only if we have a new
        // version and a valid url.
        String url = getDownloadUrl();
        if (url != null && url.length() > 0) {
            DialogController dialControll = controller.getDialogController();
            int selection = dialControll.promptSelection(UPDATE_POPUP_MESSAGE,
                                                    new String[]{UPDATE_NOW_LABEL,UPDATE_LATER_LABEL,SKIP_UPDATE_LABEL},
                                                    new int[]{UPDATE_NOW_VALUE,UPDATE_LATER_VALUE,SKIP_UPDATE_VALUE},
                                                    UPDATE_NOW_VALUE,
                                                    true,
                                                    DisplayManager.NO_LIMIT);
            if (selection == UPDATE_NOW_VALUE){
                this.update();
            }else if (selection == UPDATE_LATER_VALUE){
                //selected UPDATE_LATER_VALUE
                this.userSelectedLater();
            }else{
                //selected SKIP_UPDATE_VALUE
                this.userSelectedSkip();
            }
            
        }
    }

    public void updateLastReminder() {
        long now = System.currentTimeMillis();
        updater.setLastReminder(now);
    }

    public void userSelectedLater() {
        Log.info("UpdaterManager - updater later");
        long time = System.currentTimeMillis();
        System.out.println("Update Later has been setted at: " + new Date(time+ config.getReminderInterval()));
        this.config.setLastReminder(time);
        config.save();
    }

    public void userSelectedSkip() {
        Log.info("UpdaterManager - skip updater");
        updater.setSkip();
    }

    public boolean isUpdateAvailable() {
        // In this client an update is available only if we have a new
        // version and a valid url.
        String url = getDownloadUrl();
        if (url != null && url.length() > 0) {
            return updater.isUpdateAvailable();
        } else {
            return false;
        }
    }

    public String getDownloadUrl() {
        return config.getDownloadUrl();
    }

    public void setUrl(String url) {
        config.setUrl(url);
        // Since a new url has been set, we also reset our timers to force a new
        // check and possibly a new reminder
        config.setLastReminder(0);
        config.setLastCheck(0);
        config.save();
    }

    public void update(){
        Log.info("UpdaterManager - update client from : "+config.getDownloadUrl());
        Browser.getDefaultSession().displayPage(config.getDownloadUrl());
    }
}

