/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.source.AdditionalItemsAttributes;

public abstract class EventHelper extends PIMItemHelper
{

    public static final long        SECOND_FACTOR       = 1000;
    public static final long        MINUTE_FACTOR       = 60*EventHelper.SECOND_FACTOR;
    public static final long        HOUR_FACTOR         = 60*EventHelper.MINUTE_FACTOR;
    public static final long        DAY_FACTOR          = 24*EventHelper.HOUR_FACTOR;
    public static final long        MONTH_FACTOR        = 30*EventHelper.DAY_FACTOR;
    
    protected EventMailListener mailListener            = null;

    protected long limitPast;
    protected long limitFuture;
    
    public void init()
    {
        if (mailListener != null) {
            mailListener.attach();
            mailListener.unblockAllOutgoingSubjects();
        }
    }
    
    public void setIncomingMailListenerStatus(boolean enabled)
    {
        if (mailListener != null) {
            if (enabled)
                mailListener.enableBlockIncoming();
            else
                mailListener.disableBlockIncoming();
        }
    }
    
    public EventHelper(PIMList list, EventMailListener listener, int pastlimit, int futurelimit)
    {
        super(list);
        this.limitPast = pastlimit;
        this.limitFuture = futurelimit;
        
        // Creating EventMailListener to filter emails from events
        mailListener = listener;
        
        if (mailListener != null) {
            mailListener.setHelper(this);
            mailListener.enableBlockOutgoing();
            if (Configuration.blockIncomingInvites) {
                mailListener.enableBlockIncoming();
            }
        }
    }
    
    public EventList getEventList()
    {
        PIMList list = getPIMList();
        if (list != null && list instanceof EventList)
        {
            return (EventList)list;
        }
        return null;
    }
    
    public void handleUpdatedItem(PIMItem item)
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)item;
            mailListener.blockIncomingEvent(
                    event.getString(BlackBerryEvent.SUMMARY, 0),
                    event
                );
        }
    }
    
    public AdditionalItemsAttributes handleAddedItem(PIMItem item)
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)item;
            mailListener.blockIncomingEvent(
                    event.getString(BlackBerryEvent.SUMMARY, 0),
                    event
                );
        }

        return null;
    }

    public void saveNewPIMItem(PIMItem pitem) throws PIMException
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)pitem;
            
            if (isSupportedField(BlackBerryEvent.ATTENDEES))
            {
                int size = event.countValues(BlackBerryEvent.ATTENDEES);
                if (size > 0)
                {
                    mailListener.blockOutgoingEvent(
                        event.getString(BlackBerryEvent.SUMMARY, 0),
                        EventMailListener.NEW);
                }
            }
        }
        
        pitem.commit();
    }

    public void saveUpdatedPIMItem(PIMItem pitem) throws PIMException
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)pitem;
            
            if (isSupportedField(BlackBerryEvent.ATTENDEES))
            {
                int size = event.countValues(BlackBerryEvent.ATTENDEES);
                if (size > 0)
                {
                    mailListener.blockOutgoingEvent(
                            event.getString(BlackBerryEvent.SUMMARY, 0),
                        EventMailListener.UPD);
                }
            }
        }
            
        pitem.commit();
    }
    
    
    public boolean deleteItem(PIMItem item) throws PIMException
    {
        EventList list = getEventList();
        if (list != null &&
                item != null &&
                item instanceof Event &&
                isInSyncRange(item))
        {
            if (mailListener != null) {
                mailListener.blockOutgoingEvent(
                        item.getString(BlackBerryEvent.SUMMARY, 0),
                        EventMailListener.DEL);
            }
            list.removeEvent((Event)item);
            return true;
        }
        return false;
    }
    
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#createPIMItem(PIMList pimlist)
     */
    public PIMItem createItem() throws PIMException
    {
        EventList list = getEventList();
        if (list != null)
        {
            return list.createEvent();
        }
        return null;
    }
    
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof BlackBerryEvent)
        {
            try
            {
                return item.getString(BlackBerryEvent.UID,0);
            }
            catch(final Exception e)
            {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Calendar]";
                Log.error(msg);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Calendar] is not a BlackBerryEvent.";
        Log.error(msg);
        throw new PIMException(msg);
    }
}
