/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;

import javax.microedition.pim.Event;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;

import net.rim.blackberry.api.mail.BodyPart;
import net.rim.blackberry.api.mail.Folder;
import net.rim.blackberry.api.mail.Message;
import net.rim.blackberry.api.mail.MessagingException;
import net.rim.blackberry.api.mail.Multipart;
import net.rim.blackberry.api.mail.SendListener;
import net.rim.blackberry.api.mail.ServiceConfiguration;
import net.rim.blackberry.api.mail.Session;
import net.rim.blackberry.api.mail.Store;
import net.rim.blackberry.api.mail.TextBodyPart;
import net.rim.blackberry.api.mail.event.FolderEvent;
import net.rim.blackberry.api.mail.event.FolderListener;
import net.rim.blackberry.api.mail.event.StoreEvent;
import net.rim.blackberry.api.mail.event.StoreListener;
import net.rim.device.api.i18n.DateFormat;
import net.rim.device.api.servicebook.ServiceBook;
import net.rim.device.api.servicebook.ServiceRecord;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.SyncClient;
import com.funambol.util.Appender;
import com.funambol.util.Log;

/**
 * 
 */
public class EventMailListener implements SendListener, FolderListener, StoreListener {

    public static final String NEW    = "";

    public static final String UPD    = "Updated: ";

    public static final String DEL    = "Cancelled: ";

    private Hashtable          blockOutgoingSubjects;

    private Hashtable          blockIncomingEvents;

    private boolean            blockOutgoingEnabled;

    private boolean            blockIncomingEnabled;

    private EventHelper        helper = null;

    private String             inviteReplacementText;

    private String             eventDetailsText;

    private String             lastSentSubject;

    private String             lastBlockedSubject;

    public EventMailListener(String repl) {
        this.inviteReplacementText = repl;

        blockOutgoingEnabled = false;
        blockOutgoingSubjects = new Hashtable();
        blockIncomingEvents = new Hashtable();

        blockIncomingEnabled = false;
        attach();
    }

    public void setHelper(EventHelper helper) {
        this.helper = helper;
    }

    private void reload() {
        // We must reload the config, because this function is called from a
        // different context, and the config might not be loaded cause its all
        // static

        // TODO FIXME: this does not work as everything static here does not
        // work properly
        Log.initLog(SyncClient.getAppender(), Log.INFO);

        Configuration.load();
        if (Configuration.blockIncomingInvites) {
            Log.info("Incoming invite block enabled");
            enableBlockIncoming();
        } else {
            Log.info("Incoming invite block disabled");
            disableBlockIncoming();
        }

        Log.info("Outgoing invite block enabled");

        Log.setLogLevel(Configuration.logLevel);

        enableBlockOutgoing();
    }

    public void batchOperation(StoreEvent e) {

        reload();

        Log.info("EventMailListener: Batch operation received");
    }

    public void messagesAdded(FolderEvent e) {

        reload();

        Log.trace("Messages added");

        Message m = e.getMessage();

        if (m == null) {
            Log.trace("Message is null!");
            return;
        }

        if (Log.getLogLevel() == Log.TRACE) {
            StringBuffer details = new StringBuffer();
            details.append("Mail details: \n");
            details.append("Subject: ").append(m.getSubject()).append("\n");
            details.append("MessageId: ").append(m.getMessageId()).append("\n");
            details.append("MessageType: ").append(m.getMessageType()).append("\n");
            details.append("ContentType: ").append(m.getContentType()).append("\n");
            details.append("BodyText: ").append(m.getBodyText()).append("\n");
            details.append("Size: ").append(m.getSize()).append("\n");
            details.append("Flags: ").append(m.getFlags()).append("\n");
            details.append("Status: ").append(m.getStatus()).append("\n");
            details.append("IsInbound: ").append(m.isInbound()).append("\n");
            details.append("toString: ").append(m.toString()).append("\n");

            Log.trace(details.toString());
        }

        if (m.isInbound()) {

            Log.info("EventMailListener: Incoming message received - " + m.getSubject());

            Object parts = m.getContent();

            if (blockIncomingEnabled) {

                Log.info("Message blocking enabled");

                if (parts == null) {
                    Log.trace("EventMailListener: Message has no parts - returning");
                    return;
                }

                if (parts instanceof Multipart) {

                    Multipart mp = (Multipart) parts;
                    int count = mp.getCount();

                    if (Log.getLogLevel() == Log.TRACE) {
                        StringBuffer details = new StringBuffer();
                        details.append("EventMailListener:\nMultiPart Info : \n").append(
                                "contentType: ").append(mp.getContentType()).append("\n").append(
                                "toString: ").append(mp.toString()).append("\n").append("Count: ")
                                .append(count).append("\n").append("Parent: ").append(
                                        mp.getParent().toString()).append("\n");

                        Log.info(details.toString());
                    }

                    if (count == 0) {
                        Log
                                .info("EventMailListener: Multipart Message has no parts - likely an event");

                        if (isIncomingEventBlocked(m.getSubject())) {
                            Log.info("Attempting to delete event from incoming email: "
                                    + m.getSubject());
                            if (manageBlockedEvent(m.getSubject())) {
                                clearBlockedIncomingEvents();

                                String text = "\n" + inviteReplacementText + "\n\n"
                                        + eventDetailsText;

                                BodyPart txt = new TextBodyPart(mp, text);
                                mp.addBodyPart(txt);
                                try {
                                    m.setContent(mp);
                                } catch (MessagingException ex) {
                                    Log.info("Unable to add new part");
                                }
                            } else {
                                Log.info("Did not remove event");
                            }
                        }
                    } else {
                        Log
                                .info("EventMailListener: Multipart Message has parts - I dont know what to do!");
                    }
                }
            }
        } else {
            Log.trace("Message is not inbound");
        }
        Log.trace("messagesAdded complete");
    }

    public void messagesRemoved(FolderEvent e) {
        // Log.info("--Messages Removed");
    }

    public boolean sendMessage(Message message) {

        reload();

        String subject = message.getSubject();

        Log.info("Attempting to send message with subject " + subject);

        if (blockOutgoingEnabled) {
            Log.info("Outgoing message blocking is enabled");

            boolean block = isOutgoingSubjectBlocked(subject);

            lastSentSubject = subject;
            if (block) {
                Log.info("Blocking outgoing email for subject: " + subject);
                unblockOutgoingSubject(subject);
                if (!message.getFolder().deleteMessage(message)) {
                    Log.info("Unable to delete outgoing email: " + subject);
                }
                return false;
            }
        }

        Log.info("sendMessage complete");

        return true;
    }

    public void enableBlockOutgoing() {
        blockOutgoingEnabled = true;
    }

    public void disableBlockOutgoing() {
        blockOutgoingEnabled = false;
    }

    public void enableBlockIncoming() {
        blockIncomingEnabled = true;
    }

    public void disableBlockIncoming() {
        blockIncomingEnabled = false;
    }

    public void blockOutgoingEvent(String subject, String type) {
        if (type.equals(EventMailListener.NEW) || type.equals(EventMailListener.UPD)
                || type.equals(EventMailListener.DEL)) {
            blockOutgoingSubject(type + subject);
        }
    }

    public void unblockOutgoingEvent(String subject, String type) {
        if (type.equals(EventMailListener.NEW) || type.equals(EventMailListener.UPD)
                || type.equals(EventMailListener.DEL)) {
            unblockOutgoingSubject(type + subject);
        }
    }

    public void unblockAllOutgoingSubjects() {
        synchronized (blockOutgoingSubjects) {
            blockOutgoingSubjects.clear();
        }
    }

    /*
     * private boolean isIncomingTypeBlocked(String type) { if (type == null)
     * return false; boolean ret = false; synchronized(blockIncomingTypes) { if
     * (blockIncomingTypes.contains(type)) { ret = true; } } return ret; }
     */
    /*
     * private boolean isIncomingExtensionBlocked(String filename) { if
     * (filename == null) return false; boolean ret = false;
     * synchronized(blockIncomingExtensions) { int index =
     * filename.indexOf("."); if (index > 0) { String ext =
     * filename.substring(index); if (blockIncomingExtensions.contains(ext)) {
     * ret = true; } } } return ret; }
     */

    private boolean isOutgoingSubjectBlocked(String subject) {
        if (subject == null)
            return false;

        boolean ret = false;
        synchronized (blockOutgoingSubjects) {
            if (blockOutgoingSubjects.containsKey(subject)) {
                ret = true;
            }
        }
        return ret;
    }

    private boolean isIncomingEventBlocked(String subject) {
        Log.info("isIncomingEventBlocked?");
        if (subject == null) {
            Log.info("isIncomingEventBlocked: null subject");
            return false;
        }

        boolean ret = false;
        synchronized (blockIncomingEvents) {
            java.util.Enumeration keys = blockIncomingEvents.keys();

            while (keys.hasMoreElements()) {
                String key = (String) keys.nextElement();
                Log.info("Comparing " + key + " with subject " + subject);
                if (subject.endsWith(key))
                    ;
                {
                    ret = true;
                    break;
                }
            }

            Log.info("isIncomingEventBlocked: No more subjects to check");
        }
        return ret;
    }

    private boolean manageBlockedEvent(String subject) {
        lastBlockedSubject = subject;

        Log.info("Managing blocked event " + subject);

        boolean ret = false;
        synchronized (blockIncomingEvents) {
            PIMItem item = null;
            java.util.Enumeration keys = blockIncomingEvents.keys();
            while (keys.hasMoreElements()) {
                String key = (String) keys.nextElement();
                if (subject.endsWith(key))
                    ;
                {
                    item = (PIMItem) blockIncomingEvents.get(key);
                    break;
                }
            }

            eventDetailsText = new String("");

            if (item != null) {
                Calendar temp;
                if (helper.isSupportedField(Event.SUMMARY)) {
                    try {
                        eventDetailsText += "Subject: " + item.getString(Event.SUMMARY, 0) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if (helper.isSupportedField(Event.LOCATION)) {
                    try {
                        eventDetailsText += "Location: " + item.getString(Event.LOCATION, 0) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if ((temp = helper.getFieldDate(item, Event.START, false, false, false)) != null) {
                    try {
                        eventDetailsText += "Start: "
                                + DateFormat.getInstance(
                                        DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT).format(
                                        new Date(PIMItemHelper.convertGMTToLocal(PIMItemHelper
                                                .getTimestamp(temp)))) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if ((temp = helper.getFieldDate(item, Event.END, false, false, false)) != null) {
                    try {
                        eventDetailsText += "End: "
                                + DateFormat.getInstance(
                                        DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT).format(
                                        new Date(PIMItemHelper.convertGMTToLocal(PIMItemHelper
                                                .getTimestamp(temp)))) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if (helper.isSupportedField(Event.NOTE)) {
                    try {
                        eventDetailsText += "Notes: " + item.getString(Event.NOTE, 0) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if (((Event) item).getRepeat() != null)
                    eventDetailsText += "This event is recurring." + "\n";

                Log.info("Attemptng to delete event");
                try {
                    helper.deleteItem(item);
                    ret = true;
                } catch (PIMException e) {
                    String msg = helper.handlePIMException(e);
                    Log.error("Error removing event from email: " + msg);
                }
            }
        }
        return ret;
    }

    private void clearBlockedIncomingEvents() {
        synchronized (blockIncomingEvents) {
            Log.info("Clearing blocked incoming event list");
            blockIncomingEvents.clear();
        }
    }

    private void blockOutgoingSubject(String subject) {
        synchronized (blockOutgoingSubjects) {
            Log.debug("Registering subject to be blocked: " + subject);
            if (blockOutgoingSubjects.containsKey(subject)) {
                Integer val = (Integer) blockOutgoingSubjects.get(subject);
                val = new Integer(val.intValue() + 1);
                blockOutgoingSubjects.put(subject, val);
            } else {
                blockOutgoingSubjects.put(subject, new Integer(1));
            }
        }
    }

    private void unblockOutgoingSubject(String subject) {
        synchronized (blockOutgoingSubjects) {
            Log.debug("Unregistering subject to be blocked: " + subject);
            if (blockOutgoingSubjects.containsKey(subject)) {
                Integer val = (Integer) blockOutgoingSubjects.get(subject);
                if (val.intValue() - 1 <= 0) {
                    blockOutgoingSubjects.remove(subject);
                } else {
                    blockOutgoingSubjects.put(subject, new Integer(val.intValue() - 1));
                }
            }
        }
    }

    public void blockIncomingEvent(String subject, PIMItem item) {
        synchronized (blockIncomingEvents) {
            clearBlockedIncomingEvents();
            Log.debug("Registering next subject to be blocked: " + subject);
            blockIncomingEvents.put(subject, item);
        }
    }

    public void unblockIncomingEvent(String subject) {
        synchronized (blockIncomingEvents) {
            Log.debug("Registering next subject to be blocked: " + subject);
            if (!blockIncomingEvents.contains(subject))
                blockIncomingEvents.remove(subject);
        }
    }

    public void attach() {
        reload();

        Log.info("Mail listener attaching");
        Store mailStore = null;

        ServiceBook sb = ServiceBook.getSB();
        ServiceRecord[] srs = sb.getRecords();

        for (int cnt = srs.length - 1; cnt >= 0; --cnt) {

            // identify the service record associated with a mail message
            // service via a CID of 'CMIME'
            if (srs[cnt].getCid().equals("CMIME")) {
                ServiceConfiguration sc = new ServiceConfiguration(srs[cnt]);
                mailStore = null;
                Session sess = Session.getInstance(sc);
                if (sess != null) {
                    mailStore = sess.getStore();
                }

                if (mailStore == null) {
                    Log.error("Unable to get mail store for " + srs[cnt].getName());
                } else {
                    attachSendListener(mailStore);
                    attachFolderListener(mailStore);
                    attachStoreListener(mailStore);
                }
            }
        }

        Log.info("Mail listeners attach complete");
    }

    private void attachSendListener(Store mailStore) {
        // Remove the listeners in case we're attaching for a second time
        try {
            mailStore.removeSendListener(this);
        } catch (Exception e) {
        }
        mailStore.addSendListener(this);
    }

    private void attachFolderListener(Store mailStore) {
        Folder[] folders;

        folders = mailStore.list();

        for (int foldercnt = folders.length - 1; foldercnt >= 0; --foldercnt) {
            Folder f = folders[foldercnt];
            attachFolderListenerRecursive(f);
        }
    }

    private void attachFolderListenerRecursive(Folder f) {
        Log.trace("Attaching folder listener to: " + f.getFullName());
        try {
            f.removeFolderListener(this);
        } catch (Exception e) {
        }

        f.addFolderListener(this);
        Folder[] farray = f.list();
        for (int fcnt = farray.length - 1; fcnt >= 0; --fcnt) {
            attachFolderListenerRecursive(farray[fcnt]);
        }
    }

    private void attachStoreListener(Store mailStore) {
        mailStore.addStoreListener(this);
    }

    /**
     * @return the lastSentSubject
     */
    public String getLastSentSubject() {
        return lastSentSubject;
    }

    /**
     * @return the lastBlockedSubject
     */
    public String getLastBlockedSubject() {
        synchronized (blockIncomingEvents) {
            return lastBlockedSubject;
        }
    }
}
