/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMList;

import net.rim.blackberry.api.pdap.BlackBerryContact;
import net.rim.blackberry.api.pdap.BlackBerryContactList;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.source.AdditionalItemsAttributes;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.XmlUtil;
import com.funambol.util.Log;

public class SIFContactHelper extends ContactHelper
{
    private static final String             COMPANY                 = "CompanyName";                // BlackBerryContact.ORG
    private static final String             BUSINESS_CITY           = "BusinessAddressCity";
    private static final String             BUSINESS_COUNTRY        = "BusinessAddressCountry";
    private static final String             BUSINESS_PO             = "BusinessAddressPostalCode";
    private static final String             BUSINESS_STATE          = "BusinessAddressState";
    private static final String             BUSINESS_STREET         = "BusinessAddressStreet";
        
    private static final String             CATEGORIES              = "Categories";
    private static final String             PHOTO                   = "Photo";

        
    //private static final String             FILE_AS                 = "FileAs";
    private static final String             FIRST_NAME              = "FirstName";                  // BlackBerryContact.NAME_GIVEN
    private static final String             LAST_NAME               = "LastName";                   // BlackBerryContact.NAME_FAMILY
    private static final String             TITLE                   = "JobTitle";                   // BlackBerryContact.TITLE
    private static final String             WEBPAGE                 = "HomeWebPage";                // BlackBerryContact.URL
    private static final String             EMAIL_1                 = "Email1Address";              // BlackBerryContact.EMAIL
    private static final String             EMAIL_2                 = "Email2Address";              // BlackBerryContact.EMAIL
    private static final String             EMAIL_3                 = "Email3Address";              // BlackBerryContact.EMAIL
                
    private static final String             HOME_CITY               = "HomeAddressCity";
    private static final String             HOME_COUNTRY            = "HomeAddressCountry";
    private static final String             HOME_PO                 = "HomeAddressPostalCode";
    private static final String             HOME_STATE              = "HomeAddressState";
    private static final String             HOME_STREET             = "HomeAddressStreet";

    private static final String             BUSINESS_TEL            = "BusinessTelephoneNumber";    // BlackBerryContact.TEL - BlackBerryContact.ATTR_WORK        
    private static final String             HOME_TEL                = "HomeTelephoneNumber";        // BlackBerryContact.TEL - BlackBerryContact.ATTR_HOME
    private static final String             MOBILE_TEL              = "MobileTelephoneNumber";      // BlackBerryContact.TEL - BlackBerryContact.ATTR_MOBILE
    private static final String             PAGER_NUMBER            = "PagerNumber";                // Bug 856 BlackBerryContact.ATTR_PAGER
    private static final String             BUSINESS_FAX            = "BusinessFaxNumber";          // BlackBerryContact.TEL -BlackBerryContact.ATTR_FAX
    //private static final String             HOME_FAX                = "HomeFaxNumber";              // not used in BlackBerry
    private static final String             OTHER                   = "OtherTelephoneNumber";       // Bug 856 the field ist not supported by RIM        
    //private static final String             BUSINESS_TEL2           = "Business2TelephoneNumber";   // not used in BlackBerry
        
    //private static final String             INSTANT_MESSENGER       = "IstantMessenger";
    private static final String             NOTE                    = "Body";                       // BlackBerryContact.NOTE
    private static final String             PREFIX_TITLE            = "Title";                      // Bug 856 BlackBerryContact.NAME_PREFIX

    public static final String              CONTACT_TYPE_KEY        = "contact";
        
    private static final String[] telAttrs = 
    {
        SIFContactHelper.BUSINESS_TEL, // <BusinessTelephoneNumber>
        SIFContactHelper.HOME_TEL, // <HomeTelephoneNumber>
        SIFContactHelper.MOBILE_TEL, // <MobileTelephoneNumber>
        SIFContactHelper.PAGER_NUMBER, // <PagerNumber>
        SIFContactHelper.BUSINESS_FAX, // <BusinessFaxNumber>
        SIFContactHelper.OTHER //<OtherTelephoneNumber>
    };
    
    private static final int[] SIF_SUPPORTED_FIELDS =
    {
        BlackBerryContact.NAME
        ,BlackBerryContact.EMAIL
        ,BlackBerryContact.NOTE
        ,BlackBerryContact.TITLE
        ,BlackBerryContact.ORG
        ,BlackBerryContact.URL
        ,BlackBerryContact.TEL
        ,BlackBerryContact.ADDR
        ,BlackBerryContact.PHOTO
    };

    /**
     * check OS version
     * @param handled OS
     * @return true if OS >= 4.0.2
     */
    private static boolean checkOS(String os)
    {
        if (os == null || os.length() == 0)
        {
            os = StaticDataHelper.getOS();
        }

        if (os == null || !(os.length() > 3))
        {
            return false;
        }

        final int major = Integer.valueOf(os.substring(0, 1)).intValue();
        final int middle = Integer.valueOf(os.substring(2, 3)).intValue();

        int minor = 0;

        if (os.length() > 4)
        {
            minor = Integer.valueOf(os.substring(4, 5)).intValue();
        }

        if (major > 4 || major == 4 && (middle == 0 && minor >= 2 || middle >= 1))
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * The initialization constructor
     * @param BlackBerryContactList A BlackBerry contacts list
     * @param BlackBerryContact A BlackBerry contact
     * @param boolean To modify or not to modify: that is the question...
     */
    public SIFContactHelper(PIMList list)
    {
        super(list);
    }
    

    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException
    {
        if (item instanceof BlackBerryContact)
        {
            // should we remove this line after cleaning out the code
            BlackBerryContactList list = getContactList();
            
            
            BlackBerryContact contact = (BlackBerryContact)item;
            final StringBuffer contactBuffer = new StringBuffer();
                
            contactBuffer.append(XmlUtil.createOpenTagNewLine(SIFContactHelper.CONTACT_TYPE_KEY));
                
            appendXMLName(contactBuffer, contact);
            appendXMLField(list,contactBuffer, contact
                                , BlackBerryContact.ORG // the int native code of this contact field on the device
                                ,0
                                ,SIFContactHelper.COMPANY);// <CompanyName></CompanyName>
                
            appendXMLField(list,contactBuffer, contact
                                , BlackBerryContact.NOTE // the int native code of this contact field on the device
                                ,0
                                ,SIFContactHelper.NOTE);// <Body></Body>
                
            appendXMLField(list,contactBuffer, contact
                                , BlackBerryContact.TITLE // the int native code of this contact field on the device
                                ,0
                                ,SIFContactHelper.TITLE);// <JobTitle></JobTitle>
                
            appendXMLField(list,contactBuffer, contact
                                , BlackBerryContact.EMAIL // the int native code of this contact field on the device
                                ,0
                                ,SIFContactHelper.EMAIL_1);// <Email1Address></Email1Address>

            appendXMLField(list,contactBuffer, contact
                                , BlackBerryContact.EMAIL // the int native code of this contact field on the device
                                ,1
                                ,SIFContactHelper.EMAIL_2);// <Email2Address></Email2Address>

            appendXMLField(list,contactBuffer, contact
                                , BlackBerryContact.EMAIL // the int native code of this contact field on the device
                                ,2
                                ,SIFContactHelper.EMAIL_3);// <Email3Address></Email3Address>
                
            appendXMLField(list,contactBuffer
                            , contact, BlackBerryContact.URL // the int native code of this contact field on the device
                            ,0
                            ,SIFContactHelper.WEBPAGE);// <WebPage></WebPage>
                
            // bugfix 877
            XmlUtil.addElementNewLine(contactBuffer, SIFContactHelper.CATEGORIES, PIMItemHelper.extractCategories(contact),true);

            
            appendXMLTelephoneFields(contactBuffer, contact);
            appendXMLAddress(list,contactBuffer, contact);
            contactBuffer.append(XmlUtil.createCloseTagNewLine(SIFContactHelper.CONTACT_TYPE_KEY));
                
            contact = null;
            return contactBuffer.toString();
        }
        return null;
    }


    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(item,BlackBerryContact.UID,SIFContactHelper.SIF_SUPPORTED_FIELDS);
    }


    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item,
                              BlackBerryContact.UID,
                              SIFContactHelper.SIF_SUPPORTED_FIELDS);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString, AdditionalItemsAttributes additionals) throws PIMException
    {
        Log.debug("pimString ["+pimString+"]");
        BlackBerryContactList list = getContactList();
        if (list != null)
        {
            Hashtable contactMap = PIMItemHelper.buildMapFromXML(SIFContactHelper.CONTACT_TYPE_KEY,pimString,true,null,null,null);
            if (contactMap != null)
            {
                addContact((BlackBerryContact)item, contactMap);
            }
            else
            {
                Log.debug("Hashtable properties is null.");
            }
            list = null;
            contactMap = null;
        }
        //return null because Additionals attributes does not used
        return null;
    }

    protected BlackBerryContact addContact(BlackBerryContact contact, Hashtable contactMap) throws PIMException
    {
        BlackBerryContactList list = getContactList();
        if (list == null)
            return null;
        
        String fieldname = null;
        try
        {
            String osVersion = null;
            osVersion = StaticDataHelper.getOS();

            /*
            * This array is added here below to the contact through
            * BlackBerryContact.NAME (so you can see BlackBerryContact.NAME
            * as BlackBerryContact.NAME_FAMILY +
            * BlackBerryContact.NAME_GIVEN)
            */
            fieldname = "Name";
            String[] arrayField = new String[5];

            arrayField[BlackBerryContact.NAME_FAMILY] = PIMItemHelper.getValue(contactMap, LAST_NAME, true);// LastName
            arrayField[BlackBerryContact.NAME_GIVEN]  = PIMItemHelper.getValue(contactMap, FIRST_NAME, true);// FirstName
            arrayField[BlackBerryContact.NAME_PREFIX] = PIMItemHelper.getValue(contactMap, PREFIX_TITLE, true);// Title

            if (isSupportedField(BlackBerryContact.NAME))
            {
                contact.addStringArray(BlackBerryContact.NAME, PIMItem.ATTR_NONE, arrayField);
            }

            fieldname = SIFContactHelper.EMAIL_1;
            String field = PIMItemHelper.getValue(contactMap, EMAIL_1, true);// Email1Address
            if (isSupportedField(BlackBerryContact.EMAIL))
            {
                contact.addString(BlackBerryContact.EMAIL, BlackBerryContact.ATTR_NONE, field);
            }

            fieldname = SIFContactHelper.EMAIL_2;
            field = PIMItemHelper.getValue(contactMap, EMAIL_2, true);// Email2Address
            if (isSupportedField(BlackBerryContact.EMAIL))
            {
                contact.addString(BlackBerryContact.EMAIL, BlackBerryContact.ATTR_NONE, field);
            }

            fieldname = SIFContactHelper.EMAIL_3;
            field = PIMItemHelper.getValue(contactMap, EMAIL_3, true);// Email3Address
            if (isSupportedField(BlackBerryContact.EMAIL))
            {
                contact.addString(BlackBerryContact.EMAIL, BlackBerryContact.ATTR_NONE, field);
            }

            fieldname = SIFContactHelper.NOTE;
            field = PIMItemHelper.getValue(contactMap, NOTE, true);// Body
            if (isSupportedField(BlackBerryContact.NOTE))
            {
                contact.addString(BlackBerryContact.NOTE, BlackBerryContact.ATTR_NONE, field);
            }
                                
            fieldname = SIFContactHelper.TITLE;
            field = PIMItemHelper.getValue(contactMap, TITLE, true);// JobTitle
            if (isSupportedField(BlackBerryContact.TITLE))
            {
                contact.addString(BlackBerryContact.TITLE, BlackBerryContact.ATTR_NONE, field);
            }
                                
            fieldname = SIFContactHelper.COMPANY;
            field = PIMItemHelper.getValue(contactMap, COMPANY, true);// CompanyName
            if (isSupportedField(BlackBerryContact.ORG))
            {
                contact.addString(BlackBerryContact.ORG, BlackBerryContact.ATTR_NONE, field);
            }
                                
            fieldname = SIFContactHelper.WEBPAGE;
            field = PIMItemHelper.getValue(contactMap, WEBPAGE, true);// HomeWebPage
            if (isSupportedField(BlackBerryContact.URL))
            {
                contact.addString(BlackBerryContact.URL, BlackBerryContact.ATTR_NONE, field);
            }

            //=========================================================================
            //=========================================================================
            //=========================================================================
            fieldname = SIFContactHelper.BUSINESS_FAX;
            field = PIMItemHelper.getValue(contactMap, BUSINESS_FAX, false);// BusinessFaxNumber
            if (isSupportedAttributedField(BlackBerryContact.TEL,BlackBerryContact.ATTR_FAX))
            {
                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_FAX, field);
            }
                                
            fieldname = SIFContactHelper.PAGER_NUMBER;
            field = PIMItemHelper.getValue(contactMap, PAGER_NUMBER,false);// PagerNumber
            if (isSupportedAttributedField(BlackBerryContact.TEL,BlackBerryContact.ATTR_PAGER))
            {
                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_PAGER, field);
            }

            fieldname = SIFContactHelper.MOBILE_TEL;
            field = PIMItemHelper.getValue(contactMap, MOBILE_TEL,false);// MobileTelephoneNumber
            if (isSupportedAttributedField(BlackBerryContact.TEL,BlackBerryContact.ATTR_MOBILE))
            {
                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_MOBILE, field);
            }

//            field = "";
//            if (PIMItemHelper.isSupportedAttributedField(list,BlackBerryContact.TEL, BlackBerryContact.ATTR_HOME2))
//            {
//                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_HOME2, field);
//            }

            fieldname = SIFContactHelper.HOME_TEL;
            field = PIMItemHelper.getValue(contactMap, HOME_TEL,false);// HomeTelephoneNumber
            if (isSupportedAttributedField(BlackBerryContact.TEL,BlackBerryContact.ATTR_HOME))
            {
                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_HOME, field);
            }

//            field = "";
//            if (PIMItemHelper.isSupportedAttributedField(list,BlackBerryContact.TEL, BlackBerryContact.ATTR_WORK2))
//            {
//                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_WORK2, "");
//            }
                                
            fieldname = SIFContactHelper.BUSINESS_TEL;
            field = PIMItemHelper.getValue(contactMap, BUSINESS_TEL,false);// BusinessTelephoneNumber
            if (isSupportedAttributedField(BlackBerryContact.TEL,BlackBerryContact.ATTR_WORK))
            {
                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_WORK, field);
            }

            fieldname = SIFContactHelper.OTHER;
            field = PIMItemHelper.getValue(contactMap, OTHER, false);// OtherTelephoneNumber
            if (isSupportedAttributedField(BlackBerryContact.TEL,BlackBerryContact.ATTR_OTHER))
            {
                contact.addString(BlackBerryContact.TEL, BlackBerryContact.ATTR_OTHER, field);
            }
            //=========================================================================
            //=========================================================================
            //=========================================================================
            
            fieldname = SIFContactHelper.CATEGORIES;
            PIMItemHelper.addCategories(contactMap,list,contact,SIFContactHelper.CATEGORIES);
                                            
            // if the OS version is 4.0.2 upward
            if (SIFContactHelper.checkOS(osVersion))
            {
                // in case field BlackBerryContact.ADDR is supported with
                // work (business) attributes...
                fieldname = "BusinessContact";
                if (isSupportedField(BlackBerryContact.ADDR))
                {
                    arrayField = new String[7];
                    arrayField[BlackBerryContact.ADDR_COUNTRY] = PIMItemHelper.getValue(contactMap,BUSINESS_COUNTRY, true);
                    arrayField[BlackBerryContact.ADDR_STREET] = PIMItemHelper.getValue(contactMap,BUSINESS_STREET, true);
                    arrayField[BlackBerryContact.ADDR_REGION] = PIMItemHelper.getValue(contactMap,BUSINESS_STATE, true);
                    arrayField[BlackBerryContact.ADDR_LOCALITY] = PIMItemHelper.getValue(contactMap,BUSINESS_CITY, true);
                    arrayField[BlackBerryContact.ADDR_POSTALCODE] = PIMItemHelper.getValue(contactMap,BUSINESS_PO, true);
                                                
                    contact.addStringArray(BlackBerryContact.ADDR, BlackBerryContact.ATTR_WORK,arrayField);
                }
                                        
                // in case the field BlackBerryContact.ADDR supports the
                // Home attributes too...
                fieldname = "HomeContact";
                if (isSupportedAttributedField(BlackBerryContact.ADDR,BlackBerryContact.ATTR_HOME))
                {
                    arrayField = new String[7];
                    arrayField[BlackBerryContact.ADDR_COUNTRY] = PIMItemHelper.getValue(contactMap,HOME_COUNTRY, true);
                    arrayField[BlackBerryContact.ADDR_STREET] = PIMItemHelper.getValue(contactMap,HOME_STREET, true);
                    arrayField[BlackBerryContact.ADDR_REGION] = PIMItemHelper.getValue(contactMap, HOME_STATE, true);
                    arrayField[BlackBerryContact.ADDR_LOCALITY] = PIMItemHelper.getValue(contactMap,HOME_CITY, true);
                    arrayField[BlackBerryContact.ADDR_POSTALCODE] = PIMItemHelper.getValue(contactMap,HOME_PO, true);
                                                
                    contact.addStringArray(BlackBerryContact.ADDR, BlackBerryContact.ATTR_HOME,arrayField);
                }
            }

            if (Customization.getInstance().syncContactPhoto()) {
                fieldname = SIFContactHelper.PHOTO;
                field = PIMItemHelper.getValue(contactMap, PHOTO, false);
                if (isSupportedField(BlackBerryContact.PHOTO) &&
                    !field.equals(PIMItemHelper.EMPTY_STRING))
                {
                    byte[] byteField = field.getBytes();
                    contact.addBinary(BlackBerryContact.PHOTO, BlackBerryContact.ATTR_NONE, byteField ,0,byteField.length);
                }
            }
            return contact;
        } catch (final Throwable e) {
            Log.error("SIFContactHelper.addContact() on field ["+fieldname+"]. Exception -> "+ e.toString());
            throw new PIMException("Exception while setting value to Contact field ["+fieldname+"]");
        }                
    }
    
    /**
     * This function supports toString() function. This method only appends the data corresponding to contact's address
     * details.
     * @param StringBuffer: Blackberry contact list
     * @param BlackBerryContact: Blackberry contact
     * @return void
     */
    private void appendXMLAddress(BlackBerryContactList list, StringBuffer buffer, BlackBerryContact contact)
    {
        boolean findHome = false;
        boolean findBusiness = false;

        String osVersion = null;                
        osVersion = StaticDataHelper.getOS();
        if (SIFContactHelper.checkOS(osVersion))
        {
            if (isSupportedField(BlackBerryContact.ADDR) && contact.countValues(BlackBerryContact.ADDR) > 0)
            {
                for (int i = 0; i < contact.countValues(BlackBerryContact.ADDR); i++)
                {
                    final String[] addr = contact.getStringArray(BlackBerryContact.ADDR, i);
                    if (addr != null && addr.length > 0)
                    {
                        final int addrType = contact.getAttributes(BlackBerryContact.ADDR, i);
                        if (addrType == BlackBerryContact.ATTR_HOME)
                        {
                            findHome = true;
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_STREET,addr[BlackBerryContact.ADDR_STREET],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_STATE, addr[BlackBerryContact.ADDR_REGION],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_CITY,addr[BlackBerryContact.ADDR_LOCALITY],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_PO,addr[BlackBerryContact.ADDR_POSTALCODE],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_COUNTRY,addr[BlackBerryContact.ADDR_COUNTRY],true);
                        }
                        else
                        {
                            findBusiness = true;
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_STREET,addr[BlackBerryContact.ADDR_STREET],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_STATE,addr[BlackBerryContact.ADDR_REGION],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_CITY,addr[BlackBerryContact.ADDR_LOCALITY],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_PO,addr[BlackBerryContact.ADDR_POSTALCODE],true);
                            XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_COUNTRY,addr[BlackBerryContact.ADDR_COUNTRY],true);
                        }
                    }
                }
            }
            if (!findHome)
            {
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_STREET, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_STATE, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_CITY, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_PO, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.HOME_COUNTRY, "",false);
            }
            if (!findBusiness)
            {
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_STREET, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_STATE, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_CITY, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_PO, "",false);
                XmlUtil.addElementNewLine(buffer, SIFContactHelper.BUSINESS_COUNTRY, "",false);
            }
        }
    }
        
    /**
     * This function supports toString() function. Appends the contact information supported by the BlackBerry to a string
     * buffer (uses #appendToContact)
     * @param StringBuffer String buffer containing contact information
     * @param BlackBerryContact A BlackBerry contact
     * @param int Type of field (native code of the device's contact field)
     * @param String An XML tag for the SIF format
     */
    private void appendXMLField(BlackBerryContactList list, StringBuffer contactBuffer, BlackBerryContact contact, int field, int pos, String tag)
    {
        String value = null;
                
        if (isSupportedField(field))
        {
            value = "";
            try
            {
                int count = contact.countValues(field);
                if (count>0 && (0<=pos && pos<count))
                {
                    value = contact.getString(field, pos);
                }
            }
            catch (final Exception e)
            {
                Log.error("SIFContactHelper.appendField(): Could not get value of field: " + tag + ". --> " + e.toString());
            }
            XmlUtil.addElementNewLine(contactBuffer, tag, value,true);
        }
    }
    
    /**
     * This function supports toString() function. This method appends the data corresponding to the contact's name array
     * 
     * @param StringBuffer String buffer to which data is appended
     * @param BlackBerryContact The BlackBerry contact object
     */
    private void appendXMLName(StringBuffer buffer, BlackBerryContact contact)
    {
        if (contact.countValues(BlackBerryContact.NAME) > 0)
        {
            final String[] name = contact.getStringArray(BlackBerryContact.NAME, 0);
                        
            XmlUtil.addElementNewLine(buffer, SIFContactHelper.FIRST_NAME, name[BlackBerryContact.NAME_GIVEN],true);// e.g.<FirstName>John</FirstName>
            XmlUtil.addElementNewLine(buffer, SIFContactHelper.LAST_NAME, name[BlackBerryContact.NAME_FAMILY],true);// e.g.<LastName>Doe</LastName>
            XmlUtil.addElementNewLine(buffer, SIFContactHelper.PREFIX_TITLE, name[BlackBerryContact.NAME_PREFIX],true);// e.g.<Title>Dr</Title>
        }
        else
        {
            XmlUtil.addElementNewLine(buffer, SIFContactHelper.FIRST_NAME, "",false);// e.g.<FirstName></FirstName>
            XmlUtil.addElementNewLine(buffer, SIFContactHelper.LAST_NAME, "",false);// e.g.<LastName></LastName>
            XmlUtil.addElementNewLine(buffer, SIFContactHelper.PREFIX_TITLE, "",false);// e.g. <Title></Title>
        }
    }
    
    /**
    * This function supports toString() function. This method appends the data corresponding to the contact's telephone
    * numbers array
    * @param StringBuffer BlackBerry contact list
    * @param BlackBerryContact BlackBerry contact
    */
    private void appendXMLTelephoneFields(StringBuffer buffer, BlackBerryContact contact)
    {
        /**
        * The number of values contained in the <code>BlackBerryContact.TEL</code> field
        */
        final int totalTelFields = contact.countValues(BlackBerryContact.TEL);
        final Vector addedAttrs = new Vector();
                
        for (int i = 0; i < totalTelFields; i++)
        {
            final String value = contact.getString(BlackBerryContact.TEL, i);
            final int attr = contact.getAttributes(BlackBerryContact.TEL, i);
            String tag = "";
            switch (attr)
            {
                case BlackBerryContact.ATTR_WORK:
                    tag = SIFContactHelper.BUSINESS_TEL;// <BusinessTelephoneNumber>
                    break;
                case BlackBerryContact.ATTR_HOME:
                    tag = SIFContactHelper.HOME_TEL;// <HomeTelephoneNumber>
                    break;
                case BlackBerryContact.ATTR_MOBILE:
                    tag = SIFContactHelper.MOBILE_TEL;// <MobileTelephoneNumber>
                    break;
                case BlackBerryContact.ATTR_PAGER:
                    tag = SIFContactHelper.PAGER_NUMBER;// <PagerNumber>
                    break;
                case BlackBerryContact.ATTR_FAX:
                    tag = SIFContactHelper.BUSINESS_FAX;// <BusinessFaxNumber>
                    break;
                case BlackBerryContact.ATTR_OTHER:
                    tag = SIFContactHelper.OTHER;// <OtherTelephoneNumber>
                    break;
            }
            if (!tag.equals(""))
            {
                XmlUtil.addElementNewLine(buffer, tag, value,true);
                addedAttrs.addElement(tag);
            }
        }
        // this is just to create an empty element
        // for each element that wasn't added here
        // above
        for (int i = 0; i < telAttrs.length; i++)
        {
            if (!addedAttrs.contains(telAttrs[i]))
            {
                XmlUtil.addElementNewLine(buffer, telAttrs[i], "",false);
            }
        }
    }
} 

