/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Hashtable;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import net.rim.blackberry.api.pdap.BlackBerryMemo;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;

import com.funambol.util.XmlUtil;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;

import com.funambol.blackberry.source.AdditionalItemsAttributes;

public class SIFNoteHelper extends NoteHelper
{
    public static final String NOTE_TYPE_KEY            =   "note";
    public static final String BODY_KEY                 =   "Body";
    public static final String SUBJECT_KEY              =   "Subject";
    public static final String CATEGORIES_KEY           =   "Categories";
    public static final String COLOR_KEY                =   "Color";
    public static final String HEIGHT_KEY               =   "Height";
    public static final String WIDTH_KEY                =   "Width";
    public static final String LEFT_KEY                 =   "Left";
    public static final String TOP_KEY                  =   "Top";
    
    private static final int[] SIF_SUPPORTED_FIELDS =
    {
        BlackBerryMemo.TITLE
        ,BlackBerryMemo.NOTE
    };
    
    public SIFNoteHelper(PIMList list)
    {
        super(list);
    }
    
    private BlackBerryMemo fillMemo(BlackBerryMemo memo, Hashtable memoMap) throws PIMException
    {
        String fieldname = null;
        
        try
        {
            fieldname = SIFNoteHelper.SUBJECT_KEY;
            if (isSupportedField(BlackBerryMemo.TITLE))
            {
                memo.addString(BlackBerryMemo.TITLE,PIMItem.ATTR_NONE, PIMItemHelper.getValue(memoMap, SIFNoteHelper.SUBJECT_KEY, true));
            }
            fieldname = SIFNoteHelper.BODY_KEY;
            if (isSupportedField(BlackBerryMemo.NOTE))
            {
                memo.addString(BlackBerryMemo.NOTE,PIMItem.ATTR_NONE, PIMItemHelper.getValue(memoMap, SIFNoteHelper.BODY_KEY, true));
            }
            return memo;
        }
        catch(final Throwable e)
        {
            Log.error("SIFNoteHelper.addMemo() on field ["+fieldname+"]. Exception -> "+ e.toString());
            throw new PIMException("Exception while setting value to Memo field ["+fieldname+"]");
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException
    {
       
        if (item instanceof BlackBerryMemo)
        {
            BlackBerryMemo note = (BlackBerryMemo)item;
            StringBuffer temp = new StringBuffer(XmlUtil.createOpenTagNewLine(SIFNoteHelper.NOTE_TYPE_KEY));
            if (isSupportedField(BlackBerryMemo.TITLE) && item.countValues(BlackBerryMemo.TITLE)>0)
            {
                XmlUtil.addElementNewLine(temp,SIFNoteHelper.SUBJECT_KEY,note.getString(BlackBerryMemo.TITLE,0),true);
            }
            if (isSupportedField(BlackBerryMemo.NOTE) && item.countValues(BlackBerryMemo.NOTE) > 0)
            {
                XmlUtil.addElementNewLine(temp,SIFNoteHelper.BODY_KEY,note.getString(BlackBerryMemo.NOTE,0),true);
            }
            XmlUtil.addElementNewLine(temp,SIFNoteHelper.CATEGORIES_KEY,"",false);
            XmlUtil.addElementNewLine(temp,SIFNoteHelper.COLOR_KEY,"",false);
            XmlUtil.addElementNewLine(temp,SIFNoteHelper.HEIGHT_KEY,"",false);
            XmlUtil.addElementNewLine(temp,SIFNoteHelper.WIDTH_KEY,"",false);
            XmlUtil.addElementNewLine(temp,SIFNoteHelper.LEFT_KEY,"",false);
            XmlUtil.addElementNewLine(temp,SIFNoteHelper.TOP_KEY,"",false);
            temp.append(XmlUtil.createCloseTagNewLine(SIFNoteHelper.NOTE_TYPE_KEY));
            note = null;
            return temp.toString();
        }
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof BlackBerryMemo)
        {
            try
            {
                return item.getString(BlackBerryMemo.UID,0);
            }
            catch(final Exception e)
            {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Note]";
                Log.error(msg);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Note] is not a BlackBerryMemo.";
        Log.error(msg);
        throw new PIMException(msg);
    }
    

    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString, AdditionalItemsAttributes additionals) throws PIMException
    {
        Log.debug("pimString ["+pimString+"]");
        if (item instanceof BlackBerryMemo)
        {
            BlackBerryMemo memo = (BlackBerryMemo)item;
            Hashtable noteMap = PIMItemHelper.buildMapFromXML(SIFNoteHelper.NOTE_TYPE_KEY,pimString,true,null,null,null);
            if (noteMap!=null)
            {
                fillMemo(memo,noteMap);
                noteMap = null;
                memo = null;
            }
            else
            {
                Log.debug("Hashtable properties is null.");
            }
            memo = null;
            noteMap = null;
        }
        //return null because Additionals attributes does not used
        return null;
    }

    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem _item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(_item,BlackBerryMemo.UID,SIFNoteHelper.SIF_SUPPORTED_FIELDS);
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item, BlackBerryMemo.UID,
                              SIFNoteHelper.SIF_SUPPORTED_FIELDS);
    }

}
