/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;

import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.ToDo;
import javax.microedition.pim.ToDoList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.RepeatRule;

import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.util.XmlUtil;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;
import com.funambol.blackberry.source.AdditionalItemsAttributes;

public class SIFTaskHelper extends TaskHelper
{
    public static final String          TASK_TYPE_KEY       = "task";

    public static final String          BODY                = "Body";
    public static final String          SUBJECT             = "Subject";
    public static final String          COMPLETE            = "Complete";
    public static final String          SENSITIVITY         = "Sensitivity";
    public static final String          IMPORTANCE          = "Importance";
    public static final String          DATE_COMPLETED      = "DateCompleted";
    public static final String          DUE_DATE            = "DueDate";
    public static final String          REMINDER_SET        = "ReminderSet";
    public static final String          REMINDER_TIME       = "ReminderTime";
    public static final String          IS_RECURRING        = "IsRecurring";
    public static final String          CATEGORIES              = "Categories";

    public static final String          STATUS              = "Status";
    public static final int             BB_STATUS           = PIMItem.EXTENDED_FIELD_MIN_VALUE + 9;
    public static final int             BB_ALARM            = 100;

    //private static final String             DATETIME_FORMAT_ALLDAY                  = "yyyy-MM-dd";
    //private static final String             DATETIME_FORMAT_UTC                     = "yyyyMMdd'T'HHmmss'Z'";
    
    private static final int            SIF_BB_Important[][] = {{0,9}// LOW
                                                                ,{1,5} // NORMAL
                                                                ,{2,1} // HIGH
                                                            };
    private static final int            SIF_BB_Status[][]   = {{0,0}// STATUS_NOT_STARTED
                                                                ,{1,1} // STATUS_IN_PROGRESS
                                                                ,{2,2} // STATUS_COMPLETED
                                                                ,{3,3} // STATUS_WAITING
                                                                ,{4,4} // STATUS_DEFERRED
                                                            };
    private static final int            SIF_BB_Sensitivity[][]   = {{0,BlackBerryToDo.CLASS_PUBLIC}// NORMAL - PUBLIC
                                                                ,{1,BlackBerryToDo.CLASS_PRIVATE} // PERSONAL
                                                                ,{2,BlackBerryToDo.CLASS_PRIVATE} // PRIVATE
                                                                ,{3,BlackBerryToDo.CLASS_CONFIDENTIAL} // CONFIDENTIAL
                                                            };

    private static final int[] SIF_SUPPORTED_FIELDS =
    {
        BlackBerryToDo.SUMMARY
        ,BlackBerryToDo.NOTE
        ,BlackBerryToDo.COMPLETED
        ,BlackBerryToDo.DUE
        ,BlackBerryToDo.COMPLETION_DATE
        ,SIFTaskHelper.BB_STATUS
        ,BlackBerryToDo.PRIORITY
        ,BlackBerryToDo.CLASS
    };
    
    
    public SIFTaskHelper(PIMList list)
    {
        super(list);
    }
    

    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException
    {
        String fieldname = null;
        
        PIMList pimlist = getPIMList();
        
        try
        {
            if (item instanceof BlackBerryToDo)
            {
                BlackBerryToDo task = (BlackBerryToDo)item;
                StringBuffer temp = new StringBuffer(XmlUtil.createOpenTagNewLine(SIFTaskHelper.TASK_TYPE_KEY));

                fieldname = SIFTaskHelper.SUBJECT;
                if (isSupportedField(BlackBerryToDo.SUMMARY) && item.countValues(BlackBerryToDo.SUMMARY)>0)
                {
                    XmlUtil.addElementNewLine(temp,SIFTaskHelper.SUBJECT,task.getString(BlackBerryToDo.SUMMARY,0),true);
                }

                fieldname = SIFTaskHelper.BODY;
                if (isSupportedField(BlackBerryToDo.NOTE) && item.countValues(BlackBerryToDo.NOTE)>0)
                {
                    XmlUtil.addElementNewLine(temp,SIFTaskHelper.BODY,task.getString(BlackBerryToDo.NOTE,0),true);
                }

                fieldname = SIFTaskHelper.COMPLETE;
                appendXMLFieldBoolean(temp,task,BlackBerryToDo.COMPLETED,SIFTaskHelper.COMPLETE);
                fieldname = SIFTaskHelper.SENSITIVITY;
                appendXMLSensitivity(pimlist,temp,task);
                fieldname = SIFTaskHelper.IMPORTANCE;
                appendXMLPriority(pimlist,temp,task);
                fieldname = SIFTaskHelper.DATE_COMPLETED;
                
                Calendar CompletedDate = getFieldDate(task, BlackBerryToDo.COMPLETION_DATE, false, false, true);
                String formattedCompletedDate = CompletedDate != null ? PIMItemHelper.formatDateTimeUTC(CompletedDate) : "";
                Log.info("SIFTaskHelper.toString - COMPLETION_DATE : my format ["+ formattedCompletedDate +"]");
                XmlUtil.addElementNewLine(temp, SIFTaskHelper.DATE_COMPLETED, formattedCompletedDate, false);
                
                fieldname = SIFTaskHelper.DUE_DATE;
                Calendar dueDate = getFieldDate(task, BlackBerryToDo.DUE, false, false, true);
                String formattedDueDate = dueDate != null ? PIMItemHelper.formatAllDay(dueDate) : "";

                Log.info("SIFTaskHelper.toString - DUE : my format ["+ formattedDueDate +"]");
                XmlUtil.addElementNewLine(temp, SIFTaskHelper.DUE_DATE, formattedDueDate, false);
                
                fieldname = SIFTaskHelper.STATUS;
                appendXMLStatus(pimlist,temp,task);
                fieldname = SIFTaskHelper.IS_RECURRING;
                formattedDueDate = dueDate != null ? PIMItemHelper.formatAllDay(dueDate) : "";
                appendXMLRecurrence(pimlist,temp, task,formattedDueDate,formattedDueDate);
                fieldname = SIFTaskHelper.CATEGORIES;
                XmlUtil.addElementNewLine(temp, SIFTaskHelper.CATEGORIES, PIMItemHelper.extractCategories(task),true);
                
                if (isSupportedField(SIFTaskHelper.BB_ALARM))
                {
                    fieldname = SIFTaskHelper.REMINDER_TIME;
                    //long value = task.getDate(BlackBerryToDo.REMINDER, 0);
                    long value = task.getInt(SIFTaskHelper.BB_ALARM,0);
                    if (value>0)
                    {
                        XmlUtil.addElementNewLine(temp,SIFTaskHelper.REMINDER_SET,"1",false);
                        //SIFTaskHelper.appendXMLFieldDate(pimlist,temp,task,BlackBerryToDo.REMINDER,SIFTaskHelper.REMINDER_TIME);
                        XmlUtil.addElementNewLine(temp,SIFTaskHelper.REMINDER_TIME,""+value,false);
                    }
                    else
                    {
                        XmlUtil.addElementNewLine(temp,SIFTaskHelper.REMINDER_SET,"0",false);
                    }
                }
                else
                {
                    Log.debug("BlackBerryToDo does not support BlackBerryToDo.ALARM");
                }
                
                temp.append(XmlUtil.createCloseTagNewLine(SIFTaskHelper.TASK_TYPE_KEY));
                task = null;
                return temp.toString();
            }
            return null;
        }
        catch(final Throwable e)
        {
            Log.error("SIFTaskHelper.toString() on field ["+fieldname+"]. Exception -> "+ e.toString());
            throw new PIMException("Exception while converting Task to SIF format ["+fieldname+"]");
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof ToDo)
        {
            try
            {
                return item.getString(ToDo.UID,0);
            }
            catch(final Exception e)
            {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Task]";
                Log.error(msg);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Task] is not a BlackBerryToDo/ToDo.";
        Log.error(msg);
        throw new PIMException(msg);
    }
    

    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString, AdditionalItemsAttributes additionals) throws PIMException
    {
        Log.debug("pimString ["+pimString+"]");
        
        ToDoList list = getToDoList();
        
        if (list != null)
        {
            Vector multiValueTags = new Vector(2);
            multiValueTags.addElement(SIFRepeatRuleHelper.EXCLUDE_DATE);
            multiValueTags.addElement(SIFRepeatRuleHelper.INCLUDE_DATE);
            
            
            Hashtable taskMap = PIMItemHelper.buildMapFromXML(SIFTaskHelper.TASK_TYPE_KEY,pimString,true,multiValueTags,null,null);
            if (taskMap != null)
            {
                fillTask((BlackBerryToDo)item,taskMap);
            }
            else
            {
                Log.debug("Hashtable properties is null.");
            }
        }
        //return null because Additionals attributes does not used
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem _item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(_item,BlackBerryToDo.UID,SIFTaskHelper.SIF_SUPPORTED_FIELDS);
        //((BlackBerryToDo)_item).setRepeat(null);
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item, BlackBerryToDo.UID,
                              SIFTaskHelper.SIF_SUPPORTED_FIELDS);
    }

    
    
    
    
    
    private void parseStatus(ToDoList list, BlackBerryToDo task, Hashtable taskMap)
    {
        if (list.isSupportedField(SIFTaskHelper.BB_STATUS))
        {
            int status = SIFTaskHelper.SIF_BB_Status[0][1];
            try
            {
                int sif_status = Integer.parseInt(PIMItemHelper.getValue(taskMap, SIFTaskHelper.STATUS, false));
                //Log.info("SIF Status is ["+sif_status+"]");
                for (int i=0;i<SIFTaskHelper.SIF_BB_Status.length;i++)
                {
                    if (sif_status == SIFTaskHelper.SIF_BB_Status[i][0])
                    {
                        status = SIFTaskHelper.SIF_BB_Status[i][1];
                    }
                }
            }
            catch (final Exception e)
            {
                Log.info("Exception while matching BB Status");
            }
            //Log.info("BB Status is ["+status+"]");
            task.addInt(SIFTaskHelper.BB_STATUS,BlackBerryToDo.ATTR_NONE,status);
        }
    }
    
    private void parsePriority(BlackBerryToDo task, Hashtable taskMap)
    {
        if (isSupportedField(BlackBerryToDo.PRIORITY))
        {
            int priority = SIFTaskHelper.SIF_BB_Important[0][1];
            try
            {
                int sif_prior = Integer.parseInt(PIMItemHelper.getValue(taskMap, SIFTaskHelper.IMPORTANCE, false));
                for (int i=0;i<SIFTaskHelper.SIF_BB_Important.length;i++)
                {
                    if (sif_prior == SIFTaskHelper.SIF_BB_Important[i][0])
                    {
                        priority = SIFTaskHelper.SIF_BB_Important[i][1];
                    }
                }
            }
            catch (final Exception e)
            {}
            task.addInt(BlackBerryToDo.PRIORITY,BlackBerryToDo.ATTR_NONE,priority);
        }
    }
    
    private void parseSensitivity(BlackBerryToDo task, Hashtable taskMap)
    {
        if (isSupportedField(BlackBerryToDo.CLASS))
        {
            int sesitivity = SIFTaskHelper.SIF_BB_Sensitivity[0][0];
            try
            {
                int sif_sensitivity = Integer.parseInt(PIMItemHelper.getValue(taskMap, SIFTaskHelper.SENSITIVITY, false));
                for (int i=0;i<SIFTaskHelper.SIF_BB_Sensitivity.length;i++)
                {
                    if (sif_sensitivity == SIFTaskHelper.SIF_BB_Sensitivity[i][0])
                    {
                        sesitivity = SIFTaskHelper.SIF_BB_Sensitivity[i][1];
                    }
                }
            }
            catch (final Exception e)
            {}
            task.addInt(BlackBerryToDo.CLASS,BlackBerryToDo.ATTR_NONE,sesitivity);
        }
    }
    
    private ToDo fillTask(BlackBerryToDo task, Hashtable taskMap) throws PIMException
    {
        ToDoList list = getToDoList();
        if (list == null)
            return null;
        
        String fieldname = null;
        try
        {
            String field = null;
            
            //BlackBerryToDo.SUMMARY
            fieldname = SIFTaskHelper.SUBJECT;
            if (isSupportedField(BlackBerryToDo.SUMMARY))
            {
                field = PIMItemHelper.getValue(taskMap, SIFTaskHelper.SUBJECT, true);
                if (task.countValues(BlackBerryToDo.SUMMARY)==0)
                {
                    task.addString(BlackBerryToDo.SUMMARY, BlackBerryToDo.ATTR_NONE, field);
                }
                else
                {
                    task.setString(BlackBerryToDo.SUMMARY, 0, BlackBerryToDo.ATTR_NONE, field);
                }
            }
            
            //BlackBerryToDo.NOTE
            fieldname = SIFTaskHelper.BODY;
            if (isSupportedField(BlackBerryToDo.NOTE))
            {
                field = PIMItemHelper.getValue(taskMap, SIFTaskHelper.BODY, true);
                if (task.countValues(BlackBerryToDo.NOTE)==0)
                {
                    task.addString(BlackBerryToDo.NOTE, BlackBerryToDo.ATTR_NONE, field);
                }
                else
                {
                    task.setString(BlackBerryToDo.NOTE, 0, BlackBerryToDo.ATTR_NONE, field);
                }
            }

            //BlackBerryToDo.DUE
            fieldname = SIFTaskHelper.DUE_DATE;
            if (isSupportedField(BlackBerryToDo.DUE))
            {
                field = PIMItemHelper.getValue(taskMap, SIFTaskHelper.DUE_DATE, false);
                if (field.length() > 0)
                {
                    long dateStart = PIMItemHelper.parseDateTime(field).getTime().getTime();
                    dateStart = PIMItemHelper.convertGMTToLocal(dateStart);
                    task.addDate(BlackBerryToDo.DUE, BlackBerryToDo.ATTR_NONE, dateStart);
                }
            }

            //BlackBerryToDo.COMPLETION_DATE
            fieldname = SIFTaskHelper.DATE_COMPLETED;
            if (isSupportedField(BlackBerryToDo.COMPLETION_DATE))
            {
                field = PIMItemHelper.getValue(taskMap, SIFTaskHelper.DATE_COMPLETED, false);
                if (field.length() > 0)
                {
                    long dateStart = PIMItemHelper.parseDateTime(field).getTime().getTime();
                    dateStart = PIMItemHelper.convertGMTToLocal(dateStart);
                    task.addDate(BlackBerryToDo.COMPLETION_DATE, BlackBerryToDo.ATTR_NONE, dateStart);
                }
            }

            fieldname = SIFTaskHelper.IMPORTANCE;
            parsePriority(task,taskMap);
            fieldname = SIFTaskHelper.SENSITIVITY;
            parseSensitivity(task,taskMap);
            fieldname = SIFTaskHelper.CATEGORIES;
            PIMItemHelper.addCategories(taskMap,list,task,SIFTaskHelper.CATEGORIES);

            //BlackBerryToDo.COMPLETE
            boolean completed = false;
            fieldname = SIFTaskHelper.COMPLETE;
            if (isSupportedField(BlackBerryToDo.COMPLETED))
            {
                field = PIMItemHelper.getValue(taskMap, SIFTaskHelper.COMPLETE, false);
                if (field.length()>0)
                {
                    if (!"false".equalsIgnoreCase(field) && !"0".equals(field))
                    {
                        completed = true;
                    }
                    task.addBoolean(BlackBerryToDo.COMPLETED,BlackBerryToDo.ATTR_NONE,completed);
                }
            }
                        
            // RepeatRule
            if (!completed)
            {
                fieldname = SIFTaskHelper.IS_RECURRING;
                field = PIMItemHelper.getValue(taskMap, SIFTaskHelper.IS_RECURRING, false);// e.g. 1
                if (field.length()>0)
                {
                    if (!"false".equalsIgnoreCase(field) && !"0".equals(field))
                    {
                        //System.out.println("Before creating a new RepeatRule");
                        RepeatRule rrule = new RepeatRule();
                        //System.out.println("Before setting data to the new RepeatRule");
                        SIFRepeatRuleHelper.setRepeatRule(taskMap,rrule,true);
                        //System.out.println("Before setting the new RepeatRule to task ");
                        task.setRepeat(rrule);
                        //System.out.println("After setting the new RepeatRule to task ");
                    }
                }
            }
            else
            {
                Log.info("Can not set Recurrence and Completed on task.");
            }

            // Status or not
            if (!completed)
            {
                fieldname = SIFTaskHelper.STATUS;
                parseStatus(list,task,taskMap);
            }

            return task;
        }
        catch (final Throwable e)
        {
            if (fieldname.equals(SIFTaskHelper.IS_RECURRING))
            {
                fieldname = SIFRepeatRuleHelper.popSettingFieldName();
            }
            Log.error("SIFTaskHelper.addTask() on field ["+fieldname+"]. Exception -> "+ e.toString());
            throw new PIMException("Exception while setting value to Task field ["+fieldname+"]");
        }
    }
    
    private void appendXMLRecurrence(PIMList list, StringBuffer taskBuffer, BlackBerryToDo task, String patternStartDate, String dueDate) throws PIMException
    {
        // enhancement 868 (recurrence)
        RepeatRule rule = task.getRepeat();
        if (rule == null)
        {
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.IS_RECURRING, "0",false);
        }
        else if (dueDate!=null)
        {
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.IS_RECURRING, "1",false);
            taskBuffer.append(SIFRepeatRuleHelper.toString(rule,patternStartDate,dueDate,true));
            rule = null;
        }
        else
        {
            throw new PIMException("A task must have a DueDate to have Recurrence.");
        }
    }
    
    private void appendXMLStatus(PIMList list,StringBuffer taskBuffer, BlackBerryToDo task)
    {
        if (list.isSupportedField(SIFTaskHelper.BB_STATUS))
        {
            int c = SIFTaskHelper.SIF_BB_Status[0][0];
            try
            {
                int status = task.getInt(SIFTaskHelper.BB_STATUS, 0);
                Log.debug("BlackBerry status ["+status+"]");
                
                for (int i=0;i<SIFTaskHelper.SIF_BB_Status.length;i++)
                {
                    if (status == SIFTaskHelper.SIF_BB_Status[i][1])
                    {
                        c = SIFTaskHelper.SIF_BB_Status[i][0];
                    }
                }
            }
            catch (final Exception e)
            {}
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.STATUS, ""+c,false);
        }
        else
        {
            Log.info("BlackBerryToDo does not support BlackBerryToDo.STATUS or SIFTaskHelper.STATUS");
        }
    }
    
    private void appendXMLPriority(PIMList list,StringBuffer taskBuffer, BlackBerryToDo task)
    {
        if (list.isSupportedField(BlackBerryToDo.PRIORITY))
        {
            int c = SIFTaskHelper.SIF_BB_Important[0][0];
            try
            {
                int priority = task.getInt(BlackBerryToDo.PRIORITY, 0);
                for (int i=0;i<SIFTaskHelper.SIF_BB_Important.length;i++)
                {
                    if (priority == SIFTaskHelper.SIF_BB_Important[i][1])
                    {
                        c = SIFTaskHelper.SIF_BB_Important[i][0];
                    }
                }
            }
            catch (final Exception e)
            {
                Log.debug("Default value was used because of exception while retrieving SIF-XML field ["+SIFTaskHelper.IMPORTANCE+"]");
            }
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.IMPORTANCE, ""+c,false);
        }
        else
        {
            Log.info("BlackBerryToDo does not support BlackBerryToDo.PRIORITY or SIFTaskHelper.IMPORTANCE");
        }
    }
    
    private void appendXMLSensitivity(PIMList list,StringBuffer taskBuffer, BlackBerryToDo task)
    {
        /*
        * using unsupported Event.CLASS field causes an
        * UnsupportedFieldException
        */
        if (list.isSupportedField(BlackBerryToDo.CLASS))
        {
            int c = SIFTaskHelper.SIF_BB_Sensitivity[0][0];
            try
            {
                int sensitivity = task.getInt(BlackBerryToDo.CLASS, 0);
                Log.debug("BlackBerry Sensitivity/Class ["+sensitivity+"]");
                
                for (int i=0;i<SIFTaskHelper.SIF_BB_Sensitivity.length;i++)
                {
                    if (sensitivity == SIFTaskHelper.SIF_BB_Sensitivity[i][1])
                    {
                        c = SIFTaskHelper.SIF_BB_Sensitivity[i][0];
                    }
                }
            }
            catch (final Exception e)
            {
                Log.debug("Default value was used because of exception while retrieving SIF-XML field ["+SIFTaskHelper.SENSITIVITY+"]");
            }
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.SENSITIVITY, ""+c,false);
        }
        else
        {
            Log.info("BlackBerryToDo does not support BlackBerryToDo.CLASS or SIFTaskHelper.SENSITIVITY");
        }
    }
} 
