/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import com.funambol.blackberry.controller.SyncStatusController;
import com.funambol.blackberry.controller.SynchronizationController;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.SyncListener;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.protocol.SyncMLStatus;

import com.funambol.util.Log;

/**
 * @author mtaczak
 * 
 */
public class BBPIMSyncListener implements SyncListener {

    private SyncStatusController      syncStatusController;

    private SynchronizationController synchronizationController;

    private AppSyncSource             appSource;

    private int                       sourceMask;

    private int                       totalSent;

    private int                       totalSending;

    private int                       totalReceived;

    private int                       totalReceiving;

    private boolean                   sendingLO = false;

    public BBPIMSyncListener(SyncStatusController controller,
                             SynchronizationController synchronizationController,
                             AppSyncSource appSource) {

        this.syncStatusController = controller;
        this.synchronizationController = synchronizationController;
        this.appSource = appSource;
        this.sourceMask = appSource.getId();
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#dataReceived(java.lang.String, int)
     */
    public void dataReceived(String date, int size) {
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#endConnecting(int)
     */
    public void endConnecting(int action) {
        syncStatusController.setSyncStatus(sourceMask, StaticDataHelper.getLanguage("status_connecting"));
        try {
            Thread.sleep(250);
        } catch (Exception e) {
        }

    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#endMapping()
     */
    public void endMapping() {
        syncStatusController.setSyncStatus(sourceMask, StaticDataHelper.getLanguage("status_mapping-done"));
        try {
            Thread.sleep(250);
        } catch (Exception e) {
        }

    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#endReceiving()
     */
    public void endReceiving() {
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#endSending()
     */
    public void endSending() {
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#endSession(int)
     */
    public void endSession(int status) {
        // This source sync is over, set the proper status
        switch (status) {
            case SyncListener.SUCCESS:
                syncStatusController.setSyncSuccess(sourceMask,
                        StaticDataHelper.getLanguage("status_complete"));
                break;
            case SyncListener.INVALID_CREDENTIALS:
                syncStatusController.setSyncError(sourceMask,
                        StaticDataHelper.getLanguage("status_invalid-credentials"));
                break;
            case SyncListener.FORBIDDEN_ERROR:
                syncStatusController.setSyncError(sourceMask,
                        StaticDataHelper.getLanguage("status_forbidden-error"));
                break;
            case SyncListener.READ_SERVER_RESPONSE_ERROR:
            case SyncListener.WRITE_SERVER_REQUEST_ERROR:
            case SyncListener.CONN_NOT_FOUND:
                syncStatusController.setSyncError(sourceMask,
                        StaticDataHelper.getLanguage("status_network-error"));
                break;
            case SyncListener.CONNECTION_BLOCKED_BY_USER:
                syncStatusController.setSyncError(sourceMask,
                        StaticDataHelper.getLanguage("status_connection-blocked"));
                break;
            case SyncListener.CANCELLED:
                syncStatusController.setSyncError(sourceMask,
                        StaticDataHelper.getLanguage("status_cancelled"));
                break;
            default:
                syncStatusController.setSyncError(sourceMask,
                        StaticDataHelper.getLanguage("status_generic-error"));
                break;
        }
        // set the status into the app source
        appSource.setLastSyncStatus(status);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#endSyncing()
     */
    public void endSyncing() {
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemAddSent(java.lang.Object)
     */
    public void itemAddSent(Object arg0) {
        sentChange((SyncItem)arg0);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemDeleteSent(java.lang.Object)
     */
    public void itemDeleteSent(Object syncItem) {
        sentChange((SyncItem)syncItem);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemDeleted(java.lang.Object)
     */
    public void itemDeleted(Object itemId) {
        receivedChange();

    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemReceived(java.lang.Object)
     */
    public void itemReceived(Object arg0) {
        receivedChange();

    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemReplaceSent(java.lang.Object)
     */
    public void itemReplaceSent(Object arg0) {
        sentChange((SyncItem)arg0);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemUpdated(java.lang.Object)
     */
    public void itemUpdated(Object arg0) {
        receivedChange();
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#itemUpdated(java.lang.Object,
     * java.lang.Object)
     */
    public void itemUpdated(Object item, Object update) {
        receivedChange();
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#startConnecting()
     */
    public void startConnecting() {
        syncStatusController.setSyncStatus(sourceMask, StaticDataHelper.getLanguage("status_connecting"));
        try {
            Thread.sleep(250);
        } catch (Exception e) {
        }

    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#startMapping()
     */
    public void startMapping() {
        syncStatusController.setSyncStatus(sourceMask, StaticDataHelper.getLanguage("status_mapping"));
        try {
            Thread.sleep(250);
        } catch (Exception e) {
        }

    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#startReceiving(int)
     */
    public void startReceiving(int numItems) {
        if (totalReceiving == ITEMS_NUMBER_UNKNOWN) {
            totalReceiving = numItems;
        }
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#startSending(int, int, int)
     */
    public void startSending(int numNewItems, int numUpdItems, int numDelItems) {
        totalSending = numNewItems + numUpdItems + numDelItems;
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#startSession()
     */
    public void startSession() {
        totalReceiving = ITEMS_NUMBER_UNKNOWN;
        totalReceived = 0;
        totalSent = 0;
        totalSending = 0;
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#startSyncing(int)
     */
    public boolean startSyncing(int mode) {
        if (mode == com.funambol.syncml.protocol.SyncML.ALERT_CODE_SLOW) {
            if (Customization.getInstance().confirmSlowSync()) {
                String text = StaticDataHelper.getLanguage("status_confirm-slow");
                syncStatusController.setSyncStatus(sourceMask, text);
                if (!synchronizationController.confirmSlowSync(sourceMask)) {
                    synchronizationController.abortSlow(sourceMask);
                    return false;
                }
            }
        }
        return true;
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.util.SyncListener#syncStarted(int)
     */
    public void syncStarted(int arg0) {
    }

    private void receivedChange() {
        Log.trace("[BBPIMSyncListener.receivedChange]");

        totalReceived++;
        StringBuffer sb = new StringBuffer(StaticDataHelper.getLanguage("status_receiving-item"));
        sb.append(" ").append(totalReceived);
        if (totalReceiving != ITEMS_NUMBER_UNKNOWN) {
            sb.append("/").append(totalReceiving);
        }
        syncStatusController.setSyncStatus(sourceMask, sb.toString());
    }

    private void sentChange(SyncItem item) {

        Log.trace("[BBPIMSyncListener.sentChange]");

        if (item.hasMoreData()) {
            if (sendingLO) {
                return;
            }
            sendingLO = true;
        } else if (sendingLO) {
            // This is the last chunk
            sendingLO = false;
            return;
        }

        totalSent++;
        StringBuffer sb = new StringBuffer(StaticDataHelper.getLanguage("status_sending-item"));
        sb.append(" ").append(totalSent);
        if (totalSending > 0) {
            sb.append("/").append(totalSending);
        }
        syncStatusController.setSyncStatus(sourceMask, sb.toString());
    }

    public void removingAllData() {
        syncStatusController.setSyncStatus(sourceMask, StaticDataHelper.getLanguage("status_recover"));
    }

}
