/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Stack;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;

import net.rim.blackberry.api.pdap.PIMListListener;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.system.ControlledAccessException;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.util.Persistable;

import com.funambol.blackberry.controller.SynchronizationController;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.syncml.protocol.SyncFilter;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.protocol.SyncMLStatus;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncSource;

import com.funambol.util.SyncListener;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;

/**
 * Class for PIM sync management in BlackBerry devices
 */
public class BBPIMSyncSource implements PIMListListener, SyncSource
{
    private static final int        TRACKING_STORAGE_STATUS_NORMAL      = 0;
    private static final int        TRACKING_STORAGE_STATUS_ADD_ERR     = 1;
    private static final int        TRACKING_STORAGE_STATUS_DEL_ERR     = 2;
    private static final int        TRACKING_STORAGE_STATUS_UPD_ERR     = 4;
    
    public static final String      ERR_SAVE_ITEM_ON_SERVER_WARNING     = "message_not-save-on-server";
    
    protected int                     minCacheSize                        = 10;
    protected int                     maxCacheSize                        = 100;
    
    protected SynchronizationController controller;
    
    protected long persistenceStorageKey;
    protected long addAttributesPersistenceStorageKey;

    protected PIMList pimList;
    protected SyncSourceTrackingData trackingData;
    protected SyncSourceTrackingData trackingDataBackup;
    protected PIMItemHelper helper;
    
    protected Stack delItems;
    protected AutoCache allItems, newItems, updItems;
    
    protected int totalAdded,totalUpdated,totalDeleted;
    protected int totalSent,totalOutOfRangeNotSent,totalNewSent,totalUpdatedSent,totalDeletedSent;
    
    private String warningCode;
    
    /**
     * The number of errors in a sync
     **/
    private int errorCount;

    /**
     * The number of items to be sent to the server in the session
     **/
    private int clientItemsNumber;
    
    /**
     * The number of items that the server announced to send in the session
     **/
    private int serverItemsNumber;

    /**
     * The number of new items to be sent to the server in the session
     **/
    private int clientAddItemsNumber;

    /**
     * The number of replaced items to be sent to the server in the session 
     **/
    private int clientReplaceItemsNumber;

    /**
     * The number of deleted items to be sent to the server in the session
     **/
    private int clientDeleteItemsNumber;

    /**
     * Status of the sync source summarized in an integer value. See constants
     * defined in SyncSource
     */
    protected int globalStatus = SyncSource.STATUS_SUCCESS;

    /**
     * Listener of the sync process
     **/
    private BBPIMSyncListener listener = null;
    
    /**
     * Synchronization filter
     **/
    protected SyncFilter filter;
    
    /**
     * SyncSource configuration
     **/
    private SourceConfig config;

    protected Additionals additionalsMap;

    
    public BBPIMSyncSource(SourceConfig config,
                           PIMList list, PIMItemHelper helper,
                           long persistenceStorageKey,
                           long addAttributesPersistenceStorageKey) throws SyncException
    {
        this.controller             = controller;
        this.config                 = config;
        this.pimList                = list;
        this.helper                 = helper;
        this.persistenceStorageKey  = persistenceStorageKey;
        this.addAttributesPersistenceStorageKey = addAttributesPersistenceStorageKey;
        if(addAttributesPersistenceStorageKey!= -1){
            initAdditionalsMap();
        }

        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            bbList.addListener(this);
            initTrackingData();
        }
    
        filter = null;
    }

    public void setController(SynchronizationController controller) {
        this.controller = controller;
    }

    private void resetResource()
    {
        trackingDataBackup.clear();

        allItems = null;
        newItems = null;
        updItems = null;
        delItems = null;        

        // Init number of chages counters
        clientItemsNumber = 
        serverItemsNumber =
        clientAddItemsNumber =
        clientReplaceItemsNumber =
        clientDeleteItemsNumber = -1;
        
        totalAdded =
        totalDeleted =
        totalUpdated = 0;

        totalSent =
        totalNewSent =
        totalUpdatedSent =
        totalDeletedSent =
        totalOutOfRangeNotSent = 0;
        
        errorCount = 0;
        warningCode = null;
    }


    // This function must be called before resetResource() function
    private void stopCaching()
    {
        if (allItems != null)
        {
            allItems.stopCaching();
        }
        if (newItems != null)
        {
            newItems.stopCaching();
        }
        if (updItems != null)
        {
            updItems.stopCaching();
        }
    }


    /*
     * (non-Javadoc)
     * @see net.rim.blackberry.api.pdap.PIMListListener#itemAdded(PIMItem item)
     */
    public void itemAdded(PIMItem item)
    {
        try
        {
            Log.debug("BBPIMSyncSource - adding item from PIMListListener");
            final String uid = helper.getUID(item);
            Log.debug("SyncSource ["+config.getName()+"] added UID ["+uid+"]");
            trackingData.addNewKey(uid);
            AdditionalItemsAttributes additionals = helper.handleAddedItem(item);
            if(additionals != null){
                additionals.setLuid(uid);
                this.setAdditionals(uid, additionals);
            }
        }
        catch(final PIMException e)
        {
            trackingData.setStatus(BBPIMSyncSource.TRACKING_STORAGE_STATUS_ADD_ERR | trackingData.getStatus());
        }
        if (!persistTrackingData())
        {
            Log.error("Unable to persist tracking data");
        }
        Log.trace("itemAdded completed");
    }


    /*
     * (non-Javadoc)
     * @see net.rim.blackberry.api.pdap.PIMListListener#itemRemoved(PIMItem item)
     */
    public void itemRemoved(PIMItem item)
    {
        try
        {
            Log.debug("BBPIMSyncSource - removing item from PIMListListener");
            final String uid = helper.getUID(item);
            Log.debug("SyncSource ["+config.getName()+"] removed UID ["+uid+"]");
            trackingData.addDeletedKey(uid);
            helper.handleUpdatedItem(item);
            this.removeAdditionals(uid);
        }
        catch(final PIMException e)
        {
            trackingData.setStatus(BBPIMSyncSource.TRACKING_STORAGE_STATUS_DEL_ERR | trackingData.getStatus());
        }
        if (!persistTrackingData())
        {
            Log.error("Unable to persist tracking data");
        }
        Log.trace("itemRemoved completed");
    }


    /*
     * (non-Javadoc)
     * @see net.rim.blackberry.api.pdap.PIMListListener#itemUpdated(PIMItem oldItem,PIMItem newItem)
     */
    public void itemUpdated(PIMItem oldItem, PIMItem newItem)
    {
        try
        {
            Log.debug("BBPIMSyncSource - updating item from PIMListListener");
            final String uid = helper.getUID(newItem);
            Log.debug("SyncSource ["+config.getName()+"] updated UID ["+uid+"]");
            trackingData.addUpdatedKey(uid);
            if(additionalsMap != null){
                AdditionalItemsAttributes additionals = (AdditionalItemsAttributes) additionalsMap.get(uid);
                if(additionals != null){
                    AdditionalItemsAttributes newAdditionals = helper.updateAdditionals(newItem, additionals);
                    removeAdditionals(uid);
                    setAdditionals(uid, newAdditionals);
                }
            }

        }
        catch(final PIMException e)
        {
            trackingData.setStatus(BBPIMSyncSource.TRACKING_STORAGE_STATUS_UPD_ERR | trackingData.getStatus());
        }
        if (!persistTrackingData())
        {
            Log.error("Unable to persist tracking data");
        }
        
        Log.trace("itemUpdate completed");
    }


    private void checkTrackingData()
    {
        if (trackingData.getStatus() != BBPIMSyncSource.TRACKING_STORAGE_STATUS_NORMAL)
        {
            if (controller != null) {
                controller.serverOperationFailed();
            }
        }
    }


    private boolean initTrackingData() throws SyncException
    {
        try
        {
            final PersistentObject pobj = PersistentStore.getPersistentObject(persistenceStorageKey);
            if (pobj.getContents()==null)
            {
                trackingData = new SyncSourceTrackingData();
            }
            else
            {
                trackingData = (SyncSourceTrackingData)pobj.getContents();
            }
            return true;
        }
        catch(ControlledAccessException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,"The application could not access to the TrackingData for SyncSource ["+getName()+"]");
        }
    }


    private boolean persistTrackingData() throws SyncException
    {
        try
        {
            final PersistentObject pobj = PersistentStore.getPersistentObject(persistenceStorageKey);
            pobj.setContents(trackingData);
            pobj.commit();
            return true;
        }
        catch(ControlledAccessException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,"The application could not access to the TrackingData for SyncSource ["+getName()+"]");
        }
    }
    
    //----------------------------------------------------------- Public Methods
    
    /**
     * Returns the config of the source. The client can use this method
     * to obtain the config object and change some parameter. A setConfig()
     * must be called to actually change the source configuration.
     * @return the config of the source
     */
    public SourceConfig getConfig() 
    {
        return config;
    }

    public void setConfig(SourceConfig config)
    {
        this.config = config;
    }
    
    //------------------------------------------------ SyncSource implementation
    /**
     * Returns the name of the source
     * @return the name of the source
     */
    public String getName() 
    {
        return getConfig().getName();
    }

 
    /**
     * Returns the source URI
     * @return the absolute URI of the source
     */
    public String getSourceUri() 
    {
        return getConfig().getRemoteUri();
    }

 
    /**
     * Returns the type of the source.
     * The types are defined as mime-types, for instance * text/x-vcard).
     * @return the type of the source
     */
    public String getType() 
    {
        return getConfig().getType();
    }


    /**
     * Returns the encoding of the source.
     * The encoding can be 'b64' or 'none' only. The standard defines
     * also 'des' and '3des' but they are not implemented in this version
     * of the APIs.
     * @return the encoding of the source
     */
    public String getEncoding() 
    {
        return getConfig().getEncoding();
    }


    /**
     * Returns the preferred sync mode of the source.
     * The preferred sync mode is the one that the SyncManager sends
     * to the server in the initialization phase. The server can respond
     * with a different alert code, to force, for instance, a slow.
     * @return the preferred sync mode for this source
     */
    public int getSyncMode() 
    {
        return getConfig().getSyncMode();
    }

    /**
     * Returns the current filter for this SyncSource.
     */
    public SyncFilter getFilter() 
    {
        return filter;
    }
     
    /**
     * Set a new filter for this SyncSource
     */
    public void setFilter(SyncFilter filter) 
    {
        this.filter = filter;
    }
    
    /** 
     * Returns the next item of the store.
     **/
    public SyncItem getNextItem() throws SyncException 
    {
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        if (allItems==null)
        {
            Log.info("Source "+getName()+": no more items to send for slow sync");
            return null;
        }                

        SyncItem item = (SyncItem)allItems.getNextItem();
        if (item!=null)
        {
            totalSent++;
            Log.info("Source "+getName()+": sending item "+item.getKey());
            return item;
        }
        else
        {
            Log.info("Source "+getName()+": no more items to send for slow sync");
            // All Items sent, we can free memory
            allItems = null;
            return null;
        }
    }
     
    /**
     * Returns the next new item of the store
     * (not yet sent to the server)
     */
    public SyncItem getNextNewItem() throws SyncException 
    {
        
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        if (newItems == null) 
        {
            Log.info("Source "+getName()+": no more new items to send");
            return null;
        }

        SyncItem item = (SyncItem)newItems.getNextItem();
        if (item!=null) 
        {
            totalSent++;
            totalNewSent++;
            Log.info("Source "+getName()+": sending item "+item.getKey());
            return item;
        }
        else 
        {
            Log.info("Source "+getName()+": no more new items to send");
            // All Items sent, we can free memory
            newItems = null;
            return null;
        }
    }
     
    /**
     * Returns the first/next updated item of the store
     * (changed from the last sync)
     */
    public SyncItem getNextUpdatedItem() throws SyncException 
    {
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        if (updItems == null) 
        {
            Log.info("Source "+getName()+": no more updated items to send");
            return null;
        }

        SyncItem item = (SyncItem)updItems.getNextItem();
        if (item!=null) 
        {
            totalSent++;
            totalUpdatedSent++;
            Log.info("Source "+getName()+": sending item "+item.getKey());
            return item;
        }
        else 
        {
            Log.info("Source "+getName()+": no more updated items to send");
            // All Items sent, we can free memory
            updItems = null;
            return null;
        }
    }
     
    /** 
     * Returns a SyncItem containing the key of the first/next
     * deleted item of the store (locally removed after the last sync,
     * but not yet deleted on server)
     */
    public SyncItem getNextDeletedItem() throws SyncException 
    {
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        if (delItems == null) 
        {
            Log.info("Source "+getName()+": no more deleted items to send");
            return null;
        }
        if (!delItems.empty()) 
        {
            SyncItem item = (SyncItem)delItems.pop();
            totalSent++;
            totalDeletedSent++;
            Log.info("Source "+getName()+": sending item "+item.getKey());
            // No need to get the content here
            return item;
        }
        else 
        {
            Log.info("Source "+getName()+": no more deletetd items to send");
            // All Items sent, we can free memory
            delItems = null;
            return null;
        }
    } 
     
    /**
     * Return the number of changes that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientItemsNumber() 
    {
        return clientItemsNumber;
    }
     
    /**
     * Return the number of new items (add) that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of new items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientAddNumber() 
    {
        return clientAddItemsNumber;
    }
     
    /**
     * Return the number of replaced items that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of replaced items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientReplaceNumber() 
    {
        return clientReplaceItemsNumber;
    }
     
    /**
     * Return the number of deleted items that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of delted items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientDeleteNumber() 
    {
        return clientDeleteItemsNumber;
    }
     
    /**
     * Return the number of changes that the server will send during the
     * session. This method, after the beginSync() call, should return
     * the number of items to be sent to the server.
     * @return number of changes from the server, or -1 if not announced.
     */
    public int getServerItemsNumber() 
    {
        return serverItemsNumber;
    }
     
    /**
     * Set the number of changes that the server will send during the
     * session. This method is called by the engine to notify the Source
     * of the number of changes announced by the server. If the server
     * does not announce the number of changes, the engine will call
     * this method with parameter -1.
     * @param number of changes from the server, or -1 if not announced.
     */
    public void setServerItemsNumber(int number) 
    {
        serverItemsNumber = number;
    }
     
    /** 
     * Return the Last Anchor for this source
     */
    public long getLastAnchor() 
    {
        return getConfig().getLastAnchor();
    }
     
    /** 
     * Set the value of the Last Anchor for this source
     */
    public void setLastAnchor(long time) 
    {
        getConfig().setLastAnchor(time);
    }
     
    /** 
     * Return the Next Anchor for this source
     */
    public long getNextAnchor() 
    {
        return getConfig().getNextAnchor();
    }
     
    /** 
     * Set the value of the Next Anchor for this source
     */
    public void setNextAnchor(long time) 
    {
        getConfig().setNextAnchor(time);
    }
     
    /**
     * Called after SyncManager preparation and initialization just before start
     * the synchronization of the SyncSource.
     * @param syncMode the synchronization type: one of the values in
     *         sync4j.framework.core.AlertCode
     * @throws SyncException in case of error. This will stop the sync process
     */
    public void beginSync(int syncMode) throws SyncException 
    {
        //System.out.println("Begin sync for source- "+ getName() + " : " + Calendar.getInstance().getTime().toString());
        Log.info("Begin sync for source '" + getName() + "' with mode " + syncMode);

        warningCode = null;
        
        helper.init();
        
        errorCount = 0;
        totalAdded = 0;
        totalDeleted = 0;
        totalUpdated = 0;

        checkTrackingData();
        trackingDataBackup = (SyncSourceTrackingData)trackingData.clone();

        // Init lists
        switch(syncMode)
        {
            case SyncML.ALERT_CODE_REFRESH_FROM_CLIENT:
                Log.trace("SyncML.ALERT_CODE_REFRESH_FROM_CLIENT");
                // A refresh from client is like a slow here
            case SyncML.ALERT_CODE_SLOW:
                Log.trace("SyncML.ALERT_CODE_SLOW");
                initAllItems();
                
                // Init number of changes counter
                clientAddItemsNumber     = (newItems != null) ? newItems.size() : 0 ;
                clientReplaceItemsNumber = (updItems != null) ? updItems.size() : 0 ;
                clientDeleteItemsNumber  = (delItems != null) ? delItems.size() : 0 ;
                
                clientItemsNumber        = (allItems != null) ? allItems.size() : 0 ;
                break;
                
            case SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT:
                Log.trace("SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT");
                // A one way from client is like a fast here
            case SyncML.ALERT_CODE_FAST:
                Log.trace("SyncML.ALERT_CODE_FAST");
                initNewItems();
                initUpdItems();
                initDelItems();
                // Init number of changes counter
                clientAddItemsNumber     = (newItems != null) ? newItems.size() : 0 ;
                clientReplaceItemsNumber = (updItems != null) ? updItems.size() : 0 ;
                clientDeleteItemsNumber  = (delItems != null) ? delItems.size() : 0 ;
                
                clientItemsNumber        = clientAddItemsNumber + clientReplaceItemsNumber + clientDeleteItemsNumber;
                break;
                
            case SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER:
                Log.trace("SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER");
                // No modifications to send (it's not
                // strictly necessary to reset the lists,
                // because the engine will not ask items to
                // the SyncSource, but it's good to do it)
                newItems = null;
                updItems = null;
                delItems = null;
                
                // Init number of changes counter
                clientAddItemsNumber     = 0;
                clientReplaceItemsNumber = 0;
                clientDeleteItemsNumber  = 0;
                
                clientItemsNumber        = 0;
                break;
                
            case SyncML.ALERT_CODE_REFRESH_FROM_SERVER:
                Log.trace("SyncML.ALERT_CODE_REFRESH_FROM_SERVER");
                // In this case, the SyncSource should
                // delete all the items in the database
                // (possibly asking the user before that)
                // No modifications to send.
                newItems = null;
                updItems = null;
                delItems = null;

                // Init number of changes counter
                clientAddItemsNumber     = 0;
                clientReplaceItemsNumber = 0;
                clientDeleteItemsNumber  = 0;
                
                clientItemsNumber        = 0;
                try
                {
                    listener.removingAllData();
                    helper.deleteAllItems();
                }
                catch(final Exception e)
                {
                    throw new SyncException(SyncException.CLIENT_ERROR, "SyncSource "+getName()+" could not delete all PIMItems. Please manually delete them before re-sync");
                }
                
                // The following code lines are important to ensure the client throws away 
                // all tracking data before recover from server
                trackingData.clear();
                trackingDataBackup.clear();
                persistTrackingData();
                break;
                
            default:
                throw new SyncException(SyncException.SERVER_ERROR, "SyncSource "+getName()+
                                        ": invalid sync mode "+getSyncMode());
        }

        totalSent = 0;
        Log.debug("For source ["+getName()+"] There are ["+clientAddItemsNumber+"] new items should be sent");
        Log.debug("For source ["+getName()+"] There are ["+clientDeleteItemsNumber+"] deleted items should be sent");
        Log.debug("For source ["+getName()+"] There are ["+clientReplaceItemsNumber+"] updated items should be sent");
        Log.debug("For source ["+getName()+"] There are ["+clientItemsNumber+"] ALL items should be sent");
        
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
    }
     
    /**
     * Called just before committing the synchronization process by the
     * SyncManager. The SyncSource can stop the commit phase raising an
     * exception here.
     * @throws SyncException in case of error, to stop the commit.
     */
    public void endSync() throws SyncException
    {
        //System.out.println("End sync for source- "+ getName() + " : " + Calendar.getInstance().getTime().toString());
        Log.info("End sync for source " + getName());
        Log.info(
                "For source ["+getName()+"]: \n"+
                "there were ["+totalSent+"] total items sent \n" +
                "there were ["+totalOutOfRangeNotSent+"] total OutOfRange items were not sent \n" +
                
                "there were ["+totalNewSent    +"] NEW items sent \n" +
                "there were ["+totalDeletedSent+"] DELETED items sent \n" +
                "there were ["+totalUpdatedSent+"] UPDATED items sent \n" +
                
                "there were ["+totalAdded  +"] total items tried to add \n" +
                "there were ["+totalDeleted+"] total items tried to delete \n" +
                "there were ["+totalUpdated+"] total items tried to update \n"
        );

        // Release resources
        resetResource();
        
        //trackingData.clear();
        trackingDataBackup.clear();
        persistTrackingData();
    }
     
    /**
     * Set a sync listener.
     * @param listener the listener or null to remove it
     */
    public void setListener(SyncListener listener) 
    {
        this.listener = (BBPIMSyncListener)listener;
    }
     
    /**
     * Returns the current listener (or null if not set)
     */
    public SyncListener getListener() 
    {
        return listener;
    }
     
    /**
     * Returns the status of the sync source. The status is encoded as a bit
     * mask of the STATUS_* values
     */
    public int getStatus() 
    {
        return globalStatus;
    }
     
    public void dataReceived(String date, int size)
    {
        //Do nothing here
    }

    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#addItem(com.funambol.syncml.spds.SyncItem)
     */
    public int addItem(SyncItem item) throws SyncException
    {
        //Log.info("Total memory free is ["+Runtime.getRuntime().freeMemory()+"] bytes");
        //Log.info("Total memory is ["+Runtime.getRuntime().totalMemory()+"] bytes");
        totalAdded++;
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        try
        {
            Log.trace("Before creating a blank PIMItem.");
            PIMItem pitem = helper.createItem();
            if (pitem!=null)
            {
                Log.trace("Before calling setPIMProperties function.");
                
                AdditionalItemsAttributes addits = null;
                
                AdditionalItemsAttributes additionals = helper.fillItem(pitem,new String(item.getContent()),addits);

                Log.trace("Before commiting PIMItem.");
                try {
                    helper.saveNewPIMItem(pitem);
                } catch (PIMException e) {
                    String error = helper.handlePIMException(e);
                    logError("Unable to save new item in source [" + getName() + "]: " + error);
                    return SyncMLStatus.GENERIC_ERROR;
                }
                Log.trace("Before trying to get UID of PIMItem.");
                String key = helper.getUID(pitem);
                item.setKey(key);
                if(additionals != null){
                    additionals.setLuid(key);
                    setAdditionals(key, additionals);
                }

                Log.trace("Purposfully deleting new item key: " + key);
                trackingData.deleteNewKey(key,true);
                trackingDataBackup.deleteNewKey(key);

                key = null;
                return SyncMLStatus.SUCCESS;
            }
            else
            {
                Log.trace("Can not create a blank PIMItem.");
            }
        } catch(final PIMException e) {
            String error = helper.handlePIMException(e);
            logError("PIMException while adding item to SyncSource [" + getName() + "]: " + error);
            return SyncMLStatus.GENERIC_ERROR;
        } catch(final Exception e) {
            logError("RuntimeException while updating item to SyncSource ["
                      + getName() + "]: " + e.toString());
            return SyncMLStatus.GENERIC_ERROR;
        }
        logError("Unable to save new item in source [" + getName() + "]: no error");
        return SyncMLStatus.GENERIC_ERROR;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#updateItem(com.funambol.syncml.spds.SyncItem)
     */
    public int updateItem(SyncItem item) throws SyncException
    {
        //Log.info("Total memory free is ["+Runtime.getRuntime().freeMemory()+"] bytes");
        //Log.info("Total memory is ["+Runtime.getRuntime().totalMemory()+"] bytes");
        totalUpdated++;
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        try
        {
            Log.trace("Before searching for existed PIMItem.");
            PIMItem pitem = helper.findItem(item.getKey());
            if (pitem!=null)
            {
                Log.trace("Before clearing properties of found PIMItem.");
                AdditionalItemsAttributes additionals = null;
                
                if(additionalsMap != null){
                    additionals = (AdditionalItemsAttributes)additionalsMap.get(helper.getUID(pitem));
                }
                
                helper.clearItem(pitem);
                Log.trace("Before calling setPIMProperties functions.");

                AdditionalItemsAttributes newAdditionals = helper.fillItem(pitem,new String(item.getContent()),additionals);
               
                Log.trace("Before commiting PIMItem.");
                try {
                    helper.saveUpdatedPIMItem(pitem);
                    
                } catch (PIMException e) {
                    String error = helper.handlePIMException(e);
                    logError("Unable to save updated item in source [" + getName() + "]: " + error);
                    return SyncMLStatus.GENERIC_ERROR;
                }
                Log.trace("Before trying to get UID of PIMItem.");
                String key = helper.getUID(pitem); 
                item.setKey(key);

                if(newAdditionals != null){
                    removeAdditionals(key);
                    setAdditionals(key, newAdditionals);
                }

                Log.trace("Purposfully deleting updated item key: " + key);
                trackingData.deleteUpdatedKey(key,true);
                trackingDataBackup.deleteUpdatedKey(key);
                
                return SyncMLStatus.SUCCESS;
            }
            logError("Unable to save updated item in source [" + getName() + "]: item not found");
            return SyncMLStatus.NOT_FOUND;
        } catch(final PIMException e) {
            String error = helper.handlePIMException(e);
            logError("PIMException while updating item to SyncSource [" + getName() + "]: " + error);
            return SyncMLStatus.GENERIC_ERROR;
        } catch (Exception e) {
            // We had some sort of generic runtime exception
            logError("RuntimeException while updating item to SyncSource ["
                      + getName() + "]: " + e.toString());
            return SyncMLStatus.GENERIC_ERROR;
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#deleteItem(java.lang.String)
     */
    public int deleteItem(java.lang.String item) throws SyncException
    {
        //Log.info("Total memory free is ["+Runtime.getRuntime().freeMemory()+"] bytes");
        //Log.info("Total memory is ["+Runtime.getRuntime().totalMemory()+"] bytes");
        totalDeleted++;
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        try
        {
            Log.trace("Before trying to find and delete existed PIMItem.");
            if (helper.deleteItemFromKey(item)!=null)
            {
                Log.trace("Purposfully deleting deleted item key: " + item);
                trackingData.deleteDeletedKey(item,true);
                trackingDataBackup.deleteDeletedKey(item);
                removeAdditionals(item);
                return SyncMLStatus.SUCCESS;
            }
            logError("Unable to delete item in source [" + getName() + "]: no error");
            return SyncMLStatus.NOT_FOUND;
        }
        catch(final Exception e)
        {
            logError("Exception while deleting item from SyncSource ["+getName()+"]");
            return SyncMLStatus.GENERIC_ERROR;
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#getItemContent(com.funambol.syncml.spds.SyncItem)
     */
    protected SyncItem getItemContent(SyncItem item) throws SyncException
    {
        if (controller != null && controller.isCancelled()) {
            cancelSync();
        }
        try
        {
            PIMItem pitem = helper.findItem(item.getKey());
            AdditionalItemsAttributes additionals = null;
            if(additionalsMap != null){
                additionals = (AdditionalItemsAttributes) additionalsMap.get(helper.getUID(pitem));
            }

            if (pitem!=null)
            {
                item.setContent(helper.itemToString(pitem,additionals).getBytes());
                return item;
            }
        }
        catch(final PIMException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "Exception while retrieving content of item from SyncSource ["
                                    +getName()+"]");
        }
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initAllItems()
     */
    protected void initAllItems() throws SyncException
    {
        try
        {
            //allItems = new Stack();
            //allItemsCache = new Stack();
            String key = null;
            Stack temp = new Stack();
            Enumeration items = helper.getItemsList();
            while(items.hasMoreElements())
            {
                key = helper.getUID((PIMItem)items.nextElement());
                //allItems.push(new SyncItem(key));
                temp.push(new SyncItem(key));
            }
            items = null;
            key = null;
            allItems = new AutoCache(helper,pimList,temp,minCacheSize,maxCacheSize,getName());
        }
        catch(final PIMException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "Exception while initializing all items of SyncSource ["
                                    +getName()+"]");
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initDelItems()
     */
    protected void initDelItems()
    {
        Log.debug("Start building DelItems list");
        delItems = new Stack();
        String uid = null;
        while((uid=trackingData.popDeletedKey())!=null)
        {
            // This is a workaround for a BB bug. In some cases
            // the systems sends bogus delete notifications for all items.
            // For this reason before sending a delete command to the server
            // we double check if the item is still present
            try{
                if(helper.findItem(uid)== null ){
                    String luid = getLuidFromUID(uid);
                    Log.trace("Popped key from tracking database ["+uid+"]");
                    Log.trace("Creating a sync item to be deleted with Luid ["+luid+"]");
                    delItems.push(new SyncItem(luid));
                }else{
                    Log.error("Detected spurious delete for item: " + uid);
                }
            } catch (PIMException e) {
                String msg = helper.handlePIMException(e);
                Log.error("BBPIMSyncSource - Error finding item creating delItems stack: " + msg);
            }
        }
        uid = null;
        Log.trace("End building DelItems list");
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initNewItems()
     */
    protected void initNewItems()
    {
        Log.debug("Start building NewItems list");
        //newItems = new Stack();
        //newItemsCache = new Stack();
        String key = null;
        Stack temp = new Stack();
        while((key=trackingData.popNewKey())!=null)
        {
            temp.push(new SyncItem(key));
            Log.trace("Pop key from tracking database ["+key+"]");
        }
        key = null;
        Log.trace("End building NewItems list");
        newItems = new AutoCache(helper,pimList,temp,minCacheSize,maxCacheSize,getName());
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initUpdItems()
     */
    protected void initUpdItems()
    {
        //updItems = new Stack();
        //updItemsCache = new Stack();
        Log.trace("Start building UpdItems list");
        String key = null;
        Stack temp = new Stack();
        while((key=trackingData.popUpdatedKey())!=null)
        {
            //updItems.push(new SyncItem(key));
            temp.push(new SyncItem(key));
            Log.debug("Pop key from tracking database ["+key+"]");
        }
        key = null;
        Log.debug("End building UpdItems list");
        updItems = new AutoCache(helper,pimList,temp,minCacheSize,maxCacheSize,getName());
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#setItemStatus(java.lang.String key, int status)
     */
    private static final int ERROR_STATUS_CLASS   =   5;
    public void setItemStatus(String key, int status)
    {
        if (status/100 >= ERROR_STATUS_CLASS)
        {
            warningCode = BBPIMSyncSource.ERR_SAVE_ITEM_ON_SERVER_WARNING;
            Log.error("Could not save to server item with UID ["+key+"] for SyncSource ["+getName()+"] returned status ["+status+"]");
        }
        trackingDataBackup.deleteNewKey(key);
        trackingDataBackup.deleteUpdatedKey(key);
        trackingDataBackup.deleteDeletedKey(key);        
    }


    public String getWarningCode()
    {
        return warningCode;
    }


    public void restoreTrackingData()
    {
        if (trackingDataBackup != null)
        {
            trackingData.append(trackingDataBackup);
        }
    }


    protected void cancelSync() throws SyncException
    {
        Log.info("Cancelling sync for source ["+getName()+"]");
        restoreTrackingData();
        stopCaching();
        resetResource();
        throw new SyncException(SyncException.CANCELLED, StaticDataHelper.getLanguage("message_cancelled"));
    }
   
    public boolean shouldWarn()
    {
        int deleted = trackingData.countDeletedKey();

        try{
            int remainders = 0;
            Enumeration items = pimList.items();
            while(items.hasMoreElements())
            {
                remainders++;
                items.nextElement();
            }
                
            int total = deleted + remainders;
            Log.info("Considering warning user about " + deleted + " deletes when " + remainders + " items remain");
               
            //if (deleted > (.5 * remainders))
            if (deleted > ((2*total)/3))
            { // 2/3 deleted or mail account deleted bug
                return true;
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return false;
    }


    /**
     * By default UID and Luid are the same. This method can be overloaded by
     * subclasses that want different values for luids and uids
     */
    protected String getLuidFromUID(String uid) {
        if(additionalsMap == null){
            return uid;
        }else{
            AdditionalItemsAttributes addits = (AdditionalItemsAttributes)additionalsMap.get(uid);
            if(addits== null){
                return uid;
            }else{
                return addits.getLuid();
            }
        }
    }

    class AutoCache extends Thread
    {
        private PIMItemHelper      _helper;
        private Stack              _inputSyncItemList;
        private Stack              _bufferedSyncItemList;
        private String             _name;
        private int                _size;
        private int                _maxcachesize;
        private int                _mincachesize;
        private int                _totalOutOfRangeNotSent;
        private int                _cacheSize;
        private boolean            _stopFlag;
        private boolean            _firstRun;
        private Integer            _totalItemsLeft;


        public AutoCache(PIMItemHelper helper, PIMList pimlist, Stack input_itemlist, int mincachesize, int maxcachesize, String name)
        {
            _helper = helper;
            _inputSyncItemList = input_itemlist;
            _size = _inputSyncItemList.size();
            _mincachesize = mincachesize;
            _maxcachesize = maxcachesize;
            _name = name;

            _totalOutOfRangeNotSent = 0;
            _bufferedSyncItemList = new Stack();
            _cacheSize = _maxcachesize < _size/10 ? _maxcachesize : _size/10;
            _cacheSize = _cacheSize < _mincachesize ? _mincachesize : _cacheSize;
            _stopFlag = false;
            _firstRun = true;
            _totalItemsLeft = new Integer(_size);
            
        }


        public int size()
        {
            return _size;
        }


        public synchronized boolean empty()
        {
            return (_bufferedSyncItemList.empty() && _inputSyncItemList.empty() && _totalItemsLeft.intValue()==0);
        }


        public SyncItem getNextItem()
        {
            if (_firstRun)
            {
                cacheItemContent();
                _firstRun = false;
                start();
            }

            boolean wait = false;
            do
            {
                int available = 0;
                // get the number of items available immediately
                synchronized(_bufferedSyncItemList)
                {
                    available = _bufferedSyncItemList.size();
                }
                
                if (available>0)// There are some item to read immediately
                {
                    synchronized(_bufferedSyncItemList)
                    {
                        Log.info("Return the first available SyncItem");
                        return (SyncItem)_bufferedSyncItemList.pop();
                    }
                }
                else // There is no item to read immediately
                {
                    // get the number of items left
                    synchronized(_totalItemsLeft)
                    {
                        wait = (_totalItemsLeft.intValue()>0);
                    }
                    // There is no item left, return null;
                    if (!wait)
                    {
                        Log.info("There is no more SyncItem to read");
                        return null;
                    }
                    // There are some items left so we wait to read them
                    else
                    {
                        Log.info("Waiting to read the next available SyncItem");
                        try{ Thread.sleep(500); } catch(final InterruptedException e) {}
                    }
                }
            }
            while(wait);
            return null;
        }


        // This is a force to-stop. Clear all data in queue and in cache
        public synchronized void stopCaching()
        {
            Log.info("Trying to stop caching data items");
            _stopFlag = true;
            _inputSyncItemList.removeAllElements();
            _bufferedSyncItemList.removeAllElements();
            _totalItemsLeft = new Integer(0);
        }


        private void cacheItemContent()
        {
            try
            {
                int size = 0;
                int realsize = 0;
                synchronized(_totalItemsLeft)
                {
                    _totalItemsLeft = new Integer(_inputSyncItemList.size());
                    realsize = _totalItemsLeft.intValue();
                }
                synchronized(_bufferedSyncItemList)
                {
                    size = _cacheSize - _bufferedSyncItemList.size();
                }
                size = realsize < size ? realsize : size;
            
                //Log.info("_TotalItemLeft ["+_TotalItemLeft+"] items content for SyncSource ["+_name+"]");
                //Log.info("Realsize ["+realsize+"] items content for SyncSource ["+_name+"]");
                Log.info("Start trying to cache ["+size+"] items content for SyncSource ["+_name+"]");
                Stack keys = new Stack();
                for(int i=0;i<size;i++)
                {
                    keys.push(((SyncItem)_inputSyncItemList.pop()).getKey());
                }
                
                Log.info("Before calling find_and_deletePIMItem function");
                Hashtable out = _helper.findItems(keys);
                keys.removeAllElements();
                keys = null;
                Enumeration ks = out.keys();
                while(ks.hasMoreElements())
                {
                    String itemKey = (String)ks.nextElement();
                    String luid;
                    PIMItem pimItem = (PIMItem)out.get(itemKey);
                    if (_helper.isInSyncRange(pimItem))
                    {
                        luid = getLuidFromUID(itemKey);
                        SyncItem item = new SyncItem(luid);
                        AdditionalItemsAttributes additionals = null;
                         
                        if(additionalsMap != null){
                            additionals = (AdditionalItemsAttributes) additionalsMap.get(helper.getUID(pimItem));
                            if (additionals == null){
                                additionals = helper.createAdditionals(pimItem);
                            }
                        }
                        
                        
                        item.setContent(_helper.itemToString(pimItem,additionals).getBytes());
                        Log.debug("PIM Item content ["+new String(item.getContent())+"]");
                        synchronized(_bufferedSyncItemList)
                        {
                            _bufferedSyncItemList.addElement(item);
                            if(additionals != null){
                                additionals.setLuid(luid);
                                setAdditionals(itemKey,additionals);
                            }
                        }
                        Log.info("Caching SyncItem with UID ["+itemKey+"] and LUID [" + luid + "]");
                    }
                    else
                    {
                        _totalOutOfRangeNotSent++;
                        Log.info("Out of Sync range item with UID ["+itemKey+"] will not be sent to server.");
                    }
                }
                out = null;
                ks = null;
                synchronized(_totalItemsLeft)
                {
                    _totalItemsLeft = new Integer(_inputSyncItemList.size());
                }
            }
            catch(final Throwable e)
            {
                throw new SyncException(SyncException.CLIENT_ERROR,"Exception while caching item's content from SyncSource ["+_name+"] "+e.getMessage());
            }
        }


        public void run()
        {
            while(!_stopFlag)
            {
                synchronized(_totalItemsLeft)
                {
                    _stopFlag = (_totalItemsLeft.intValue()==0);
                }
                if (_stopFlag)
                {
                    Log.info("There is no more data to cache. set stopFlag = true");
                }
                else 
                {
                    int size = 0;
                    synchronized(_bufferedSyncItemList)
                    {
                        size = _bufferedSyncItemList.size();
                    }
                    if (size<=_cacheSize/2)
                    {
                        cacheItemContent();
                    }
                    else
                    {
                        yield();
                    }
                }
            }
        }
    }


    protected void logError(String message)
    {
        Log.error(message);
        errorCount++;
    }


    public int getErrorCount()
    {
        return errorCount;
    }
    
    public PIMItemHelper getHelper()
    {
        return helper;
    }

    private boolean initAdditionalsMap() throws SyncException
    {
        try
        {
            Log.info("[initAdditionalsMap]");
            final PersistentObject pobj = PersistentStore.getPersistentObject(addAttributesPersistenceStorageKey);
            if (pobj.getContents()==null)
            {
                additionalsMap = new Additionals();
            }
            else
            {
                additionalsMap = (Additionals)pobj.getContents();
            }
            return true;
        }
        catch(Exception e)
        {
            Log.error("Cannot load Additional Attributes Map: " + e.toString());
            throw new SyncException(SyncException.CLIENT_ERROR, "Cannot create Additional Attributes map");
        }
    }

     private boolean persistAdditionalsMap() throws SyncException
    {
        Log.info("[persistAdditionalsMap]");
        try
        {
            final PersistentObject pobj = PersistentStore.getPersistentObject(addAttributesPersistenceStorageKey);
            pobj.setContents(additionalsMap);
            pobj.commit();
            return true;
        }
        catch(ControlledAccessException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "The application could not persist persistAdditionalsMap");
        }catch(Exception e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "The application could not persist persistAdditionalsMap");
        }

    }

    protected void setAdditionals(String uid, AdditionalItemsAttributes addits) throws PIMException, SyncException {
        Log.info("[setAdditionals] " + uid );
        if(additionalsMap != null){
            additionalsMap.put(uid, addits);
            persistAdditionalsMap();
        }else{
            Log.debug("[setAdditionals] additionalMap does not initializated; " +
                    "additionals does not supported for SyncSource "+config.getName());
        }
    }

    protected void removeAdditionals(String uid) {
        Log.debug("[removeAdditionals] " + uid);
        if(additionalsMap != null){
            additionalsMap.remove(uid);
            persistAdditionalsMap();
        }else{
            Log.debug("[removeAdditionals] additionalMap does not initializated; " +
                    "additionals does not supported for SyncSource "+config.getName());
        }
    }


     /**
      * This class holds the uid - additionals attributes for an item
      * and can be persisted to the BB storage.
      */
    public static final class Additionals implements Persistable {
        private Hashtable addAttributesMap;

        public Additionals() {
            Log.debug("BBPIMSyncSource - creating additionals map");
            addAttributesMap = new Hashtable();
        }

        public AdditionalItemsAttributes put(String uid, AdditionalItemsAttributes additionalAttributes) {
            return (AdditionalItemsAttributes)this.addAttributesMap.put(uid, additionalAttributes);
        }

        public AdditionalItemsAttributes get(String uid) {
            return (AdditionalItemsAttributes)this.addAttributesMap.get(uid);
        }

        public Enumeration keys() {
            return this.addAttributesMap.keys();
        }

        public void remove(String uid) {
            this.addAttributesMap.remove(uid);
        }

    }
}
