/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import java.util.Vector;
import net.rim.device.api.util.Persistable;
import com.funambol.util.Log;


/**
 * Base class for additionals attributes of itmes that not have
 * the correct fields in the native applications
 */
public class ContactAdditionalAttributes extends AdditionalItemsAttributes implements Persistable
{
    private Vector contactEmails;

    public static final int EMAIL_INTENET = 0;
    public static final int EMAIL_HOME = 1;
    public static final int EMAIL_WORK = 2;

    public static final int FAX_NONE = 0;
    public static final int FAX_WORK = 1;
    public static final int FAX_HOME = 2;
    public static final int FAX_OTHER = 3;

    private int faxTypeStored = FAX_NONE;

    public ContactAdditionalAttributes()
    {
        contactEmails = new Vector();
        EmailItem emailInternet = new EmailItem(EMAIL_INTENET, "", false);
        EmailItem emailHome = new EmailItem(EMAIL_HOME, "", false);
        EmailItem emailWork = new EmailItem(EMAIL_WORK, "", false);
    }

    public void setFaxType (int faxType){
        Log.debug("Setting fax type: " + faxType);
        faxTypeStored = faxType;
    }

    public int getFaxType (){
        return faxTypeStored;
    }

    public void addEmail(int emailAttribute, String email, boolean preferred){
        EmailItem emailItem = new EmailItem(emailAttribute, email, preferred);
        contactEmails.addElement(emailItem);

    }

    public void setEmail(int emailAttribute, String email, boolean preferred){
        EmailItem emailItem = null;
        try{
            emailItem = (EmailItem)contactEmails.elementAt(emailAttribute);
            emailItem.setEmailAddress(email);
            emailItem.setPreferred(preferred);
            contactEmails.setElementAt(emailItem, emailAttribute);
        }catch (ArrayIndexOutOfBoundsException aex){
            emailItem = new EmailItem(emailAttribute, email, preferred);
            contactEmails.addElement(emailItem);
        }


        

    }

    public String[] getContactEmailsToStore(){

        String[] emailsToReturn = new String [3];
        String internetToSend = "";
        String homeToSend = "";
        String workToSend = "";
        
        for(int i = 0; i<contactEmails.size(); i++){
           EmailItem ei = (EmailItem)contactEmails.elementAt(i);
           int type = ei.getEmailAttribute();
           if(type == EMAIL_INTENET){
               internetToSend = ei.getEmailAddress();
           }else if(type == EMAIL_HOME){
               homeToSend = ei.getEmailAddress();
           }else if(type == EMAIL_WORK){
               workToSend = ei.getEmailAddress();
           }
        }

        emailsToReturn[0] = internetToSend;
        emailsToReturn[1] = homeToSend;
        emailsToReturn[2] = workToSend;

        return emailsToReturn;

    }

    public String getEmail(int emailAttribute){

        try{
            EmailItem emailItem = (EmailItem)contactEmails.elementAt(emailAttribute);
            return emailItem.getEmailAddress();
        }catch (ArrayIndexOutOfBoundsException aex){
            return "";
        }
       
    }


    public class EmailItem implements Persistable{

        private int emailAttribute; // 0 = internet email, 1 = home email, 2 = work email
        private String email;
        boolean isPreferred;

        public EmailItem(int attribute, String emailAddress, boolean preferred){

            emailAttribute = attribute;
            email = emailAddress;
            isPreferred = preferred;
        }

        public int getEmailAttribute(){
            return emailAttribute;
        }

        public String getEmailAddress(){
            return email;
        }

        public boolean isPreferred(){
            return isPreferred;
        }

        public void setEmailAddress(String emailAddress){
            email = emailAddress;
        }

        public void setPreferred(boolean preferred){
            isPreferred = preferred;
        }

    }


    
}
