package com.funambol.blackberry.ui;

public interface DisplayManager extends Screen {

    public static final long NO_LIMIT = -1;

    /**
     * Pull a screen to the back
     * 
     * @param scr
     *            The screen to hide
     */
    public void hideScreen(Screen scr) throws Exception;

    public void showScreen(Screen scr) throws Exception;

    public void showScreen(Screen scr, boolean donotwait) throws Exception;
    
    /**
     * Helper function to prompt the user for a yes/no answer
     * 
     * @param question
     * @param defaultyes
     * @param timeToWait
     *            TODO
     * @return True on yes
     */
    public boolean askYesNoQuestion(String question, boolean defaultyes, long timeToWait);

    /**
     * Helper function to prompt the user for an accept/deny answer
     * 
     * @param question
     * @param defaultaccept
     * @param timeToWait
     *            TODO
     * @return True on accept
     */
    public boolean askAcceptDenyQuestion(String question, boolean defaultyes, long timeToWait);



    /**
     * Helper function to prompt to continue or cancel
     * 
     * @param message
     * @return True for next
     */
    public boolean promptNext(String message);

    /**
     * Helper function to display a simple notice
     * 
     * @param message
     */
    public void showMessage(String message);

    /**
     * Prompt the user to select an option from a list
     * 
     * @param message
     *            The message to prompt the user with
     * @param options
     *            An array of option names
     * @param values
     *            An array of option values
     * @param defaultValue
     *            The default selected value
     * @param needResponse
     *            TODO
     * @param cancelAfterDelay
     *            TODO
     * @return The chosen value
     */
    public int promptSelection(String message, String[] options, int[] values, int defaultValue,
            boolean needResponse, long cancelAfterDelay);
    
    public void toForeground();

    public void toBackground();

    public void loadBrowser(String url);

}
