/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui;

import com.funambol.blackberry.controller.HomeScreenController;


/**
 * HomeScreen is an inner class that creates the home screen for SyncClient on
 * the BlackBerry. It extends MainScreen.
 * 
 */
public interface HomeScreen extends Screen {

    void initialize(HomeScreenController controller);

    void syncStarted(int syncMask);

    void sourceStarted(int syncMask);

    void sourceEnded(int syncMask);

    void syncEnded();
    
    /**
     * @param mask
     * @param enabled TODO
     */
    void setEnabled(int mask, boolean enabled);

    /**
     * 
     * @param message
     */
    public void displayError(String message);

    /**
     * Display an error for a particular category
     * 
     * @param mask
     * @param message
     */
    void displayError(int mask, String message);

    /**
     * Display a failure string globally
     * 
     * @param message
     */
    void displayFailure(String message);

    /**
     * Display a failure string for a particular category
     * 
     * @param mask
     * @param message
     */
    void displayFailure(int mask, String message);

    /**
     * Display the current sync status globally
     * 
     * @param message
     */
    void displayStatus(String message);

    /**
     * Display the current sync status for a particualr category
     * 
     * @param mask
     * @param message
     */
    void displayStatus(int mask, String message);

    /**
     * Display success globally
     * 
     * @param message
     */
    void displaySuccess(String message);

    /**
     * Display success for a particular category
     * 
     * @param mask
     * @param message
     */
    void displaySuccess(int mask, String message);

    /**
     * Display a category as disabled
     * 
     * @param mask
     * @param message
     */
    void displayDisabled(int mask, String message);

    /**
     * Display a warning globally
     * 
     * @param message
     */
    void displayWarning(String message);

    /**
     * Display a warning for a particular category
     * 
     * @param mask
     * @param message
     */
    void displayWarning(int mask, String message);
    
    /**
     * 
     */
    void redraw();
}
