/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.container.MainScreen;

import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.ui.AboutScreen;
import com.funambol.util.StaticDataHelper;

public class RimAboutScreen extends MainScreen implements AboutScreen {

    public static final String IMAGE_LOGO = "icon_logo_64x64.png";
    public static final String POWERED_BY_LOGO = "powered_by.png";

    private final MenuItem backMenu = new MenuItem(StaticDataHelper.getLanguage("menu_back"),
                                                  20000, 10) {
                                              public void run() {
                                                  onClose();
                                              }
                                          };

    public RimAboutScreen(Controller controller) {
        
        this.setFont(RimHelper.getDefaultFont());
        
        LabelField title = new LabelField(StaticDataHelper.getLanguage("about_title"));

        title.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(title);

        Bitmap icon = Bitmap.getBitmapResource(IMAGE_LOGO);
        BitmapField image = new BitmapField(icon, Field.FIELD_HCENTER | Field.FIELD_TOP
                | Field.NON_FOCUSABLE);

        // add blank space
        add(new LabelField("", Field.FOCUSABLE));

        add(image);

        // add blank space
        add(new LabelField("", Field.NON_FOCUSABLE));

        RichTextField clientName = new RichTextField(StaticDataHelper.getLanguage("home_title") + " "
                + BBVersion.VERSION, 
                Field.FIELD_HCENTER | RichTextField.TEXT_ALIGN_HCENTER | Field.NON_FOCUSABLE);
        clientName.setEditable(false);
        clientName.setFont(getFont().derive(Font.BOLD));
        add(clientName);

        // add blank space
        add(new LabelField(""));

        String copy = Customization.getInstance().getAboutCopyright();
        if(!copy.equals("")) {
            RichTextField copyright = new RichTextField(copy,
                    Field.FIELD_HCENTER | RichTextField.TEXT_ALIGN_HCENTER | Field.NON_FOCUSABLE);
            copyright.setEditable(false);
            copyright.setFont(getFont().derive(Font.BOLD));
            add(copyright);

            /*
            LabelField vendor = new LabelField("Funambol Inc.", Field.FIELD_HCENTER);
            vendor.setFont(getFont().derive(Font.BOLD));
            add(vendor);
            */

            RichTextField rightsReserved = new RichTextField("All rights reserved.",
                    Field.FIELD_HCENTER | RichTextField.TEXT_ALIGN_HCENTER | Field.NON_FOCUSABLE);
            rightsReserved.setEditable(false);
            rightsReserved.setFont(getFont().derive(Font.BOLD));
            add(rightsReserved);
        }

        // add blank space
        add(new LabelField(""));

        RichTextField site = new RichTextField(Customization.getInstance().getAboutSite(),
                Field.FIELD_HCENTER | RichTextField.TEXT_ALIGN_HCENTER | Field.NON_FOCUSABLE);
        site.setEditable(false);
        site.setFont(getFont().derive(Font.BOLD));
        add(site);

        // add blank space
        add(new LabelField(""));
        
        if(Customization.getInstance().showAboutLicence()) {
            addLicense();
        }
        if(Customization.getInstance().showPoweredBy()) {
            addPoweredBy();
        }
    }

    private void addLicense() {
        String license = "This program is provided AS IS, without warranty licensed under AGPLV3. The "
                + "Program is free software; you can redistribute it and/or modify it under the "
                + "terms of the GNU Affero General Public License version 3 as published by the Free "
                + "Software Foundation including the additional permission set forth source code "
                + "file header.\n\n"
                +

                "The interactive user interfaces in modified source and object code versions of "
                + "this program must display Appropriate Legal Notices, as required under Section 5 "
                + "of the GNU Affero General Public License version 3.\n\n "
                +

                "In accordance with Section 7(b) of the GNU Affero General Public License version 3, "
                + "these Appropriate Legal Notices must retain the display of the \"Powered by "
                + "Funambol\" logo. If the display of the logo is not reasonably feasible for "
                + "technical reasons, the Appropriate Legal Notices must display the words \"Powered "
                + "by Funambol\". Funambol is a trademark of Funambol, Inc.";

        RichTextField rtf = new RichTextField(license, RichTextField.TEXT_ALIGN_HCENTER |
                Field.FIELD_HCENTER | Field.FOCUSABLE);
        rtf.setEditable(false);
        rtf.setFont(getFont().derive(Font.PLAIN));
        add(rtf);
    }

    private void addPoweredBy() {
        Bitmap pow = Bitmap.getBitmapResource(POWERED_BY_LOGO);
        BitmapField pow_image = new BitmapField(pow, Field.FIELD_HCENTER | Field.FIELD_TOP
                | Field.NON_FOCUSABLE);
        add(pow_image);
        // add blank space
        add(new LabelField("", Field.FOCUSABLE));
    }

    public void makeMenu(Menu menu, int instance) {
        menu.add(backMenu);
    }
}
