/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.util.Vector;
import java.util.Enumeration;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.KeyListener;
import net.rim.device.api.system.TrackwheelListener;
import net.rim.device.api.system.KeypadListener;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Keypad;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.ExternalAppManager;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.blackberry.ui.HomeScreen;
import net.rim.device.api.ui.Manager;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;


/**
 * RimBasicHomeScreen is an inner class that creates the home screen for SyncClient on
 * the BlackBerry. The class provides a basic home screen that can be extended
 * to accomodate specific versions behavior. The class extends MainScreen.
 * 
 */
public class RimBasicHomeScreen extends MainScreen implements HomeScreen {

    final public static String   IMAGE_ARROWS_IDLE       = "arrows1.png";
    final public static String   IMAGE_ARROWS_A          = "arrows1.png";
    final public static String   IMAGE_ARROWS_B          = "arrows2.png";
    final public static String   IMAGE_ARROWS_C          = "arrows3.png";
    final public static String   IMAGE_ARROWS_D          = "arrows4.png";


    final public static String[] IMAGE_SYNCING           = { IMAGE_ARROWS_A,
                                                             IMAGE_ARROWS_B,
                                                             IMAGE_ARROWS_C,
                                                             IMAGE_ARROWS_D
                                                           };

    public HomeScreenController  controller;

    private HomeScreenMenu       menu;

    protected HomeScreenList     listField;

    private LabelField           titleField;

    private HomeScreenAnimation  animation;

    /** This is the FieldList default style. By default we center horizontally */
    protected long style = Field.FIELD_HCENTER;

    private boolean invalidateNavClick;

    /**
     * Creates the home screen with all the UI components on it
     */
    public RimBasicHomeScreen() {
        super(MainScreen.FOCUSABLE | MainScreen.NO_VERTICAL_SCROLL);
    }

    public void initialize(HomeScreenController controller) {
        
        this.setFont(RimHelper.getDefaultFont());
        
        this.controller = controller;

        menu = new HomeScreenMenu(controller);
        animation = new HomeScreenAnimation();

        if (Customization.getInstance().showTitleWithVersion()) {
            titleField = new LabelField(StaticDataHelper.getLanguage("home_title") + " "
                    + BBVersion.VERSION);
        } else {
            titleField = new LabelField(StaticDataHelper.getLanguage("home_title"));
        }

        titleField.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(titleField);

        VerticalFieldManager vManager = new VerticalFieldManager(
                Manager.VERTICAL_SCROLL | Manager.VERTICAL_SCROLLBAR);
        add(vManager);
        listField = new HomeScreenList();
        vManager.add(listField);

        // The MySeparatorField allows to fill the empty space under the
        // ListField, in order to avoid navigationClick events in this area.
        add(new MySeparatorField(MySeparatorField.NON_FOCUSABLE));
    }

    public class MySeparatorField extends SeparatorField {
        public MySeparatorField(long s) {
            super(s);
        }
        protected void layout(int width, int height) {
            setExtent(width, 0);
        }
    }

    public void setSelectedIndex(int index) {
        class SetSelectionRequest implements Runnable {

            private int index;

            public SetSelectionRequest(int index) {
                this.index = index;
            }

            public void run() {
                listField.setSelectedIndex(index);
            }
        }

        SetSelectionRequest req = new SetSelectionRequest(index);

        if (SyncClient.getUiApplication().isEventThread()) {
            req.run();
        } else {
            SyncClient.getUiApplication().invokeLater(req);
        }

        return;
    }

    public void syncStarted(int syncMask) {
        listField.removeListeners();
        
        if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            ((HomeScreenListItem) listField.listItems.elementAt(0)).setTitle(StaticDataHelper.getLanguage("status_sync"));
        }
    }

    /**
     * Check the Customization class if the Sync all main screen item is 
     * enabled. If disabled the first item in main screen is the first enabled 
     * source, so the initial index is set to 0.
     * @return the first index related to a source item  
     */
    private int getFirstSourceItemIndex() {
        if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            return 1;
        } else {
            return 0;
        }
    }
    
    public void sourceStarted(int syncMask) {
        Enumeration enabledSources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        
        while (enabledSources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) enabledSources.nextElement();
            //Index of the item to be started
            int index = appSource.getUiSourceIndex();
            int id = appSource.getId();
                if ((id & syncMask) != 0) {
                    setSelectedIndex(index);
                    animation.startAnimation(index);
                }
            index++;
        }
    }

    public void sourceEnded(int syncMask) {
    }

    public void syncEnded() {
        animation.stopAnimation();
        listField.addListeners();
        //Select the real first index in case the sync all button is 
        //not required and the first source is disabled
        setSelectedIndex(getFirstActiveItemIndex());
        if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            listField.setStatusString(0, null);
            ((HomeScreenListItem) listField.listItems.elementAt(0)).setTitle(StaticDataHelper.getLanguage("home_sync-all"));
        }
    }
    
    private int getFirstActiveItemIndex() {
        if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            return 0;
        }

        int size = listField.listItems.size();
        
        for (int i=0; i<size; i++) {
            HomeScreenListItem item = (HomeScreenListItem) listField.listItems.elementAt(i);
            AppSyncSource source = item.getSource();
            if (source.isEnabled()&&source.isWorking()) {
                return i;
            }
        }   
        return 0;
    }

    protected int getNumRows() {
        return listField.numRows();
    }

    protected int getRowHeight() {
        return listField.getRowHeight();
    }

    /**
     * This method is invoked on user hitting the escape button when in home
     * screen. It only minimizes the program
     */
    public boolean onClose() {
        controller.quitMenuSelected();
        return true;
    }

    public void setEnabled(int mask, boolean enabled) {
        if (enabled) {
            listField.enable(mask);
        } else {
            listField.disable(mask);
        }
    }

    /**
     * Display an error globally
     * 
     * @param message
     */
    public void displayError(String message) {
        // animation.stopAnimation();
        //if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            displayError(0, message);
        //}
    }

    /**
     * Display an error for a particular category
     * 
     * @param mask
     * @param message
     */
    public void displayError(int mask, String message) {
        listField.setStatusIcon(mask, Customization.getInstance().getErrorIcon());
        listField.setStatusString(mask, message);
        listField.invalidate();
    }

    /**
     * Display a failure string globally
     * 
     * @param message
     */
    public void displayFailure(String message) {
        // animation.stopAnimation();
        //if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            displayFailure(0, message);
        //}
    }

    /**
     * Display a failure string for a particular category
     * 
     * @param mask
     * @param message
     */
    public void displayFailure(int mask, String message) {
        Log.trace("[RimBasicHomeScreen.displayFailure] " + mask + "," + message);
        listField.setStatusIcon(mask, Customization.getInstance().getErrorIcon());
        listField.setStatusString(mask, message);
        listField.invalidate();
    }

    /**
     * Display the current sync status globally
     * 
     * @param message
     */
    public void displayStatus(String message) {
        //if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            displayStatus(0, message);
        //}
        listField.invalidate();
    }

    /**
     * Display the current sync status for a particualr category
     * 
     * @param mask
     * @param message
     */
    public void displayStatus(int mask, String message) {
        listField.setStatusString(mask, message);
        listField.invalidate();
    }

    /**
     * Display success globally
     * 
     * @param message
     */
    public void displaySuccess(String message) {
        // animation.stopAnimation();
        //if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            displaySuccess(0, message);
        //}
    }

    /**
     * Display success for a particular category
     * 
     * @param mask
     * @param message
     */
    public void displaySuccess(int mask, String message) {
        Log.trace("[RimBasicHomeScreen.displaySuccess] " + mask + "," + message);
        listField.setStatusIcon(mask, Customization.getInstance().getOkIcon());
        listField.setStatusString(mask, message);
        listField.invalidate();
    }

    /**
     * Display a category as disabled
     * 
     * @param mask
     * @param message
     */
    public void displayDisabled(int mask, String message) {
        listField.setStatusString(mask, message);
        listField.disable(mask);
        listField.invalidate();
    }

    /**
     * Display a warning globally
     * 
     * @param message
     */
    public void displayWarning(String message) {
        // animation.stopAnimation();
        //if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            displayWarning(0, message);
        //}
    }

    /**
     * Display a warning for a particular category
     * 
     * @param mask
     * @param message
     */
    public void displayWarning(int mask, String message) {
        Log.trace("[RimBasicHomeScreen.displayWarning] " + mask + message);
        listField.setStatusIcon(mask, Customization.getInstance().getOkIcon());
        listField.setStatusString(mask, message);
        listField.invalidate();
    }

    /**
     * Builds the menu
     * 
     * @param Menu
     *            Menu to which menuitems are to be added
     * @param int
     *            instance
     */
    public void makeMenu(Menu theMenu, int instance) {
        menu.makeMenu(theMenu, instance, controller.isSynchronizing(), controller.numErrors() != 0,
                Customization.getInstance().checkForUpdates() && controller.isUpdate());
    }

    public void redraw() {
        listField.invalidate();
    }

    protected void onExposed() {
        updateLastSyncTimes();
        super.onExposed();
    }

    protected void onDisplay() {

        // Leave some space at the bottom to avoid scrolling issue
        // on some devices. This is just a workaround
        int screenHeight = (98 * getDrawableHeight()) / 100;
        int rowHeight = screenHeight / listField.numRows();
        int minHeight = 32 + (2 * HomeScreenListItem.PADDING);
        int maxHeight = 42;

        if (rowHeight < minHeight) {
            rowHeight = minHeight;
        }
        if (rowHeight > maxHeight) {
            rowHeight = maxHeight;
        }
        listField.setRowHeight(rowHeight);
        super.onDisplay();
        updateLastSyncTimes();
        setSelectedIndex(getFirstActiveItemIndex());
    }

    private void updateLastSyncTimes() {
        controller.updateHomeScreen();
    }

    private void buttonPressed(int index) {
        Log.trace("[RimBasicHomeScreen.buttonPressed] " + index);
        
        if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            if (index != 0) {
                HomeScreenListItem item = (HomeScreenListItem) listField.listItems.elementAt(index);
                AppSyncSource source = item.getSource();
                controller.syncMenuSelected(source.getId());
            } else {
                controller.syncMenuSelected();
            }
        } else {
            HomeScreenListItem item = (HomeScreenListItem) listField.listItems.elementAt(index);
            AppSyncSource source = item.getSource();
            if (source.isEnabled()) {
                controller.syncMenuSelected(source.getId());
            } else {
                controller.syncMenuSelected();
            }
        }
    }

    protected int getDrawableHeight() {
        return getContentHeight() - titleField.getHeight();
    }

    private class HomeScreenAnimation {

        private int currentFrame;
        private int currentIndex;
        Thread anim;

        public HomeScreenAnimation() {
        }

        /**
         * Begin the animation for a sync in progress
         */
        public void startAnimation(final int index) {

            currentIndex = index;

            stopAnimation();

            if (anim == null) {
                anim = new Thread() {

                    public void run() {
                        HomeScreenAnimation.this.runAnimation();
                    }
                };

                currentFrame = 0;
            }

            anim.start();
        }

        /**
         * Stop the animation for a sync in progress
         */
        public void stopAnimation() {
            if (anim != null && anim.isAlive()) {
                anim.interrupt();
                try {
                    anim.join();
                } catch (InterruptedException e) {
                }
                anim = null;
            }
        }

        /**
         * Thread.run function for the animation
         */
        private void runAnimation() {

            try {
                while (true) {
                    listField.setSelectedStatusIcon(currentIndex, IMAGE_SYNCING[currentFrame]);
                    listField.invalidate();

                    int newFrame = (currentFrame + 1) % (IMAGE_SYNCING.length);
                    currentFrame = newFrame;

                    Thread.sleep(500);
                }
            } catch (InterruptedException e) {
                listField.setSelectedStatusIcon(currentIndex, IMAGE_SYNCING[0]);
                listField.invalidate();
                return;
            }

        }
    }
    private String pressed = "";

    private class HomeScreenListener implements KeyListener, TrackwheelListener {

        private final static int KEYCODE_ENTER = Keypad.KEY_ENTER;

        // KeypadListener
        public boolean keyChar(char key, int status, int time) {
            return false;
        }

        public boolean keyDown(int keycode, int time) {
            int mapped = Keypad.map(keycode);
            if (mapped == KEYCODE_ENTER || keycode == KEYCODE_ENTER) {
                buttonPressed(listField.getSelectedIndex());
                return true;
            }

            char mappedChar = Character.toUpperCase((char) mapped);

            if (mappedChar > 'A' && mappedChar < 'Z') {
                pressed += (char) mappedChar;
                if (pressed.length() > 3) {
                    pressed = pressed.substring(1);
                }
            }

            return false;
        }

        public boolean keyRepeat(int keycode, int time) {
            return false;
        }

        public boolean keyStatus(int keycode, int time) {
            return false;
        }

        public boolean keyUp(int keycode, int time) {
            return false;
        }

        public boolean trackwheelClick(int status, int time) {
            buttonPressed(listField.getSelectedIndex());
            return true;
        }

        public boolean trackwheelRoll(int amount, int status, int time) {
            return false;
        }

        public boolean trackwheelUnclick(int status, int time) {
            return false;
        }
    }

    private class HomeScreenList extends ListField implements ListFieldCallback {

        private final HomeScreenListener listener;

        private Vector                   listItems;

        private boolean                  blockMoves;

        public HomeScreenList() {

            // The number of rows is dynamic and it depends on the number of
            // active sources

            //Set the SyncAll Item if required when calling the constructor
            //This piece of code is a bit unreadable, but consider that the "super"
            //call must be the first into this class construsctor. Simply an item is added if
            //the custom implementation returns true: here we are checking the 
            //need of a "Sync All" item into the main screen.
            super(AppSyncSourceManager.getInstance().numberOfRegisteredSources() + getFirstSourceItemIndex(),
                  style);

            listItems = new Vector();

            Bitmap img, imgDisabled, bg, bgSel;
            HomeScreenListItem item;

            //Set the SyncAll Item if required
            if (Customization.getInstance().syncAllOnMainScreenRequired()) {
                String imageAll = Customization.getInstance().getSyncAllIcon();
                img = Bitmap.getBitmapResource(imageAll);

                String imageAllBg = Customization.getInstance().getSyncAllBackground();
                bg = Bitmap.getBitmapResource(imageAllBg);

                String imageAllBgHigh = Customization.getInstance().getSyncAllHighlightedBackground();
                bgSel = Bitmap.getBitmapResource(imageAllBgHigh);

                item = new HomeScreenListItem(StaticDataHelper.getLanguage("home_sync-all"), null, img,
                        img, bg, bgSel);
                item.setSelected(true);
                item.setSpecial(true);
                item.setTitleEnabledColor(Color.WHITE);
                item.setTitleDisabledColor(Color.WHITE);
                listItems.addElement(item);
            }
            
            listItems.setSize(AppSyncSourceManager.getInstance().numberOfRegisteredSources() + 
                                       getFirstSourceItemIndex() );
            
            String bgName = Customization.getInstance().getButtonBackground();
            String bgSelName = Customization.getInstance().getButtonHighlightedBackground();
            bg = Bitmap.getBitmapResource(bgName);
            bgSel = Bitmap.getBitmapResource(bgSelName);

            // Add an item for each registered source
            Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
            while (sources.hasMoreElements()) {
                AppSyncSource appSource = (AppSyncSource)sources.nextElement();

                String iconName = appSource.getIconName();
                String iconDisabledName = appSource.getDisabledIconName();
                String name = appSource.getName();

                img = Bitmap.getBitmapResource(iconName);
                imgDisabled = Bitmap.getBitmapResource(iconDisabledName);
                item = new HomeScreenListItem(name, "", img, imgDisabled, bg, bgSel);
                // All these buttons are associated to a given application
                // source
                item.setSource(appSource);
                int index = appSource.getUiSourceIndex();
                listItems.setElementAt(item, index);

                //listItems.addElement(item);
            }

            Vector arrows = new Vector();

            arrows.addElement(Bitmap.getBitmapResource(IMAGE_ARROWS_IDLE));
            arrows.addElement(Bitmap.getBitmapResource(IMAGE_ARROWS_A));
            arrows.addElement(Bitmap.getBitmapResource(IMAGE_ARROWS_B));
            arrows.addElement(Bitmap.getBitmapResource(IMAGE_ARROWS_C));
            arrows.addElement(Bitmap.getBitmapResource(IMAGE_ARROWS_D));

            animation = new HomeScreenAnimation();

            setEditable(false);
            setEmptyString("", DrawStyle.LEFT);

            setCallback(this);
            listener = new HomeScreenListener();
            addListeners();
        }

        protected boolean navigationClick(int status, int time) {
            Log.trace("[HomeScreenList.navigationClick]");
            if (status == 0 && !invalidateNavClick) {
                buttonPressed(listField.getSelectedIndex());
                return true;
            }
            invalidateNavClick = false;
            return (status == 0);
        }

        private int numRows() {
            return listItems.size();
        }

        public void disable(int mask) {
            if (mask == 0) {
                ((HomeScreenListItem) listItems.elementAt(0)).setEnabled(false);
            }

            for (int index = getFirstSourceItemIndex(); index < listItems.size(); index++) {
                HomeScreenListItem item = (HomeScreenListItem) listItems.elementAt(index);
                AppSyncSource      source = item.getSource();
                if ((mask & source.getId()) != 0) {
                    ((HomeScreenListItem) listItems.elementAt(index)).setEnabled(false);
                }
            }
            invalidate();
        }

        public void enable(int mask) {
            if (mask == 0) {
                ((HomeScreenListItem) listItems.elementAt(0)).setEnabled(true);
            }
            for (int index = getFirstSourceItemIndex(); index < listItems.size(); index++) {
                HomeScreenListItem item = (HomeScreenListItem) listItems.elementAt(index);
                AppSyncSource      source = item.getSource();
                if ((mask & source.getId()) != 0) {
                    ((HomeScreenListItem) listItems.elementAt(index)).setEnabled(true);
                }
            }
            invalidate();
        }

        public void setStatusString(int mask, String value) {
            if (mask == 0) {
                ((HomeScreenListItem) listItems.elementAt(mask)).setStatusString(value);
            } else {
                for (int index = getFirstSourceItemIndex(); index < listItems.size(); index++) {
                    HomeScreenListItem item = (HomeScreenListItem) listItems.elementAt(index);
                    AppSyncSource      source = item.getSource();
                    if ((mask & source.getId()) != 0) {
                        ((HomeScreenListItem) listItems.elementAt(index)).setStatusString(value);
                    }
                }
            }

            invalidate();
        }

        public void setSelectedStatusIcon(int index, String file) {
            if (file != null) {
                ((HomeScreenListItem) listItems.elementAt(index)).setStatusSelectedIcon(Bitmap
                        .getBitmapResource(file));
            } else {
                ((HomeScreenListItem) listItems.elementAt(index)).setStatusSelectedIcon(null);
            }
        }

        public void setStatusIcon2(int index, String file) {
            if (file != null) {
                ((HomeScreenListItem) listItems.elementAt(index)).setStatusIcon(Bitmap
                        .getBitmapResource(file));
            } else {
                ((HomeScreenListItem) listItems.elementAt(index)).setStatusIcon(null);
            }
        }

        public void setStatusIcon(int mask, String filename) {

            if (mask == 0) {
                // Do nothing
                // setStatusIcon2(mask, filename);
            } else {

                for (int index = getFirstSourceItemIndex(); index < listItems.size(); index++) {
                    HomeScreenListItem item = (HomeScreenListItem) listItems.elementAt(index);
                    AppSyncSource      source = item.getSource();
                    if ((mask & source.getId()) != 0) {
                        if (filename != null) {
                            ((HomeScreenListItem) listItems.elementAt(index)).setStatusIcon(Bitmap
                                    .getBitmapResource(filename));
                        } else {
                            ((HomeScreenListItem) listItems.elementAt(index)).setStatusIcon(null);
                        }
                    }
                }
            }

            invalidate();
        }

        // ListFieldCallback functions
        public void drawListRow(ListField listField, Graphics graphics, int index, int y, int width) {
            HomeScreenListItem item = ((HomeScreenListItem) listItems.elementAt(index));
            item.draw(graphics, 0, y, width, listField.getRowHeight());
        }

        public Object get(ListField listField, int index) {
            return null;
        }

        public int indexOfList(ListField listField, String prefix, int start) {
            return listField.getSelectedIndex();
        }

        public int getPreferredWidth(ListField listField) {
            return getContentWidth();
        }
        
        protected int moveFocus(int amount, int status, int time) {

            if (blockMoves) {
                return 0;
            }

            HomeScreenListItem item;

            int oldSelection = getSelectedIndex();
            item = ((HomeScreenListItem) listItems.elementAt(oldSelection));
            item.setSelected(false);

            // Forward the call
            int ret = super.moveFocus(amount, status, time);

            int newSelection = getSelectedIndex();

            // Get the next enabled item;
            item = ((HomeScreenListItem) listItems.elementAt(newSelection));
            boolean stop = false;
            while (item.isDisabled() && !stop) {
                newSelection += amount;
                stop = newSelection < 0 || newSelection >= listItems.size();
                if (!stop) {
                    item = ((HomeScreenListItem) listItems.elementAt(newSelection));
                }
            }
            
            if (stop) {
                newSelection = oldSelection;
                invalidateNavClick = true;
            }
            
            item = ((HomeScreenListItem) listItems.elementAt(newSelection));

            item.setSelected(true);
            setSelectedIndex(newSelection);

            invalidate();

            return ret;
        }

        protected void moveFocus(int x, int y, int status, int time) {
            int oldSelection = getSelectedIndex();
            super.moveFocus(x, y, status, time);
            int newSelection = getSelectedIndex();
            HomeScreenListItem item = (HomeScreenListItem)listItems.elementAt(newSelection);
            if(!item.isDisabled()) {
                ((HomeScreenListItem) listItems.elementAt(oldSelection)).setSelected(false);
                setSelectedIndex(newSelection);
            }
            else {
                setSelectedIndex(oldSelection);
                invalidateNavClick = true; 
            }
        }

        public void addListeners() {
            addKeyListener(listener);
            // If the device being used does not have a dedicated menu key,
            // then we shall intercept the trackwheel
            if (StaticDataHelper.hasDedicatedMenuKey()) {
                addTrackwheelListener(listener);
            }
            blockMoves = false;
        }

        public void removeListeners() {
            removeKeyListener(listener);
            blockMoves = true;
        }

        public void setSelectedIndex(int index) {
            int current = getSelectedIndex();
            super.setSelectedIndex(index);
            ((HomeScreenListItem) listItems.elementAt(current)).setSelected(false);
            ((HomeScreenListItem) listItems.elementAt(index)).setSelected(true);
            invalidate();
        }
    }

    private class HomeScreenListItem {

        public static final int MARGIN_SIDE = 5;

        public static final int PADDING     = 2;

        private String          title;

        private String          status;

        private Bitmap          image;

        private Bitmap          statusImage;

        private Bitmap          statusImageSelected;

        private Bitmap          imageDisabled;

        private Bitmap          bg;

        private Bitmap          bgSelected;

        private boolean         enabled     = true;

        private boolean         selected    = false;

        private boolean         special     = false;

        private int             titleEnabledColor    = Color.BLACK;
        private int             titleDisabledColor   = Color.LIGHTGREY;

        private AppSyncSource   source               = null;

        public HomeScreenListItem(String title, String status, Bitmap image, Bitmap imageDisabled,
                Bitmap background, Bitmap background_selected) {
            this.title = title;
            this.status = status;
            this.image = image;
            this.imageDisabled = imageDisabled;
            this.bg = background;
            this.bgSelected = background_selected;
            this.setStatusSelectedIcon(Bitmap.getBitmapResource(IMAGE_ARROWS_IDLE));
        }

        public void setTitle(String value) {
            title = value;
        }

        public void setTitleEnabledColor(int color) {
            this.titleEnabledColor = color;
        }

        public void setTitleDisabledColor(int color) {
            this.titleDisabledColor = color;
        }

        public void setSpecial(boolean value) {
            special = value;
        }

        public void setEnabled(boolean flag) {
            enabled = flag;
        }

        public boolean isDisabled() {
            return !enabled;
        }

        public void setStatusIcon(Bitmap image) {
            statusImage = image;
        }

        public void setStatusSelectedIcon(Bitmap image) {
            statusImageSelected = image;
        }

        public void setStatusString(String value) {
            status = value;
        }

        public void setSource(AppSyncSource source) {
            this.source = source;
        }

        public void setSelected(boolean selected) {
            this.selected = selected;
        }

        public AppSyncSource getSource() {
            return source;
        }

        public void draw(Graphics graphics, int x, int y, int width, int height) {
            // Border
            Font originalFont = graphics.getFont();
            int originalColor = graphics.getColor();
            drawBackground(graphics, x, y, width, height);

            drawBorder(graphics, x, y, width, height);

            int leftImageWidth = drawLeftImage(graphics, x, y, height);

            int rightImageWidth = drawRightImage(graphics, y, width, height);

            drawTitleText(graphics, leftImageWidth, y, width - rightImageWidth - leftImageWidth, height);

            drawStatusText(graphics, leftImageWidth, y, width - rightImageWidth - leftImageWidth, height);

            graphics.setFont(originalFont);
            graphics.setColor(originalColor);
        }

        private void drawStatusText(Graphics graphics, int x, int y, int width, int height) {
            // Last sync takes bottom third
            //int fontHeight = (height / 3) - (PADDING * 2);
            int fontHeight = ((2* height) / 5) - (PADDING * 2);
            graphics.setFont(getFont().derive(Font.PLAIN, fontHeight));

            if (enabled || special) {
                graphics.setColor(Color.BLACK);
            } else {
                graphics.setColor(Color.LIGHTGREY);
            }

            graphics.drawText(status, x + PADDING + 5, y - 4 + (height - fontHeight),
                    DrawStyle.LEFT | DrawStyle.TOP, width - PADDING);
        }

        private void drawTitleText(Graphics graphics, int x, int y, int width, int height) {
            //int fontHeight = ((int) (1.7 * (height / 3))) - (PADDING * 2);
            int fontHeight = ((int) ((3* height) / 5)) - (PADDING * 2);
            graphics.setFont(getFont().derive(Font.BOLD, fontHeight));

            if (enabled) {
                graphics.setColor(titleEnabledColor);
            } else {
                graphics.setColor(titleDisabledColor);
            }

            if (status != null) {
                // Title takes top 2/3 of list item
                graphics.drawText(title, x + PADDING + 3, y + PADDING + 2, DrawStyle.LEFT
                        | DrawStyle.TOP, width - x - (PADDING * 2));
            } else {
                // Title is vertically centered
                graphics.drawText(title, x + PADDING + 3, y + PADDING + 2 + (fontHeight / 2),
                        DrawStyle.LEFT | DrawStyle.TOP, width - x - (PADDING * 2));
            }

        }

        private int drawRightImage(Graphics graphics, int y, int width, int height) {
            // Image on right side

            int imageWidth = 0;
            int imageHeight;
            int imageTop;
            int imageLeft;
            Bitmap toDraw = null;
            if (selected) {
                toDraw = statusImageSelected;
            } else {
                toDraw = statusImage;
            }

            if (toDraw != null && enabled) {
                imageWidth = toDraw.getWidth();
                imageHeight = toDraw.getHeight();
                imageTop = y + ((height - imageHeight) / 2);
                imageLeft = (width - height) + ((height - imageWidth) / 2);

                graphics.drawBitmap(imageLeft, imageTop, imageWidth, imageHeight, toDraw, 0, 0);
            }

            return height;
        }

        private int drawLeftImage(Graphics graphics, int x, int y, int height) {
            Bitmap leftImage;

            if (enabled) {
                graphics.setColor(Color.BLACK);
                leftImage = image;
            } else {
                graphics.setColor(Color.LIGHTGREY);
                leftImage = imageDisabled;
            }

            int imageWidth = leftImage.getWidth();
            int imageHeight = leftImage.getHeight();
            int imageTop = y + ((height - imageHeight) / 2);
            int imageLeft = x + ((height - imageWidth) / 2);

            // Image on left side
            graphics.drawBitmap(imageLeft, imageTop, imageWidth, imageHeight, leftImage, 0, 0);

            return height;
        }

        private void drawBackground(Graphics graphics, int x, int y, int width, int height) {
            if (enabled || special) {
                Bitmap toDraw = null;
                if (selected) {
                    toDraw = bgSelected;
                } else {
                    toDraw = bg;
                }
                int imgWidth = toDraw.getWidth();
                while (width > -2) {
                    graphics.drawBitmap(x - 1, y - 1, width + 2, height + 1, toDraw, 0, 0);
                    width -= imgWidth;
                    // Overlap a little bit to avoid border issues
                    x += imgWidth - 2;
                }
            } else {
                graphics.setColor(Color.WHITE);
                graphics.fillRect(x - 1, y - 1, width + 2, height + 1);
            }
        }

        private void drawBorder(Graphics graphics, int x, int y, int width, int height) {

            if (special) {
                graphics.setColor(Color.BLACK);
                graphics.drawRect(x, y, width, height);
            } else {
                graphics.setColor(Color.GRAY);
                graphics.drawLine(x, y - 1, x + width, y - 1);
                graphics.drawLine(x, y + height - 1, x + width, y + height - 1);
            }
        }
    }

    private class HomeScreenMenu {

        private HomeScreenController homeScreenController;

        public HomeScreenMenu(HomeScreenController ctrl) {
            homeScreenController = ctrl;
        }

        /**
         * The menu
         */
        private final MenuItem syncAllMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_sync-all"), 20000, 10) {
                public void run() {
                    homeScreenController.syncMenuSelected();
                }
            };

        private final MenuItem syncMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_sync"), 20000, 10) {
                public void run() {
                    int index = listField.getSelectedIndex();
                    HomeScreenListItem item;
                    item = (HomeScreenListItem) listField.listItems.elementAt(index);
                    AppSyncSource source = item.getSource();
                    // Sync this specific source
                    homeScreenController.syncMenuSelected(source.getId());
                }
            };

        private final MenuItem gotoMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_goto"), 20000, 10) {
                public void run() {
                    int index = listField.getSelectedIndex();
                    HomeScreenListItem item;
                    item = (HomeScreenListItem) listField.listItems.elementAt(index);
                    AppSyncSource source = item.getSource();
                    ExternalAppManager appManager = source.getAppManager();
                    try {
                        appManager.launch();
                    } catch (Exception e) {
                        Log.error("Cannot invoke the application: " + e.toString());
                    }
                }
            };

        private final MenuItem refreshMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_refresh"), 20000, 10) {
                public void run() {
                    homeScreenController.refreshMenuSelected();
                }
            };

        private final MenuItem cancelMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_cancel-sync"), 20000, 10) {
                public void run() {
                    homeScreenController.cancelMenuSelected();
                }
            };

        private final MenuItem sendLogMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_send-logs"), 20000, 10) {
                public void run() {
                    homeScreenController.sendLogs();
                }
            };

        private final MenuItem configMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_config"), 20000, 10) {
                public void run() {
                    homeScreenController.configMenuSelected();
                }
            };

        private final MenuItem helpMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_help"), 20000, 10) {
                public void run() {
                    homeScreenController.helpMenuSelected();
                }
            };

        private final MenuItem aboutMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_about"), 20000, 10) {
                 public void run() {
                     homeScreenController.aboutMenuSelected();
                 }
             };

        private final MenuItem updateMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_update"), 20000, 10) {
                public void run() {
                    homeScreenController.updateMenuSelected();
                }
            };

        private final MenuItem quitMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_quit"), 20000, 10) {
                public void run() {
                    homeScreenController.quitMenuSelected();
                }
            };

        public void makeMenu(Menu menu, int instance, boolean inSync, boolean isErrors,
                             boolean isUpdate) {

            if (inSync) {
                menu.add(cancelMenu);
            } else {
                menu.add(syncAllMenu);
                // We add the sync menu only if the highlighted item
                // is not the sync all
                if (Customization.getInstance().syncAllOnMainScreenRequired()) {
                    if (listField.getSelectedIndex() != 0) {
                        addSourceItemListMenu(menu);
                    }    
                } else {
                    addSourceItemListMenu(menu);
                }
                
            }
            menu.add(MenuItem.separator(20000));
            menu.add(quitMenu);
            menu.add(MenuItem.separator(20000));

            if (!inSync) {
                menu.add(configMenu);
            }

            if (!inSync) {
                menu.add(refreshMenu);
            }

            if (Customization.getInstance().sendLogEnabled()) {
                if (isErrors || "LOG".equals(pressed)) {
                    menu.add(sendLogMenu);
                }
            }

            if (isUpdate) {
                menu.add(updateMenu);
            }
            if (!Customization.getInstance().hideHelpScreen()) {
                menu.add(helpMenu);
            }
            else {
                menu.add(aboutMenu);
            }
        }
        
        private void addSourceItemListMenu(Menu menu) {
            // Fix the menu text, depending on the selected source
            int index = listField.getSelectedIndex();
            HomeScreenListItem item;
            item = (HomeScreenListItem) listField.listItems.elementAt(index);
            AppSyncSource source = item.getSource();
            String menuText = StaticDataHelper.getLanguage("menu_sync");
            StringBuffer newMenuText = new StringBuffer(menuText);
            newMenuText.append(" ").append(source.getName());
            syncMenu.setText(newMenuText.toString());
            
            if (source.isEnabled()) {
                // Add the menu
                menu.add(syncMenu);

                ExternalAppManager appManager = source.getAppManager();
                // Add the proper goto menu, depending on the selected
                // source
                if (Customization.getInstance().gotoMenuEnabled() &&
                    appManager != null) {

                    menuText = StaticDataHelper.getLanguage("menu_goto");
                    newMenuText = new StringBuffer(menuText);
                    newMenuText.append(" ").append(source.getName());
                    gotoMenu.setText(newMenuText.toString());
                    // Add the menu
                    menu.add(gotoMenu);
                }
            }
        }
    }
}
