/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.util.Vector;
import java.util.Enumeration;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.NumericChoiceField;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.system.Display;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.ui.ConfigurationScreen;
import com.funambol.blackberry.ui.rim.component.ColoredBasicEditField;
import com.funambol.blackberry.ui.rim.component.ColoredLabelField;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;

/**
 * Is an class that creates the configuration screen for configuring parameters
 * required for the SyncClient
 */
public class RimConfigurationScreen extends MainScreen implements ConfigurationScreen {

    /**
     * This class is only used if CUSTOM_COMPANY_DIRECTORY is enabled.
     * 
     */
    class CDListener implements FieldChangeListener {

        private ChangeListener listener;
        private CheckboxField  field;

        public CDListener(ChangeListener cl, CheckboxField field) {
            listener   = cl;
            this.field = field;
        }

        public CDListener(ChangeListener cl) {
            listener   = cl;
            this.field = null;
        }

        public void fieldChanged(Field valueField, int context) {

            /* TODO FIXME

            CheckboxField cd = (CheckboxField) field;

            String curr = valueField.getText();
            if (cd.getChecked()) {
                if (curr.endsWith("-shared")) {
                    // Do nothing
                } else {
                    curr = curr + "-shared";
                }
            } else {
                if (curr.endsWith("-shared")) {
                    curr = curr.substring(0, curr.length() - "-shared".length());
                    controller.getDialogController().showMessage(
                            StaticDataHelper.getLanguage("custom_message_remove-cd"));
                } else {
                    // Do nothing
                }
            }
            valueField.setText(curr);
            listener.fieldChanged(field, context);
            */
        }
    }

    class ChangeListener implements FieldChangeListener {

        public void fieldChanged(Field field, int context) {
            if (context != FieldChangeListener.PROGRAMMATIC) {
                hasChanges = true;
            }
        }
    }

    private BasicEditField        serverUriField          = null;

    private BasicEditField        usernameField           = null;

    private PasswordEditField     passwordField           = null;

    // List of ColoredLabelField
    private Vector                enabledSourceLabelList  = new Vector(); 
    // List of CheckboxField
    private Vector                enabledSourceFieldList  = new Vector();
    // List of ColoredLabelField
    private Vector                sourceUriFieldList      = new Vector();
    // List of ColoredLabelField
    private Vector                disabledSourceLabelList = new Vector();

    private Vector                sourceMap               = new Vector();  

    private CheckboxField         enablePollingPIMField   = null;

    private ObjectChoiceField     pollingPIMIntervalField = null;

    private NumericChoiceField    rangePastField          = null;

    private NumericChoiceField    rangeFutureField        = null;

    private ObjectChoiceField     syncTypeField           = null;

    private ObjectChoiceField     logLevelField           = null;

    private CheckboxField         blockInvitesField       = null;

    private boolean               hasChanges              = false;

    /**
     * This creates a MenuItem containing "Save" option. on selecting the save
     * option the configuration details are saved.
     */
    private final MenuItem        saveMenu                = new MenuItem(StaticDataHelper
                                                                  .getLanguage("menu_save"), 20000,
                                                                  10) {

                                                              public void run() {
                                                                  if (saveConfiguration()) {
                                                                      controller
                                                                              .getScreenDisplayController()
                                                                              .hideConfigScreen();
                                                                  }
                                                              }
                                                          };

    private final MenuItem        backMenu                = new MenuItem(StaticDataHelper
                                                                  .getLanguage("menu_back"), 20000,
                                                                  10) {

                                                              public void run() {
                                                                  onClose();
                                                              }
                                                          };

    private final Controller      controller;

    private CheckboxField         cdEnableField;

    /**
     * The constructor here creates the Configuration screen with all the UI
     * components on it and sets default input minValues
     */
    public RimConfigurationScreen(Controller ctrl) {

        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT | Field.FIELD_LEFT | Field.FIELD_TOP);

        Log.trace("[RimConfigurationScreen.RimConfigurationScreen]");

        this.setFont(RimHelper.getDefaultFont());

        controller = ctrl;

        ChangeListener cl = new ChangeListener();

        LabelField title = new LabelField(StaticDataHelper.getLanguage("conf_title"));
        title.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(title);

        LabelField serverUriLabel = null;
        if (Customization.getInstance().syncUriEditable()) {
            serverUriLabel = new LabelField(StaticDataHelper.getLanguage("conf_server-url"));
            serverUriLabel.setFont(getFont().derive(Font.BOLD));
            serverUriField = new BasicEditField("", Configuration.syncUrl);
        }

        LabelField usernameLabel = new LabelField(StaticDataHelper.getLanguage("conf_user"));
        usernameLabel.setFont(getFont().derive(Font.BOLD));
        usernameField = new BasicEditField("", Configuration.username);

        LabelField passwordLabel = new LabelField(StaticDataHelper.getLanguage("conf_password"));
        passwordLabel.setFont(getFont().derive(Font.BOLD));
        passwordField = new PasswordEditField("", Configuration.password);

        // Add sources commands
        ColoredLabelField cdLabelField = null;
        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            String enableString = appSource.getEnabledLabel();

            ColoredLabelField enableSourceLabel = new ColoredLabelField(enableString);
            enableSourceLabel.setFont(getFont().derive(Font.BOLD));

            boolean sourceEnabled = appSource.isEnabled();
            CheckboxField enableSyncSourceField = new CheckboxField("", sourceEnabled);

            String uri = Configuration.getSourceUri(appSource.getId());
            ColoredBasicEditField sourceUriField = new ColoredBasicEditField(StaticDataHelper
                    .getLanguage("conf_remote-uri")
                    + " ", uri);
            ColoredLabelField disabledSyncSourceLabel = new ColoredLabelField();

            // Contacts is special has it has one extra option
            if (appSource.getId() == AppSyncSourceManager.SYNC_FLAG_CONTACTS) {
                cdEnableField = null;
                if (Customization.getInstance().syncCompanyDirectory()) {
                    cdEnableField = new CheckboxField("", uri.endsWith("-shared"));
                    cdLabelField = new ColoredLabelField(StaticDataHelper.getLanguage("custom_config_cd"));
                    FieldChangeListener listener = new CDListener(cl);
                    cdEnableField.setChangeListener(listener);

                    if (!appSource.isEnabled()) {
                        cdEnableField.setEditable(false);
                        cdLabelField.setColor(Color.GRAY);
                    }
                }
            }

            String disableString = appSource.getDisabledLabel();
            disabledSyncSourceLabel.setText(disableString);
            disabledSyncSourceLabel.setFont(getFont().derive(Font.ITALIC));

            // Add the UI components to the proper vectors
            enabledSourceLabelList.addElement(enableSourceLabel); // ColoredLabelField
            enabledSourceFieldList.addElement(enableSyncSourceField); // CheckboxField
            sourceUriFieldList.addElement(sourceUriField);            // ColoredBasicEditField
            disabledSourceLabelList.addElement(disabledSyncSourceLabel); // Colored

            sourceUriField.setChangeListener(cl);
            enableSyncSourceField.setChangeListener(cl);

            sourceMap.addElement(appSource);
        }


        // Shceduled sync options
        LabelField enablePollingPIMLabel = new LabelField(StaticDataHelper
                .getLanguage("conf_enable-polling-pim"));
        enablePollingPIMLabel.setFont(getFont().derive(Font.BOLD));
        enablePollingPIMField = new CheckboxField("", Configuration.enablePollingPIM);

        String[] choices = getIntervalChoices(Customization.getInstance().getPollingPimIntervalChoices());
        pollingPIMIntervalField = new ObjectChoiceField("", choices, this
                .indexFromMinutes(Configuration.pollingPIMInterval));

        /*
         * TWO_WAY = 0; ONE_WAY_TO_SERVER = 1; ONE_WAY_TO_CLIENT = 2;
         */
        choices = new String[] { StaticDataHelper.getLanguage("conf_sync-type-two-way"),
                StaticDataHelper.getLanguage("conf_sync-type-one-way-to-server"),
                StaticDataHelper.getLanguage("conf_sync-type-one-way-to-client"), };

        // The MyLabelField is a custom LabelField which allows to force the field width
        // This is used within a HorizontalFieldManager.
        class MyLabelField extends LabelField {
            private int maxWidth;
            private final int PADDING_WIDTH = 5;
            public MyLabelField(String label, int maxWidth) {
                super(label);
                this.maxWidth = maxWidth-PADDING_WIDTH;
            }
            public int getPreferredWidth() {
                int labelWidth = getFont().getAdvance(getText());
                if(labelWidth<maxWidth) {
                    maxWidth = labelWidth + PADDING_WIDTH;
                }
                return maxWidth;
            }
        }
        int syncTypeMaxLength = Display.getWidth()-getFont().getAdvance(
                        StaticDataHelper.getLanguage("conf_sync-type-two-way"));
        
        MyLabelField syncTypeLabel = new MyLabelField(
                StaticDataHelper.getLanguage("conf_sync-type"), syncTypeMaxLength);
        syncTypeLabel.setFont(getFont().derive(Font.BOLD));
        syncTypeField = new ObjectChoiceField("", choices, Configuration.syncType);

        LabelField rangeLabel = new LabelField(StaticDataHelper.getLanguage("conf_range"));
        rangeLabel.setFont(getFont().derive(Font.BOLD));

        rangePastField = new NumericChoiceField(StaticDataHelper.getLanguage("conf_range-past")
                + "  ", 1,// begin
                3,// end
                1,// increment
                Configuration.rangePast - 1);
        rangePastField.setEditable(false);

        rangeFutureField = new NumericChoiceField(StaticDataHelper.getLanguage("conf_range-future")
                + "  ", 1,// begin
                6,// end
                1,// increment
                Configuration.rangeFuture - 1);
        rangeFutureField.setEditable(false);

        choices = new String[] { StaticDataHelper.getLanguage("conf_log-level-0"),
                StaticDataHelper.getLanguage("conf_log-level-1"),
                StaticDataHelper.getLanguage("conf_log-level-2"),
                StaticDataHelper.getLanguage("conf_log-level-3") };

        LabelField logLevelLabel = new LabelField(StaticDataHelper.getLanguage("conf_log-level"));
        logLevelLabel.setFont(getFont().derive(Font.BOLD));
        logLevelField = new ObjectChoiceField("", choices, Configuration.logLevel);

        LabelField blockInvitesLabel = new LabelField(StaticDataHelper
                .getLanguage("conf_block-invites"));
        blockInvitesLabel.setFont(getFont().derive(Font.BOLD));
        blockInvitesField = new CheckboxField("", Configuration.blockIncomingInvites);

        // /////////////////////////////////////////////////////////////////////////

        final SeparatorField separator1 = new SeparatorField();
        final SeparatorField separator2 = new SeparatorField();
        final SeparatorField separator3 = new SeparatorField();
        final SeparatorField separator4 = new SeparatorField();

        // /////////////////////////////////////////////////////////////////////////

        VerticalFieldManager vfm;
        HorizontalFieldManager hfm;

        if (Customization.getInstance().syncUriEditable()) {
            add(serverUriLabel);
            add(serverUriField);
        }

        add(new LabelField(""));
        add(usernameLabel);
        add(usernameField);
        add(new LabelField(""));
        add(passwordLabel);
        add(passwordField);

        add(new LabelField(""));
        add(separator1);
        add(new LabelField(""));


        // Add all the sources configuration stuff
        for(int i=0;i<enabledSourceFieldList.size();++i) {

            AppSyncSource appSource = (AppSyncSource) sourceMap.elementAt(i);

            vfm = new VerticalFieldManager();
            hfm = new HorizontalFieldManager();

            CheckboxField enableSyncSourceField = (CheckboxField)
                                             enabledSourceFieldList.elementAt(i);
            ColoredLabelField enableSyncSourceLabel = (ColoredLabelField)
                                             enabledSourceLabelList.elementAt(i);
            ColoredBasicEditField syncSourceUriField = (ColoredBasicEditField)
                                             sourceUriFieldList.elementAt(i);
            ColoredLabelField disabledSyncSourceLabel = (ColoredLabelField)
                                             disabledSourceLabelList.elementAt(i);

            hfm.add(enableSyncSourceField);
            vfm.add(enableSyncSourceLabel);
            
            if (Customization.getInstance().isSourceUriVisible()) {
                vfm.add(syncSourceUriField);
            }
            
            if (   appSource.getId() == AppSyncSourceManager.SYNC_FLAG_CONTACTS
                && Customization.getInstance().syncCompanyDirectory()) {

                HorizontalFieldManager hfm2 = new HorizontalFieldManager();
                hfm2.add(cdEnableField);
                hfm2.add(cdLabelField);
                vfm.add(hfm2);
            }

            // If the source does not work, we show an error and disable the
            // proper controls
            if (!appSource.isWorking()) {
                vfm.add(disabledSyncSourceLabel);
                syncSourceUriField.setEditable(false);
                enableSyncSourceField.setEditable(false);
            }
            hfm.add(vfm);
            add(hfm);

            add(new LabelField(""));
        }

        add(separator2);
        add(new LabelField(""));

        if (!(Customization.getInstance().disablePollOn8830() &&
              DeviceInfo.getDeviceName().equals("8830"))) {

            vfm = new VerticalFieldManager();
            hfm = new HorizontalFieldManager();
            hfm.add(enablePollingPIMField);
            vfm.add(enablePollingPIMLabel);
            vfm.add(pollingPIMIntervalField);
            hfm.add(vfm);
            add(hfm);
            add(new LabelField(""));
        } else {
            enablePollingPIMField.setChecked(false);
        }

        hfm = new HorizontalFieldManager();
        hfm.add(syncTypeLabel);
        hfm.add(syncTypeField);
        add(new LabelField(""));
        add(new LabelField(""));
        add(hfm);


        if (Customization.getInstance().isRangePastVisible()||
                Customization.getInstance().isRangeFutureVisible()) {
            add(new LabelField(""));
            add(rangeLabel);
        }
        
        if (Customization.getInstance().isRangePastVisible()) {
            add(rangePastField);
        }
        
        if (Customization.getInstance().isRangeFutureVisible()) {
            add(rangeFutureField);
        }

        if (Customization.getInstance().isBlockInvitesFieldVisible()) {
            add(new LabelField(""));
            add(separator3);
            add(new LabelField(""));
            hfm = new HorizontalFieldManager();
            hfm.add(blockInvitesField);
            hfm.add(blockInvitesLabel);
            add(hfm);
        }


        if (Customization.getInstance().isLogEnabledInSettingsScreen()) {
            add(new LabelField(""));
            add(separator4);
            add(new LabelField(""));
            hfm = new HorizontalFieldManager();
            hfm.add(logLevelLabel);
            hfm.add(logLevelField);
            add(hfm);
        }

        // /////////////////////////////////////////////////////////////////////////

        if (Customization.getInstance().syncUriEditable()) {
            serverUriField.setChangeListener(cl);
        }

        usernameField.setChangeListener(cl);
        passwordField.setChangeListener(cl);

        enablePollingPIMField.setChangeListener(cl);
        pollingPIMIntervalField.setChangeListener(cl);

        if (Customization.getInstance().isRangePastVisible()) {
            rangePastField.setChangeListener(cl);
        }
        
        if (Customization.getInstance().isRangeFutureVisible()) {
            rangeFutureField.setChangeListener(cl);
        }

        syncTypeField.setChangeListener(cl);
        logLevelField.setChangeListener(cl);

        if (Customization.getInstance().isBlockInvitesFieldVisible()) {
            blockInvitesField.setChangeListener(cl);
        }
        
    }

    
    public void onDisplay() {
        super.onDisplay();
        loadValues();
        invalidate();
    }
    
    public void onExpose() {
        super.onExposed();
        loadValues();
        invalidate();
    }
    
    private void loadValues() {

        if (Customization.getInstance().syncUriEditable()) {
            serverUriField.setText(Configuration.syncUrl);
        }

        usernameField.setText(Configuration.username);
        passwordField.setText(Configuration.password);

        for(int i=0;i<enabledSourceFieldList.size();++i) {
            CheckboxField enableSyncSourceField = (CheckboxField)
                                             enabledSourceFieldList.elementAt(i);
            ColoredBasicEditField uriField = (ColoredBasicEditField)
                                             sourceUriFieldList.elementAt(i);

            AppSyncSource appSource = (AppSyncSource)sourceMap.elementAt(i);
            int id = appSource.getId();
            boolean enabled = Configuration.isSourceEnabled(id);
            enableSyncSourceField.setChecked(enabled);
            String sourceUri = Configuration.getSourceUri(id);
            uriField.setText(sourceUri);

            if (appSource.getId() == AppSyncSourceManager.SYNC_FLAG_CONTACTS &&
                Customization.getInstance().syncCompanyDirectory()) {

                cdEnableField.setChecked(sourceUri.endsWith("-shared"));
            }
        }

        enablePollingPIMField.setChecked(Configuration.enablePollingPIM);
        pollingPIMIntervalField
                .setSelectedIndex(indexFromMinutes(Configuration.pollingPIMInterval));
        syncTypeField.setSelectedIndex(Configuration.syncType);
        rangePastField.setSelectedIndex(Configuration.rangePast - 1);
        rangeFutureField.setSelectedIndex(Configuration.rangeFuture - 1);
        logLevelField.setSelectedIndex(Configuration.logLevel);
        blockInvitesField.setChecked(Configuration.blockIncomingInvites);
        invalidate();
    }

    /**
     * Adds "Save" menu item to the menu on Configuration screen
     * 
     * @param Menu :
     *            menu to which save menuitem is to be added
     * @param int :
     *            instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance) {
        if (hasChanges) {
            menu.add(saveMenu);
        }
        menu.add(backMenu);
    }

    /**
     * Invoked on user hitting the escape button. It prompts the user with a
     * dialog, if he wants to save/discard/cancel the changes in configuration
     * 
     * @param void.
     * @return void.
     */
    public boolean onClose() {
        if (hasChanges) {
            hasChanges = false;
            final int response = Dialog.ask(Dialog.D_SAVE, StaticDataHelper
                    .getLanguage("dialog_save"));

            if (Dialog.SAVE == response) {
                saveConfiguration();
                loadValues();
            } else if (Dialog.DISCARD == response) {
                loadValues();
            } else if (Dialog.CANCEL == response) {
                return false;
            }
        }

        controller.getScreenDisplayController().hideConfigScreen();

        return true;
    }

    private boolean sourceEnabledWithoutUri(Vector sourceEnabled, Vector sourceUri) {
        if (sourceEnabled.size() != sourceUri.size()) {
            Log.error("Internal error: sources enabled/uri cannot have different size");
            return true;
        }
        for (int i=0;i<sourceEnabled.size();++i) {
            Boolean enabled = (Boolean)sourceEnabled.elementAt(i);
            String  uri     = (String) sourceUri.elementAt(i);
            if (enabled.booleanValue() && "".equals(uri)) {
                return true;
            }
        }
        return false;
    }

    private boolean isAtLeastOneUriNull(Vector sourceUri) {
        for (int i=0;i<sourceUri.size();++i) {
            String uri = (String)sourceUri.elementAt(i);
            if (uri == null) {
                return true;
            }
        }
        return false;
    }

    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen. It saves changes on the configuration into the
     * persistant store
     * 
     */
    public boolean saveConfiguration() {
        Log.debug("ConfigurationScreen.saveConfiguration(): Entered");

        String serverUri = null;
        String username = null;
        String password = null;

        boolean enablePollingPIM = false;
        int pollingPIMInterval = Customization.getInstance().getPollingPimIntervalChoices()[0];
        int rangePast = 1;
        int rangeFuture = 6;
        int syncType = 0;
        int logLevel = 0;
        boolean blockInvites = false;

        if (Customization.getInstance().syncUriEditable()) {
            serverUri = serverUriField.getText().trim();
        } else {
            serverUri = Configuration.syncUrl;
        }

        username = usernameField.getText().trim();
        password = passwordField.getText().trim();

        Vector sourceUri = new Vector();
        Vector sourceEnabled = new Vector();
        for(int i=0;i<sourceUriFieldList.size();++i) {
            ColoredBasicEditField uriField = (ColoredBasicEditField)sourceUriFieldList.elementAt(i);
            String uri = uriField.getText().trim();
            CheckboxField enabledField = (CheckboxField)enabledSourceFieldList.elementAt(i);
            Boolean enabled = new Boolean(enabledField.getChecked());

            // Add this item to the vectors
            sourceUri.addElement(uri);
            sourceEnabled.addElement(enabled);
        }

        enablePollingPIM = enablePollingPIMField.getChecked();
        pollingPIMInterval = minutesFromIndex(pollingPIMIntervalField.getSelectedIndex());

        rangePast = rangePastField.getSelectedValue();
        rangeFuture = rangeFutureField.getSelectedValue();

        syncType = syncTypeField.getSelectedIndex();

        logLevel = logLevelField.getSelectedIndex();

        blockInvites = blockInvitesField.getChecked();

        if (serverUri == null || username == null || password == null ||
            isAtLeastOneUriNull(sourceUri)) {

            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-null"));
            return false;
        } else if (serverUri.equals("") || username.equals("") || password.equals("")) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-required"));
            return false;
        } else if (sourceEnabledWithoutUri(sourceEnabled, sourceUri)) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-incomplete"));
            return false;
        } else if ((serverUri.indexOf("http") != 0) && (serverUri.indexOf("https") != 0)) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-server"));
            return false;
        }

        // Okay, save the configuration
        Configuration.syncUrl = serverUri;
        Configuration.username = username;
        Configuration.password = password;

        for(int i=0;i<sourceUri.size();++i) {
            Boolean enabled = (Boolean)sourceEnabled.elementAt(i);
            String  uri     = (String)sourceUri.elementAt(i);
            AppSyncSource appSource = (AppSyncSource) sourceMap.elementAt(i);
            int id = appSource.getId();
            Configuration.setSourceConfig(id, enabled.booleanValue(), uri);
        }

        Configuration.syncType = syncType;
        Configuration.logLevel = logLevel;

        Configuration.pollingPIMInterval = pollingPIMInterval;
        Configuration.enablePollingPIM = enablePollingPIM;

        Configuration.rangePast = rangePast;
        Configuration.rangeFuture = rangeFuture;

        Configuration.blockIncomingInvites = blockInvites;

        loadValues();

        if (Configuration.save(controller) != Configuration.CONF_OK) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-save"));
            return false;
        }

        // -----------------------------------------------------------------------------

        controller.reapplyConfiguration();

        return true;
    }

    private int minutesFromIndex(int choice) {
        return Customization.getInstance().getPollingPimIntervalChoices()[choice];
    }

    private int indexFromMinutes(int minutes) {
        
        int ret = 0;
        
        int [] min = Customization.getInstance().getPollingPimIntervalChoices();
        
        for (int i =0; i<min.length; i++) {
            if (minutes == min[i]) {
                ret = i;
                break;
            }
        }
        
        return ret;
    }

    private String[] getIntervalChoices(int[] minValues) {
        int size = minValues.length;
        
        String minutes = StaticDataHelper.getLanguage("conf_minutes"); 
        String hour = StaticDataHelper.getLanguage("conf_hour");
        String hours = StaticDataHelper.getLanguage("conf_hours");
        String day = StaticDataHelper.getLanguage("conf_day");
        String days = StaticDataHelper.getLanguage("conf_days");
        
        String[] intervalChoices = new String[size];
        
        for (int i = 0; i < size; i++) {
            int hoursValue = minValues[i]/60;
            
            if (hoursValue == 0) {
                intervalChoices[i] = minValues[i] + " " + minutes;
            } else if (hoursValue == 1) {
                intervalChoices[i] = hoursValue + " " + hour;
            } else if (hoursValue < 24) {
                intervalChoices[i] = hoursValue + " " + hours;
            } else if (hoursValue == 24){
                intervalChoices[i] = hoursValue/24 + " " + day;
            } else if (hoursValue > 24){
                intervalChoices[i] = hoursValue/24 + " " + day;
            }
        } 
        
        return intervalChoices;
    }
}
