/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.blackberry.ui.rim;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.ui.LoginScreen;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;

/**
 *
 */
public class RimLoginScreen extends MainScreen implements LoginScreen {

    private BasicEditField    usernameField  = null;

    private BasicEditField    serverUriField = null;

    private PasswordEditField passwordField  = null;

    private Controller        controller     = null;

    private final MenuItem    saveMenu       = new MenuItem(StaticDataHelper
                                                     .getLanguage("menu_save"), 20000, 10) {

                                                 public void run() {
                                                     if (saveConfiguration()) {
                                                         controller.getScreenDisplayController()
                                                                 .hideLoginScreen();
                                                     }
                                                 }
                                             };

    private final MenuItem    quitMenu       = new MenuItem(StaticDataHelper
                                                     .getLanguage("menu_quit"), 20000, 10) {

                                                 public void run() {
                                                     controller.toBackground();
                                                 }
                                             };

    public RimLoginScreen(Controller controller) {
        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT | Field.FIELD_LEFT | Field.FIELD_TOP);
        
        this.setFont(RimHelper.getDefaultFont());
        
        this.controller = controller;

        LabelField title = new LabelField(StaticDataHelper.getLanguage("credential_title"));
        title.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(title);

        LabelField serverUriLabel = null;
        if (Customization.getInstance().syncUriEditable()) {
            serverUriLabel = new LabelField(StaticDataHelper.getLanguage("conf_server-url"));
            serverUriLabel.setFont(getFont().derive(Font.BOLD));
            serverUriField = new BasicEditField("", Configuration.syncUrl);
        }

        LabelField usernameLabel = new LabelField(StaticDataHelper.getLanguage("conf_user"));
        usernameLabel.setFont(getFont().derive(Font.BOLD));
        usernameField = new BasicEditField("", Configuration.username);

        LabelField passwordLabel = new LabelField(StaticDataHelper.getLanguage("conf_password"));
        passwordLabel.setFont(getFont().derive(Font.BOLD));
        passwordField = new PasswordEditField("", Configuration.password);

        if (Customization.getInstance().syncUriEditable()) {
            add(serverUriLabel);
            add(serverUriField);
        }

        add(new LabelField(""));
        add(usernameLabel);
        add(usernameField);
        add(new LabelField(""));
        add(passwordLabel);
        add(passwordField);

    }

    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen. It saves changes on the configuration into the
     * persistant store
     * 
     */
    public boolean saveConfiguration() {
        Log.debug("ConfigurationScreen.saveConfiguration(): Entered");

        // Load all the default settings and overwrite the parameters edited
        // here
        Configuration.load();

        String serverUri;
        if (Customization.getInstance().syncUriEditable()) {
            serverUri = serverUriField.getText().trim();
        } else {
            serverUri = Customization.getInstance().getServerUriDefault();
        }
        String username = usernameField.getText().trim();
        String password = passwordField.getText().trim();

        if (StringUtil.isNullOrEmpty(username) || StringUtil.isNullOrEmpty(password)
                || StringUtil.isNullOrEmpty(serverUri)) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-user-password"));
            return false;
        }

        // Okay, save the configuration
        Configuration.syncUrl = serverUri;
        Configuration.username = username;
        Configuration.password = password;

        if (Configuration.save(controller) != Configuration.CONF_OK) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-save"));
            return false;
        }

        controller.getScreenDisplayController().showHomeScreen();
        return true;
    }

    /**
     * Adds "Save" menu item to the menu on Configuration screen
     * 
     * @param Menu
     *            : menu to which save menuitem is to be added
     * @param int : instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance) {
        menu.add(saveMenu);
        menu.add(quitMenu);
    }
}
