/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import net.rim.device.api.system.ApplicationDescriptor;
import net.rim.device.api.system.ApplicationManager;
import net.rim.device.api.ui.Font;
import net.rim.device.api.system.DeviceInfo;

import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.AppSyncSourceManager;

/**
 * Supports general using, language dictionary access, logging. Statics data and
 * static methods
 */
public class StaticDataHelper {

    // -----------------------------------------------------------------
    // Constants
    private static int          pollingPIMIntervalDefault      = 0;

    private static boolean      enablePollingPIMDefault        = false;

    private static boolean      encodeDefault                  = false;

    private static int          syncTypeDefault                = 0;

    private static boolean      blockIncomingInvitesDefault    = false;

    private static int          logLevelDefault                = 0;

    private static String       encryptDefault                 = null;

    private static int          rangePastDefault               = 0;

    private static int          rangeFutureDefault             = 0;

    private static String       operatingSystem                = null;

    final private static String DEF_KEY_ENABLE_POLLING_PIM     = "default_enable-polling-pim";

    final private static String DEF_KEY_POLLING_PIM_INTERVAL   = "default_polling-pim-interval";

    final private static String DEF_KEY_ENCODE                 = "default_encode";

    final private static String DEF_KEY_ENCRYPTION             = "default_encryption-type";

    final private static String DEF_KEY_SYNC_TYPE              = "default_sync-type";

    final private static String DEF_KEY_RANGE_PAST             = "default_range-past";

    final private static String DEF_KEY_RANGE_FUTURE           = "default_range-future";

    final private static String DEF_KEY_LOG_LEVEL              = "default_log-level";

    final private static String DEF_KEY_BLOCK_INCOMING_INVITES = "default_block-invites";

    final public static int     FONT_SIZE                      = 14;

    final public static int     FONT_STYLE                     = Font.PLAIN;

    final public static int     TITLE_FONT_SIZE                = 16;

    final public static int     TITLE_FONT_STYLE               = Font.BOLD;

    final public  static int    TITLE_PADDING                   = 5;

    final public static int     SYNC_TYPE_TWO_WAY              = 0;

    final public static int     SYNC_TYPE_ONE_WAY_TO_SERVER    = 1;

    final public static int     SYNC_TYPE_ONE_WAY_TO_CLIENT    = 2;

    final public static int     SYNC_TYPE_REFRESH_TO_SERVER    = 3;

    final public static int     SYNC_TYPE_REFRESH_TO_CLIENT    = 4;

    // ----------------------------------------------------------------- Private
    // data
    final public static String  XML_LANGUAGE_FILE              = "/xml/language.xml";

    private static Hashtable    languageHash                   = new Hashtable();

    // ----------------------------------------------------------------- Public
    // methods

    public static String getOS() {
        return operatingSystem;
    }

    public static String[] getOSVersion()
    {
        Log.debug("[getOSVersion]");
        String os = getOS();
        Log.trace("OS version string is: " + os);
        if (os == null)
        {
            return new String[0];
        }

        String[] tokens = StringUtil.split(os, ".");
        for(int i=0;i<tokens.length;++i) {
            Log.trace("Ver["+i+"]="+tokens[i]);
        }
        return tokens;
    }


    /**
     * Returns true iff the device has a dedicated key to open the menu. The
     * implementation is based on the device model and it works for the model
     * currently supported by our APIs (>= 4.2.1)
     */
    public static boolean hasDedicatedMenuKey() {
        String deviceName = DeviceInfo.getDeviceName();
        if (deviceName.startsWith("7130")  ||
            deviceName.startsWith("8700")  ||
            deviceName.startsWith("8703")  ||
            deviceName.startsWith("8705")  ||
            deviceName.startsWith("8707")) {

            return false;
        } else {
            return true;
        }
    }
    
    public static int getSyncTypeDefault() {
        return syncTypeDefault;
    }

    public static boolean getBlockIncomingInvitesDefault() {
        return blockIncomingInvitesDefault;
    }

    public static int getLogLevelDefault() {
        return logLevelDefault;
    }

    public static boolean isEncode() {
        return encodeDefault;
    }

    public static boolean getEnablePollingPIMDefault() {
        return enablePollingPIMDefault;
    }

    public static int getPollingPIMIntervalDefault() {
        return pollingPIMIntervalDefault;
    }

    public static int getRangePastDefault() {
        return rangePastDefault;
    }

    public static int getRangeFutureDefault() {
        return rangeFutureDefault;
    }

    public static String getEncryptDefault() {
        return encryptDefault;
    }

    /**
     * Make a String by value of <i>tag</i>.
     * 
     * @param key
     *            key to find
     * @return key value
     */
    public static String getLanguage(String key) {
        Object valueFromHash = null;
        valueFromHash = StaticDataHelper.languageHash.get(key);
        if (valueFromHash != null) {
            return (String) valueFromHash;
        }

        // This should never happen
        System.out.println("Language string missing for key: " + key);
        return "";
    }

    public static void init(Class c, String languagefile) throws Exception {
        StaticDataHelper.loadVersion();
        StaticDataHelper.loadLanguage(c, languagefile);
        StaticDataHelper.loadDefaultValues();
        StaticDataHelper.setOS();
    }

    /**
     * Load default properties form jad file
     */
    private static void loadDefaultValues() throws Exception {
        String enablePollingPIM = null;
        String pollingPIMInterval = null;
        String rangePast = null;
        String rangeFuture = null;
        String encode = null;
        String syncType = null;
        String logLevel = null;
        String blockIncomingInvites = null;

        enablePollingPIM = getLanguage(DEF_KEY_ENABLE_POLLING_PIM);
        pollingPIMInterval = getLanguage(DEF_KEY_POLLING_PIM_INTERVAL);
        rangePast = getLanguage(DEF_KEY_RANGE_PAST);
        rangeFuture = getLanguage(DEF_KEY_RANGE_FUTURE);
        encode = getLanguage(DEF_KEY_ENCODE);
        syncType = getLanguage(DEF_KEY_SYNC_TYPE);
        logLevel = getLanguage(DEF_KEY_LOG_LEVEL);
        encryptDefault = getLanguage(DEF_KEY_ENCRYPTION);
        blockIncomingInvites = getLanguage(DEF_KEY_BLOCK_INCOMING_INVITES);

        if (blockIncomingInvites == null) {
            blockIncomingInvites = "";
        }

        if (syncType == null || !(syncType.length() > 0)) {
            syncTypeDefault = 0;
        } else {
            try {
                syncTypeDefault = Integer.parseInt(syncType);
            } catch (final NumberFormatException e) {
                final String msg = "StaticDataHelper.loadDefaultValue(): invalid sync type default";
                Log.error(msg);
                throw new Exception(msg);
            }
        }

        if (logLevel == null || !(logLevel.length() > 0)) {
            logLevelDefault = Log.TRACE;
        } else {
            try {
                logLevelDefault = Integer.parseInt(logLevel);
            } catch (final NumberFormatException e) {
                final String msg = "StaticDataHelper.loadDefaultValue(): invalid log level default";
                Log.error(msg);
                throw new Exception(msg);
            }
        }

        if (pollingPIMInterval == null || !(pollingPIMInterval.length() > 0)) {
            pollingPIMIntervalDefault = 0;
        } else {
            try {
                pollingPIMIntervalDefault = Integer.parseInt(pollingPIMInterval);
            } catch (final NumberFormatException e) {
                final String msg = "StaticDataHelper.loadDefaultValue(): invalid default pim polling interval";
                Log.error(msg);
                throw new Exception(msg);
            }
        }

        if (rangePast == null || !(rangePast.length() > 0)) {
            rangePastDefault = 1;
        } else {
            try {
                rangePastDefault = Integer.parseInt(rangePast);
            } catch (NumberFormatException e) {
                final String msg = "StaticDataHelper.loadDefaultValue(): invalid default range past default";
                Log.error(msg);
                throw new Exception(msg);
            }
        }

        if (rangeFuture == null || !(rangeFuture.length() > 0)) {
            rangeFutureDefault = 1;
        } else {
            try {
                rangeFutureDefault = Integer.parseInt(rangeFuture);
            } catch (NumberFormatException e) {
                final String msg = "StaticDataHelper.loadDefaultValue(): invalid default range future default";
                Log.error(msg);
                throw new Exception(msg);
            }
        }

        if (encryptDefault == null) {
            encryptDefault = "none";
        }
        if ("true".equals(enablePollingPIM)) {
            enablePollingPIMDefault = true;
        }
        if ("true".equals(encode)) {
            encodeDefault = true;
        }
        if ("true".equals(blockIncomingInvites)) {
            blockIncomingInvitesDefault = true;
        }
    }

    /**
     * Must be called on initialization
     */
    public static void loadVersion() {
        String version = BBVersion.VERSION;
        Constants.VER = version;
        Log.info("Version: " + version);
    }

    /**
     * load xml language file
     */
    private static void loadLanguage(Class c, String languagefile) throws Exception {
        Log.info("Reading language file [" + languagefile + "]");
        try {
            String language = read(c.getResourceAsStream(languagefile));
            SimpleXMLParser parser = new SimpleXMLParser();
            StaticDataHelper.languageHash = parser.parseData(
                    Customization.getInstance().getXMLLanguage(), language);
            language = null;
            parser = null;
            Log.info("Reading language file [" + languagefile + "] successfully");
        } catch (final Throwable e) {
            final String msg = "Error: load " + languagefile + " file: " + e.getMessage();
            Log.error(msg);
            throw new Exception("Error reading language definition file.");
        }
    }

    private static void setOS() {
        final ApplicationDescriptor[] appDes = ApplicationManager.getApplicationManager()
                .getVisibleApplications();
        String version = "";
        for (int i = appDes.length - 1; i >= 0; --i) {
            if (appDes[i].getModuleName().equals("net_rim_bb_ribbon_app")) {
                version = appDes[i].getVersion();
                break;
            }
        }
        operatingSystem = version;
    }

    /**
     * Reads the content of the given input stream.
     * 
     * @param is
     *            the input stream
     */
    public static String read(InputStream is) throws IOException {
        final StringBuffer sb = new StringBuffer();
        try {
            final byte[] buf = new byte[1024];
            int nbyte = -1;
            while ((nbyte = is.read(buf)) >= 0) {
                sb.append(new String(buf, 0, nbyte));
            }
        } finally {
            is.close();
        }
        return sb.toString();
    }
}
