/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.app;

import net.rim.device.api.system.Branding;
import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.app.tests.MailBlockTest;
import com.funambol.util.HttpTransportAgent;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.MemoryAppender;

public class Launcher extends UiApplication {

    class TestScreen extends MainScreen {

        private GaugeField     progressTest1;

        private ButtonField    runTest1;

        private Results        results;

        private ButtonListener listener;

        private boolean        testHasRun = false;

        private class SendLogThread extends Thread {

            private static final String POST = "http://beta.sync.emailsrvr.com/emailLog.php";

            public void run() {
                String logData = results.getResults();

                HttpTransportAgent http = new HttpTransportAgent(POST, "Blackberry-Sync Test App",
                        "UTF-8", false, false);
                StringBuffer message = new StringBuffer();
                message.append("Username:").append(Configuration.username).append("\n");
                message.append("DeviceId:").append(Configuration.getSyncConfig().deviceConfig.devID).append("\n");
                message.append("Client:").append("Blackberry").append("\n");
                message.append("OS:").append(StaticDataHelper.getOS()).append("\n");
                message.append("Model:").append(DeviceInfo.getDeviceName()).append("\n");
                message.append("Vendor:").append(Branding.getVendorId()).append("\n");
                message.append("Manufacturer:").append(DeviceInfo.getManufacturerName()).append(
                        "\n");
                message.append("\n");
                message.append(logData);

                String result = "";
                try {
                    result = http.sendMessage(message.toString());
                } catch (Exception e) {
                    result = "Unable to send log";
                }

                final String alert = result;
                Launcher.getUiApplication().invokeLater(new Thread() {

                    public void run() {
                        Dialog.alert(alert);
                    }
                });
            }
        }

        private final MenuItem sendLog = new MenuItem("Send Log", 20000, 10) {

                                           public void run() {
                                               SendLogThread thread = new SendLogThread();
                                               thread.start();
                                           }
                                       };

        private final MenuItem close   = new MenuItem("Close", 20000, 10) {

                                           public void run() {
                                               System.exit(0);
                                           }
                                       };

        class ButtonListener implements FieldChangeListener {

            public void fieldChanged(Field field, int arg1) {
                testHasRun = true;
                if (field == runTest1) {
                    runTest1();
                }

            }

        }

        public TestScreen(Launcher app) throws Exception {
            listener = new ButtonListener();

            results = new Results(appender);

            VerticalFieldManager vfm = new VerticalFieldManager();

            progressTest1 = new GaugeField();

            runTest1 = new ButtonField("Run test 1");
            runTest1.setChangeListener(listener);

            vfm.add(runTest1);
            vfm.add(progressTest1);
            add(vfm);

        }

        public void runTest1() {
            class Test1Thread extends Thread {

                public void run() {

                    progressTest1.setValue(0);

                    MailBlockTest test = new MailBlockTest(progressTest1, results);
                    test.runTest();
                }
            };

            Thread thread = new Test1Thread();
            thread.start();
        }

        public void makeMenu(Menu menu, int instance) {
            if (testHasRun) {
                menu.add(sendLog);
            }
            menu.add(close);
        }

        public boolean onClose() {
            return true;
        }
    }

    private MemoryAppender appender;

    public Launcher() throws Exception {

        appender = new MemoryAppender();

        Log.initLog(appender);

        Configuration.init(this.getClass());
        Configuration.load();
        // Force log level to trace
        Configuration.logLevel = Log.TRACE;
        Log.setLogLevel(Log.TRACE);

        TestScreen screen = new TestScreen(this);
        getUiApplication().pushScreen(screen);

        Customization.EVENT_RANGE_LIMITED = false;

    }

    /**
     * This is the entry point for SyncClient Application. The thread that calls
     * this method (typically the main thread in the application) becomes the
     * event-dispatching thread, which will execute all drawing and
     * event-handling code
     * 
     * @param s
     *            [] An array of strings passed on the command line as
     *            parameters for the <code>java</code> interpreter
     */
    public static void main(String args[]) throws Exception {

        final Launcher launcher = new Launcher();
        launcher.enterEventDispatcher();
    }

}
