/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.app.tests;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;

import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.ui.component.GaugeField;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.app.Results;
import com.funambol.blackberry.helper.EventHelper;
import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.blackberry.helper.HelperFactory;
import com.funambol.util.Log;
import com.funambol.blackberry.source.AdditionalItemsAttributes;

public class MailBlockTest extends Test {

    public MailBlockTest(GaugeField field, Results results) {
        super("MailBlockTest", results, field);
    }

    public void runTest() {
        
        testBegun();

        BlackBerryPIMList list;
        try {
            list = (BlackBerryPIMList) PIM.getInstance()
                    .openPIMList(PIM.EVENT_LIST, PIM.READ_WRITE);
        } catch (PIMException e) {
            Log.error("Error getting pim list, " + e.getMessage());
            testEnded(false);
            return;
        }
        EventMailListener listener = new EventMailListener("");
        EventHelper helper = (EventHelper) HelperFactory.getEventHelper(list, listener, -24, +24);

        listener.enableBlockOutgoing();

        Log.trace("Blocking fake outgoing item");

        listener.blockOutgoingEvent("Test email", EventMailListener.NEW);

        String itemData = "<?xml version='1.0' encoding='UTF-8'?><appointment><AllDayEvent>0</AllDayEvent><Body></Body><Location></Location><Subject>Attendees</Subject><End>20091020T143000Z</End><ReminderSet>0</ReminderSet><Start>20091020T140000Z</Start><Sensitivity>0</Sensitivity><IsRecurring>0</IsRecurring><Exceptions><IncludeDate></IncludeDate></Exceptions><Attendees><Attendee><AttendeeName>\"mail2\"</AttendeeName><AttendeeEmail>mail2@geminitest.com</AttendeeEmail><AttendeeStatus>4</AttendeeStatus><AttendeeType>0</AttendeeType></Attendee><Attendee><AttendeeName>\"mail3\"</AttendeeName><AttendeeEmail>mail3@geminitest.com</AttendeeEmail><AttendeeStatus>4</AttendeeStatus><AttendeeType>0</AttendeeType></Attendee><Attendee><AttendeeName>\"mail4\"</AttendeeName><AttendeeEmail>mail4@geminitest.com</AttendeeEmail><AttendeeStatus>4</AttendeeStatus><AttendeeType>0</AttendeeType></Attendee></Attendees></appointment>";
        String subject = "Attendees";

        boolean success = false;

        setProgress(25);

        Log.trace("Creating a blank PIMItem.");
        PIMItem pitem = null;
        String key = null;
        try {
            pitem = helper.createItem();
        } catch (PIMException e) {
            String error = helper.handlePIMException(e);
            Log.error("Error saving pim item, " + error);
            pitem = null;
        }

        try {
            if (pitem != null) {

                Log.trace("Calling fillItem.");
                AdditionalItemsAttributes additionals = null;
                helper.fillItem(pitem, itemData,additionals);

                Log.trace("Saving item.");
                try {
                    helper.saveNewPIMItem(pitem);
                } catch (PIMException e) {
                    String error = helper.handlePIMException(e);
                    Log.error("Error saving pim item, " + error);
                }

                setProgress(50);

                Log.trace("Trying to get UID of PIMItem.");
                key = helper.getUID(pitem);
            }
        } catch (PIMException e) {
            String error = helper.handlePIMException(e);
            Log.error("Error running test, " + error);
        }

        try {
            Log.trace("Waiting 5 seconds");
            Thread.sleep(5000);
        } catch (InterruptedException e) {
        }
        Log.trace("Wait over");

        setProgress(75);

        String message = listener.getLastSentSubject();

        Log.trace("Last sent subject:" + message);

        success = listener.getLastSentSubject() != null
                && listener.getLastSentSubject().equals(subject);

        if (key != null) {
            Log.trace("Removing created item - " + key);
            try {
                pitem = helper.findItem(key);
                if (pitem == null || !helper.deleteItem(pitem)) {
                    Log.error("Could not delete item!");
                }
            } catch (PIMException e) {
                String error = helper.handlePIMException(e);
                Log.error("Error deleting added item!" + error);
            }
        }

        setProgress(100);
        testEnded(success);
    }
}
